\\ Copyright 2016, 2017, 2019 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file.  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.

default(recover,0);
read("dragon-defines.gp");

check_count = 0;
check_exact(got,want,name="") =
{
  check_count++;
  if(got!= want,
     print("wrong "name);
     print("got  "got);
     print("want "want);
     quit(1),
     print("ok "check_count" "name));
}

\\-----------------------------------------------------------------------------

\\ number of odd n in level 2^k
OddCount(k) = if(k<1,0, 2^(k-1));
check_exact(OddCount(1), 1);
check_exact(OddCount(2), 2);

\\--------------
Ocount(k) = Dodd(k)*2 + OddCount(k);
print("Ocount ", vector(10,k,k--; Ocount(k)));

vOcount_new=[0,1,1,2,6,12,24,52,108,220,435]; \\ new O
vOcount=[0,1,2,4,10,22,46,98,206,426,878,1802]; \\ O
check_exact(vector(#vOcount,i,my(k=i-1); Ocount(k)), vOcount);

gOcount(x) = x*gTwoPow(x) + 2*gDodd(x);
check_exact(gf_terms(gOcount(x),100), vector(100,k,k--; Ocount(k)));

\\ vector(#vO,i,my(k=i-1); vO[i]-OddCount(k))/2
\\ vector(#vO,i,my(k=i-1); Dodd(k))

\\ recurrence_guess(gf_terms(gOcount(x),100));
Ocount_formula(k) = if(k==0,0, 2^k - LQ(k));
check_exact(vector(100,k,k--; Ocount_formula(k)), \
            vector(100,k,k--; Ocount(k)));

\\--------------
Ecount(k) = 4*Deven(k);
\\       0 1 2
vEcount=[0,0,0,0,0,4,16,44,112,268,608,1340]; \\ E
check_exact(vector(#vEcount,i,my(k=i-1); Ecount(k)), vEcount);

gEcount(x) = 4*gDeven(x);
check_exact(gf_terms(gEcount(x),100), vector(100,k,k--; Ecount(k)));

\\ recurrence_guess(gf_terms(gEcount(x),100));
Ecount_formula(k) = if(k==0,0, 2^k + 2 - 2*RQ(k));
check_exact(vector(100,k,k--; Ecount_formula(k)), \
            vector(100,k,k--; Ecount(k)));

\\--------------
vBcount=[0,0,0,1,2,5,16,43,106,253,580,1287];  \\ B
vTcount=[0,0,1,2,5,12,27,62,141,312,679,1458]; \\ T
\\ vector(min(#vT,#vB),i,my(k=i-1); if(i<2,0, vB[i-1]+vO[i-1]))

vtotal=[0,1,3,7,17,43,105,247,565,1259,2745,5887];

\\ O(k) = 2^(k-1) + 2*Dodd(k)
\\ B(k) = E(k-1) + T(k-1)
\\ T(k) = O(k-1) + B(k-1)
\\      = O(k-1) + E(k-2) + T(k-2)
\\ gT(x) = x*gO(x) + x^2*gE(x) + x^2*gT(x)

gTcount(x) = 1/(1-x^2) * (x*gOcount(x) + x^2*gEcount(x));
check_exact(gf_terms(gTcount(x),#vTcount), vTcount);

check_exact((2 + x + 2*x^2)/(1 - x - 2*x^3), gBQ(x));
gBcount(x) = 1/(1-x^2) * (x^2*gOcount(x) + x*gEcount(x));
check_exact(gf_terms(gBcount(x),#vBcount), vBcount, "gBcount vs samples");
gtotal(x) = gEcount(x) + gBcount(x) + gTcount(x) + gOcount(x);
check_exact(gf_terms(gtotal(x),#vtotal), vtotal);

Tcount(k) = if(k==0,0, 2^k + k - BQ(k));
check_exact(vector(100,k,k--; Tcount(k)), gf_terms(gTcount(x),100));


\\-------------
print("Bcount");
\\ recurrence_guess(gf_terms(gBcount(x),100));
{
  check_exact(  1/(1 - x)^2
              + 1/(1 - 2*x)
              + (-2 - 2*x - 3*x^2)/(1 - x - 2*x^3),
              gBcount(x));
}
{
  check_exact(  1/(1 - x)^2
              + gTwoPow(x)
              - gBQ(x) - 2*x^2*gLQ(x) - x,
              gBcount(x));
}
check_exact(gf_terms(1/(1 - x)^2,100), vector(100,k,k--; k+1));
check_exact(gf_terms(1/(1 - 2*x),100), vector(100,k,k--; 2^k));

pol_ascending_print(pol_partial_fractions(gBQ(x)));
pol_ascending_print(pol_partial_fractions(2*x^2*gLQ(x)));
pol_ascending_print(pol_partial_fractions(- gBQ(x) - 2*x^2*gLQ(x)));
check_exact(- gBQ(x) - 2*x^2*gLQ(x) - x, \
            (-2 - 2*x - 3*x^2)/(1 - x - 2*x^3));

\\ (2 + 2*x + 3*x^2)/(1 - x - 2*x^3)
\\ [2, 1, 2] BQ + [0, 1, 1]
Bcount(k) = if(k<2,0, 2^k + k + 1 - BQ(k) - 2*LQ(k-2));
\\ recurrence_guess(vector(100,k,k--; Bcount(k)));
check_exact(vector(#vBcount,k,k--; Bcount(k)), vBcount, "Bcount vs samples");
check_exact(vector(100,k,k--; Bcount(k)), gf_terms(gBcount(x),100), "Bcount vs gBcount");

\\-------------
print("Tcount");
recurrence_guess(gf_terms(gTcount(x),100));
Tcount(k) = if(k==0,0, 2^k + k - BQ(k));
check_exact(vector(100,k,k--; Tcount(k)), gf_terms(gTcount(x),100), "Tcount vs gTcount");

\\-------------
print("total");
recurrence_guess(gf_terms(gtotal(x),100));
{
  check_exact(  1/(1 - x)
              + 2/(1 - x)^2
              + 4/(1 - 2*x)
              - (7 + 5*x + 8*x^2)/(1 - x - 2*x^3),
              gtotal(x));
}
pol_ascending_print(pol_partial_fractions(4*gBQ(x)-2*x*gdJA(x)));
check_exact((8 + 4*x + 8*x^2)/(1 - x - 2*x^3), 4*gBQ(x));
\\ 1,-1
check_exact((7 + 5*x + 8*x^2)/(1 - x - 2*x^3), 4*gBQ(x)-2*x*gdJA(x) - 1);

total(k) = if(k==0,0, 4*2^k + 2*k + 3 - 4*BQ(k) + 2*dJA(k-1));
wcheck_exact(vector(100,k,k--; total(k)), gf_terms(gtotal(x),100), "total vs gtotal");

total_by_parts(k) = Ecount_formula(k) + Bcount(k) + Tcount(k) + Ocount_formula(k);
check_exact(vector(100,k,k--; total_by_parts(k)), gf_terms(gtotal(x),100), "total_by_parts");

total_by_parts(k) =
{
  if(k==0,0, k==1,1, 
       2^k     + 2 - 2*RQ(k)
     + 2^k + k + 1 - BQ(k) - 2*LQ(k-2)
     + 2^k + k     - BQ(k) 
     + 2^k                 - LQ(k)  );
}
check_exact(vector(100,k,k--; total_by_parts(k)), gf_terms(gtotal(x),100), "total_by_parts");

E_plus_O(k) = 4*Deven(k) + Dodd(k)*2 + OddCount(k);
check_exact(vector(100,k,k--; E_plus_O(k)), vector(100,k,k--; Ecount(k)+Ocount(k)));
\\
E_plus_O(k) = if(k==0,0, 4*Deven(k) + 2*Dodd(k) + 2^(k-1) );
check_exact(vector(100,k,k--; E_plus_O(k)), vector(100,k,k--; Ecount(k)+Ocount(k)));
\\
E_plus_O(k) = if(k==0,0, 4*D(k-1) + 2*(D(k)-D(k-1)) + 2^(k-1) );
check_exact(vector(100,k,k--; E_plus_O(k)), vector(100,k,k--; Ecount(k)+Ocount(k)));
\\
E_plus_O(k) = if(k==0,0, 2^(k-1) + 2*D(k-1) + 2*D(k) );
check_exact(vector(100,k,k--; E_plus_O(k)), vector(100,k,k--; Ecount(k)+Ocount(k)));

total_by_recurrence(k) = if(k==0,0, Ecount(k)+Ocount(k) + total(k-1) );
check_exact(vector(100,k,k--; total_by_recurrence(k)), gf_terms(gtotal(x),100));
\\
total_by_recurrence(k) = if(k==0,0,  2^(k-1) + 2*D(k) + 2*D(k-1) + total(k-1) );
check_exact(vector(100,k,k--; total_by_recurrence(k)), gf_terms(gtotal(x),100));
\\
total_by_sum(k) = sum(j=1,k, 2^(k-j) + 2*D(k-j+1) + 2*D(k-j) );
check_exact(vector(100,k,k--; total_by_sum(k)), gf_terms(gtotal(x),100));
\\
total_by_sum(k) = 2^k-1 + sum(j=1,k, 2*D(k-j+1) + 2*D(k-j) );
check_exact(vector(100,k,k--; total_by_sum(k)), gf_terms(gtotal(x),100));
\\
total_by_sum(k) = 2^k-1 + 2*D(k) + 4*sum(j=0,k-1, D(j) );
check_exact(vector(100,k,k--; total_by_sum(k)), gf_terms(gtotal(x),100));

Brad_Klee = [0, 1, 3, 7, 17, 43, 105, 247, 565 ]
gf_terms(gtotal(x),10)

\\---------------------
print("D");

D(10)
1/x^10 * x^4/(1-x)/(1-2*x)/(1 - x - 2*x^3) + O(x)
pollead(1/x^10 * x^4/(1-x)/(1-2*x)/(1 - x - 2*x^3) + O(x))
D_by_gf(k) = polcoeff(1/x^k * x^4/(1-x)/(1-2*x)/(1 - x - 2*x^3) + O(x), 0);
D_by_gf(5)

check_exact(D_by_gf(4), 1);
check_exact(vector(100,k,k--; D_by_gf(k)), vector(100,k,k--; D(k)));

\\ A003230 offset 0
\\ D(k) = A003230(k+4), so D(4)==1


D(k) = polcoeff(x^(4-k)/(1-x)/(1-2*x)/(1 - x - 2*x^3) + O(x), 0);
TripleCount(k) = 2^k-1 + 2*D(k) + 4*sum(j=0,k-1, D(j));
\\
Brad_Klee = [0, 1, 3, 7, 17, 43, 105, 247, 565 ]
vector(#Brad_Klee,k,k--; TripleCount(k)) == Brad_Klee
vector(#Brad_Klee,k,k--; TripleCount(k))
check_exact(vector(#Brad_Klee,k,k--; TripleCount(k)), Brad_Klee);
check_exact(vector(#vtotal,k,k--; TripleCount(k)), vtotal, "TripleCount() vs vtotal");

