#!/usr/bin/perl -w

# Copyright 2014, 2015, 2016, 2017, 2018, 2019, 2020 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.

use 5.010;
use strict;
use FLAT;

use FindBin;
use lib::abs "$FindBin::Bin/../tools";
use MyFLAT;

# uncomment this to run the ### lines
use Smart::Comments;


sub A203175_is_permitted {
  my ($aaL,$aaR, $aL,$aR, $L,$R) = @_;

  # every 1 immediately preceded by 0 left or above
  if ($L==1) { $aL==0 || return 0; }
  if ($R==1) { $L==0 || $aR==0 || return 0 }

  # no 0 immediately preceded by a 0
  if ($L==0 && $aL==0           ) { return 0; }
  if ($R==0 && ($L==0 || $aR==0)) { return 0; }

  # every 2 immediately preceded by 0 1 to the left or above
  if ($L==2) { ($aaL==0 && $aL==1) || return 0; }
  if ($R==2) { ($aaR==0 && $aR==1) || return 0; }
  return 1;
}

{
  # state machine of final pairs
  my $f = FLAT::DFA->new;
  my %str_to_state;
  my $str_to_state = sub {
    my ($str) = @_;
    return ($str_to_state{$str} //= $f->add_states(1));
  };
  my @pending = ('0,1,1,0');
  while (@pending) {
    my $str = shift @pending;
    my ($aaL,$aaR, $aL,$aR) = split /,/, $str;
    my $from = $str_to_state->($str);
    foreach my $L (0..2) {
      foreach my $R (0..2) {
        if (A203175_is_permitted($aaL,$aaR, $aL,$aR, $L,$R)) {
          my $new_str = "$aL,$aR,$L,$R";
          if (! exists $str_to_state{$new_str}) {
            push @pending, $new_str;
          }
          my $to = $str_to_state->($new_str);
          $f->add_transition($from, $to, "$L$R");
        }
      }
    }
  }
  $f->set_starting(0);
  $f->set_accepting(0);
  $f->MyFLAT::view;
  $f = $f->MyFLAT::minimize;
  $f->MyFLAT::view;
  exit 0;
}
