\\ Copyright 2015, 2016, 2017 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file.  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.


default(recover,0);
read("dragon-defines.gp");

\\-----------------------------------------------------------------------------

check_count = 0;
check_exact(got,want,name="") =
{
  check_count++;
  if(got!= want,
     print("wrong "name);
     print("got  "got);
     print("want "want);
     quit(1),
     print("ok "check_count" "name));
}

\\-----------------------------------------------------------------------------
\\ R/B = 1/r
\\ L/B = 2/r^2
\\ L/R = (L/B) * (B/R)  = 2/r^2 * r  = 2/r

\\ R/B + L/B = 1

\\-----------------------------------------------------------------------------

inertia_scale_area(M,f) = M*f^4;
check_exact(inertia_scale_area(geom_inertia_of_rectangle(1,1), 1/sqrt2), \
            geom_inertia_of_rectangle(1/sqrt2,1/sqrt2));
check_exact(1/4*geom_inertia_of_rectangle(1,1), \
            geom_inertia_of_rectangle(1/sqrt2,1/sqrt2));

check_exact(1/16*Ipoint(1), Ipoint(1/4));
Ipoint(1/4)
Ipoint(1/2 - 1/2*GRf)

\\ masses
\\ r^2 = r + 2/r
\\ r^2 - r = 2/r
\\ 1/r = 1/2*(r^2 - r)
\\ 1/r^2 = 1/2*(r - 1)
\\ 1/r^3 = 1/2*(1 - 1/r) = 1/2*(1 - 1/2*(r^2 - r)) = 1/4*(-2*r^2 
Bm = 1
Rm = 1/'r
Rm = 1/2*('r^2 - 'r)
Lm = 2/'r^3
Lm = ('r^2 - 'r) / 'r^2
Lm = 1 - 1/'r
Lm = 1 - 1/2*('r^2 - 'r)
Lm = -1/2*'r^2 + 1/2*'r + 1
subst(Lm,'r,r)+subst(Rm,'r,r)

Rm1 = 1/2*Rm;
Lm1 = 1/2*Lm;
Rm2 = 1/4*Rm;
Lm2 = 1/4*Lm;
Rm3 = 1/8*Rm;
Lm3 = 1/8*Lm;

IRQ = geom_inertia_of_Ix_Iy_Ixy(ix,iy,ixy);

print("GRf ",subst(GRf_symbolic,'r,r));
print("GRf first half");
1/sqrt2*conj(w8_inexact)*GRf
1/sqrt(2)*conj(w8_times_sqrt2)/sqrt(2)*GRf_symbolic
1/2*conj(w8_times_sqrt2)*GRf_symbolic
1/2*(1-I)*GRf_symbolic
GRf_rotated = 1/2*(1-I)*GRf_symbolic
print("GRf_rotated ",subst(GRf_rotated,'r,r));

\\     IRQ      IRQ
\\  *  >>   *<------* 
\\  |       ^         
\\  |       |
\\  v       |
\\  *<------*

M_to_Gr1 = -1/2 + 1/2*GRf_symbolic;
print("M_to_Gr1 ",subst(M_to_Gr1,'r,r));

print("ILQ");
{
  ILQ =
  + 1/16*IRQ + Rm2 *Ipoint(M_to_Gr1)
  + 1/16*IRQ + Rm2 *Ipoint(-M_to_Gr1);
}
reduce_r(poly) =
{
  while(1,
        my(after=substpol(ILQ, 'r^3, 'r^2+2)); \
        if(after==poly,break());
        poly=after);
  poly;
}
ILQ = reduce_r(ILQ)

\\  *               *
\\    \   G       /    IRQ(k) = R45^-1 * IRQ * R45
\\  IRQ \      / ILQ          + 1/2*Rm*Ipoint(-1/2 + 1/sqrt2*conj(w8)*GRf)
\\        v  v                + R45^-3 * ILQ * R45^3
\\         *                  + 1/2*Lm*Ipoint(1/4-1/4*I)
\\

\\ IRQ = R45^-1 * IRQ * R45   +  R45^-3 * ILQ * R45^3
\\  

G_to_start = -GRf_symbolic;
print("G_to_start ",subst(G_to_start,'r,r));

G_to_start = -GRf_symbolic;
G_to_Gr = G_to_start + GRf_rotated;
print("G_to_Gr ",subst(G_to_Gr,'r,r));

G_to_Gl = G_to_start  + 3/4 - 1/4*I;
print("G_to_Gl ",subst(G_to_Gl,'r,r));

{
  zero =
  IRQ 
  - (
     + R45^-1 * 1/4*IRQ * R45
     + Rm1 * Ipoint(G_to_Gr)

     + R45^-3 * 1/4*ILQ * R45^3
     + Lm1 * Ipoint(G_to_Gl)
  );
  zero = reduce_r(zero)
}
\\              / \
\\  *          /    *
\\    \   G    \  v  IRQ
\\  IRQ \      ^ 
\\        v  / IRQ       
\\         *          
\\

G_to_GrA = G_to_start + 1/2-1/2*I + 1/2* I*GRf_rotated
G_to_GrB = G_to_start + 1         - 1/2* I*GRf_rotated
{
  zero_3 = IRQ 
          - (
             + R45^-1 * 1/4*IRQ * R45             \\ axis +45
             + Rm1 * Ipoint(G_to_Gr)
             
             + 2 * R45 * 1/64*IRQ * R45^-1        \\ axis -45
             + Rm3 * Ipoint(G_to_GrA)
             + Rm3 * Ipoint(G_to_GrB)
            );
  zero_3 = reduce_r(zero_3);
  check_exact(zero_3, zero);
}

\\ r^3 - r^2 - 2 = 0
\\ r^3 = r^2 + 2
while(1, my(after=substpol(zero, 'r^3, 'r^2+2)); \
         if(after==zero,break()); zero=after);
zero
check_exact(zero[1,2], zero[2,1]);

z1 = zero[1,1]
z1x = polcoeff(z1,1,'ix)
z1y = polcoeff(z1,1,'iy)
z1xy = polcoeff(z1,1,'ixy)
z1c = z1 - z1x*'ix - z1y*'iy - z1xy*'ixy

z2 = zero[2,2]
z2x = polcoeff(z2,1,'ix)
z2y = polcoeff(z2,1,'iy)
z2xy = polcoeff(z2,1,'ixy)
z2c = z2 - z2x*'ix - z2y*'iy - z2xy*'ixy

z3 = zero[1,2]
z3x = polcoeff(z3,1,'ix)
z3y = polcoeff(z3,1,'iy)
z3xy = polcoeff(z3,1,'ixy)
z3c = z3 - z3x*'ix - z3y*'iy - z3xy*'ixy

col=matsolve([z1x,z1y,z1xy; z2x,z2y,z2xy; z3x,z3y,z3xy],[z1c;z2c;z3c])
ix = col[1,1]
iy = col[2,1]
ixy = col[3,1]
subst(subst(subst(zero[1,1], 'ix,ix), 'iy,iy), 'ixy,ixy)
subst(subst(subst(zero[2,2], 'ix,ix), 'iy,iy), 'ixy,ixy)
subst(subst(subst(zero[1,2], 'ix,ix), 'iy,iy), 'ixy,ixy)

print("------------------");
print("IRQ");
subst(ix,'r,r)
subst(iy,'r,r)
subst(ixy,'r,r)

IRQ_in_r = geom_inertia_of_Ix_Iy_Ixy(ix,iy,ixy)
IRQ_in_r /= Rm
IRQ_inexact = simplify(subst(IRQ_in_r,'r,r))
print("IRQ maximum ",geom_inertia_principal_xyangle(IRQ_inexact) * 180/Pi);


print("------------------");
print("ILQ");
ILQ_in_r = subst(subst(subst(ILQ, 'ix,ix), 'iy,iy), 'ixy,ixy)
ILQ_in_r /= Lm
ILQx  = subst(geom_inertia_Ix(ILQ_in_r),'r,r);
ILQy  = subst(geom_inertia_Iy(ILQ_in_r),'r,r);
ILQxy = subst(geom_inertia_Ixy(ILQ_in_r),'r,r);
print("ILQ x  ",ILQx);
print("ILQ y  ",ILQy);
print("ILQ xy ",ILQxy);

ILQ_inexact = simplify(subst(ILQ_in_r,'r,r))
print("ILQ maximum ",geom_inertia_principal_xyangle(ILQ_inexact) * 180/Pi);

\\ cf k=20 segments
\\ 0.03562074648140108988311594583
\\ 0.4094816041723560090578580989
\\ -0.04869149690478950461511039961

\\ cf k=16 squares
\\ 0.0355904525659759
\\ 0.409619979461264
\\ 0.0486982465824549

system("sleep 1")