#!/usr/bin/perl -w

# Copyright 2019, 2020 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.

use 5.010;
use strict;
use warnings;
use File::Spec;
use FLAT;
use List::Util 'max','min','sum','uniqstr';
use Math::OEIS::Stripped;
use Math::PlanePath::Base::Digits
  'bit_split_lowtohigh','digit_join_lowtohigh','digit_split_lowtohigh';
use POSIX 'ceil';

use FindBin;
use lib::abs "$FindBin::Bin/../tools";
use MyFLAT;
$|=1;

# uncomment this to run the ### lines
# use Smart::Comments;

{
  # Bean, Ehrenfeucht, McNulty, page 268
  # 
  # not in OEIS: 0,1,3,1,0,1,2,0,2,1,3,0,2,0,3,1,2,0,2,1,0,1,3,1,0
  # not in OEIS: 1,2,4,2,1,2,3,1,3,2,4,1,3,1,4,2,3,1,3,2,1,2,4,2,1
  #
  # 0,4  unch
  # 1,3  +1
  # 2    -1
  
  my @table = ([0,1,3,1,0],
               [1,2,0,2,1],
               [2,3,1,3,2],
               [3,0,2,0,3]);
  my @h = (0);
  foreach (1 .. 2) {
    @h = map {@{$table[$_]}} @h;
  }
  # @h = map {$_+1} @h;
  print join(',',@h),"\n";
  my $str = join('',@h);
  if ($str =~ /(.+)\1/) {
    print "square $&\n";
  } else {
    print "square free\n";
  }

  my $f = FLAT::DFA->new;
  $f->add_states(4);
  $f->MyFLAT::set_name('high to low');
  $f->set_starting(0);
  $f->set_accepting(0);
  foreach my $from (0 .. 3) {
    foreach my $digit (0 .. 4) {
      my $to = $table[$from]->[$digit];
      $f->add_transition($from,$to, $digit);
    }
  }
  # $f->MyFLAT::view;
  $f = $f->MyFLAT::reverse
    ->MyFLAT::set_name('low to high')
    ->MyFLAT::minimize(verbose=>1);
  $f->MyFLAT::view;

  exit 0;
}



{
  # Thue h

  # 0,1,2,0,1,0,2,0,1,2,1,0,2,1,2,0,2,1,0,1,2,0,1,0,2,0,1,2,1
  # A086713 square-free
  #
  # not in OEIS: 1,2,3,1,2,1,3,1,2,3,2,1,3,2,3,1,3,2,1,2,3,1,2,1,3,1,2,3,2
  # not A099054 Arshon's, middle match only
  
  my @table = ([0,1,2,0,1],
               [0,2,0,1,2,1],
               [0,2,1,2,0,2,1]);
  my @h = (0);
  foreach (1 .. 2) {
    @h = map {@{$table[$_]}} @h;
  }
  # @h = map {$_+1} @h;
  print join(',',@h),"\n";
  my $str = join('',@h);
  if ($str =~ /(.+)\1/) {
    print "square $&\n";
  } else {
    print "square free\n";
  }
  exit 0;
}
{
  # Thue g

  # not in OEIS: 0,1,0,2,1,0,1,2,1,0,2,0,1,0,2,1,0,1,2,0,2,1,2,0,1,2,1,0,2
  # not in OEIS: 1,2,1,3,2,1,2,3,2,1,3,1,2,1,3,2,1,2,3,1,3,2,3,1,2,3,2,1,3

  my @table = ([0,1,0,2,1],
               [0,1,2,1,0,2],
               [0,1,2,0,2,1,2]);
  my @g = (0);
  foreach (1 .. 2) {
    @g = map {@{$table[$_]}} @g;
  }
  @g = map {$_+1} @g;
  print join(',',@g),"\n";
  my $str = join('',@g);
  if ($str =~ /(.+)\1/) {
    print "square $&\n";
  } else {
    print "square free\n";
  }
  exit 0;
}
