\\ Copyright 2014, 2015, 2016, 2017 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file.  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.


read("../tools/memoize.gp");

b = 1+I;

L(k) = if(k<0,error("L k<0"), k<=3,2^k, L(k-1) + 2*L(k-3));
L=memoize(L);
gL(x) = -1 + 2* (1 + x^2) / (1 - x - 2*x^3);

R(k)=1 + sum(j=0,k-1, L(j));
R = memoize(R);
gR(x)= 1 - 2/(1-x) + 2*(1 + x + x^2)/(1-x-2*x^3);

B(k) = R(k+1);
gB(x) = 2*( - 1/(1-x) + (2 + x + 2*x^2)/(1-x-2*x^3));

A(k) = 2^(k-1) - B(k)/4;
AL(k) = if(k==0,return(0)); A(k) - A(k-1);
AR(k) = if(k==0,0, A(k-1));
gA(x)=x^4 / ((1 - x - 2*x^3)*(1-x)*(1-2*x))
gAL(x)=x^4 / ((1 - x - 2*x^3)*(1-2*x));
gAR(x) = x*gA(x);

JA(k) = A(k+1) - 2*A(k);

GS(k) = (2/5 - 1/5*I)*b^k + GS_conj_part(k);
GS_conj_part(k) = (1/10 + 1/5*I)*conj(b/2)^k;
gGS(x) = (2/5-1/5*I)/(1-b*x) + (1/10+1/5*I)/(1-conj(b/2)*x);
gGSN(x) = gGS(2*x);

gGLN(x) =                                           \
  -1/2 + I                                          \
  + 1/25 * (2 - 6*I)   / (1 + x)                    \
  - 1/10 * (2 + I)     / (1 - conj(b)*x)            \
  + 1/50 * (61 - 23*I) / (1 - b*x)                  \
  - 1/5  * (3 + I)     / (1 - b*x)^2                \
  + 1/2 * gL(b*x);

gGRtotal(x) = (1/2)/(1-x) + x/(1-x) * (b*gL(b*x) - I*gGLN(x));

\\-----------------------------------------------------------------------------

nearly_equal(x,y) = abs(x-y) <= max(abs(x),max(abs(y),1))*1e-10;

differ(x,y) =
{
  \\ if(type(x)!=type(y),
  \\    return(Strprintf("different types %s %s",type(x),type(y))));
  if(type(x)=="t_COMPLEX" || type(y)=="t_COMPLEX",
     my(d=differ(real(x),real(y)));
     if(d != 0, return(Strprintf("real %s",d)));
     d=differ(imag(x),imag(y));
     if(d != 0, return(Strprintf("imag %s",d)));
     return(0));

  if(type(x)=="t_VEC" || type(x)=="t_COL",
     if(length(x)!=length(y),
        return(Strprintf("different lengths %s %s",length(x),length(y))));
     for(i=1,#x,
        my(d=differ(x[i],y[i]));
        if(d != 0, return(Strprintf("at index %d %s",i,d))));
     return(0));

  if(! nearly_equal(x,y),
     return(Strprintf("different values\n%s\n%s",x,y)));
  return(0);
}

check_count = 0;
check_equal(got,want) =
{
  check_count++;
  my(d = differ(got,want));
  if(d != 0,
     print("wrong\n",d); quit(1),
     print("ok "check_count));
}

check_exact(got,want) =
{
  check_count++;
  if(got!= want,
     print("wrong\n");
     print("got  "got);
     print("want "want);
     quit(1),
     print("ok "check_count));
}

gf_terms(g,n) = my(x=variable(g)); \
  concat(vector(min(n,valuation(g,x)), i, 0), \
         Vec(g + O(x^n)));

\\-----------------------------------------------------------------------------

GAN_samples_by_path = { 1/4 * [
   (0) + (0)*I,   /* k=1 */
   (0) + (0)*I,   /* k=2 */
   (0) + (0)*I,   /* k=3 */
   (-10) + (6)*I,   /* k=4 */
   (-52) + (-8)*I,   /* k=5 */
   (-102) + (-142)*I,   /* k=6 */
   (140) + (-576)*I,   /* k=7 */
   (1746) + (-982)*I,   /* k=8 */
   (6140) + (1760)*I,   /* k=9 */
   (9410) + (17274)*I,   /* k=10 */
   (-17412) + (57152)*I,   /* k=11 */
   (-158494) + (83354)*I   /* k=12 */
]};
GARN_samples_by_path = { 1/4 * [
   (0) + (0)*I,   /* k=1 */
   (0) + (0)*I,   /* k=2 */
   (0) + (0)*I,   /* k=3 */
   (0) + (0)*I,   /* k=4 */
   (-10) + (-6)*I,   /* k=5 */
   (-12) + (-60)*I,   /* k=6 */
   (130) + (-194)*I,   /* k=7 */
   (836) + (-204)*I,   /* k=8 */
   (2554) + (1382)*I,   /* k=9 */
   (2932) + (8676)*I,   /* k=10 */
   (-11894) + (25622)*I,   /* k=11 */
   (-79948) + (31140)*I   /* k=12 */
]};
GALN_samples_by_path = { 1/4 * [
0,
   (0) + (0)*I,   /* k=1 */
   (0) + (0)*I,   /* k=2 */
   (0) + (0)*I,   /* k=3 */
   (-10) + (6)*I,   /* k=4 */
   (-42) + (-2)*I,   /* k=5 */
   (-90) + (-82)*I,   /* k=6 */
   (10) + (-382)*I,   /* k=7 */
   (910) + (-778)*I,   /* k=8 */
   (3586) + (378)*I,   /* k=9 */
   (6478) + (8598)*I,   /* k=10 */
   (-5518) + (31530)*I,   /* k=11 */
   (-78546) + (52214)*I   /* k=12 */
]};


\\ 4*GAL(k)*AL(k) = 2^k*GS(k) - L(k)*GL(k)
\\ GAL(k) = 1/4*2^k/AL(k)*GS(k) - 1/4*L(k)/AL(k)*GL(k)
\\ GAL(k) = GS(k) + (1/4*2^k - AL(k))/AL(k)*GS(k) - 1/4*L(k)/AL(k)*GL(k)
\\ GAL(k) = GS(k) + 1/AL(k)*( (1/4*2^k - AL(k))*GS(k) - 1/4*L(k)*GL(k) )
\\ GAL(k) = GS(k) + 1/AL(k)*( 1/4*L(k)*GS(k) - 1/4*L(k)*GL(k) )
\\ GAL(k) = GS(k) + L(k)/AL(k)*( 1/4*GS(k) - 1/4*GL(k) )


\\-----------------------------------------------------------------------------
\\ Generating Function

gGALN(k) = 1/4 * (gGS(2*x) - gGLN(k));
gGARN(k) = 1/4 * (gGS(2*x) - gGRtotal(k));
gGAN(k) = 1/4 * (2*gGS(2*x) - gGBN(k));
gf_terms(gGALN(x), 8)
check_exact(gf_terms(gGALN(x), #GALN_samples_by_path), GALN_samples_by_path)
gGALN_ref = gGALN;

gf_term(g,n) = my(v=gf_terms(g,n+1)); v[#v];
my(k=100); GALF = gf_term(gGALN(x), k) / AL(k) / b^k * 1.0;
print("GALF "GALF)
\\ GALF = 2/5 - 1/5*I

my(k=100); GSF = gf_term(gGS(x), k) / b^k * 1.0
print("GSF "GSF)
\\ GSF -> 2/5 - 1/5*I

my(k=100); GARF = gf_term(gGARN(x), k) / AR(k) / b^k * 1.0;
print("GARF "GARF)
\\ GARF = 2/5 - 1/5*I


gGALN(k) = 
{
  (1/10 - 1/20*I) / (1-2*b*x)
  + (1/40 + 1/20*I) / (1-conj(b)*x)
  + 1/8 - 1/4*I
  - 1/100 * (2 - 6*I)   / (1 + x)                    
  + 1/40 * (2 + I)     / (1 - conj(b)*x)            
  - 1/200 * (61 - 23*I) / (1 - b*x)
  + 1/20  * (3 + I)     / (1 - b*x)^2
  - 1/8 * gL(b*x)
;
}
check_exact(gGALN(x), gGALN_ref(x));

gGALN(k) = 
{
  + 1/4 - 1/4*I
  - 1/100 * (2 - 6*I)   / (1 + x)                    
  + (3/40 + 3/40*I)     / (1-conj(b)*x)
  - 1/200 * (61 - 23*I) / (1 - b*x)
  + 1/20  * (3 + I)     / (1 - b*x)^2
  + (1/10 - 1/20*I)     / (1 - 2*b*x)
  - 1/4 * (1 + (b*x)^2) / (1 - (b*x) - 2*(b*x)^3)
;
}
check_exact(gGALN(x), gGALN_ref(x));

check_exact(gAL(x), 1/4 + 1/4*1/(1-2*x) - 1/2*(1 + x^2)/(1-x-2*x^3) )

gGALN(k) = 
{
  + 1/4 - 1/4*I
  - 1/100 * (2 - 6*I)   / (1 + x)                    
  + (3/40 + 3/40*I)     / (1-conj(b)*x)
  - 1/200 * (61 - 23*I) / (1 - b*x)
  + 1/20  * (3 + I)     / (1 - b*x)^2
  + (1/10 - 1/20*I)     / (1 - 2*b*x)
  - 1/4 * (gAL(b*x) - 1/4 - 1/4*1/(1-2*(b*x)))/(-1/2)
;
}
check_exact(gGALN(x), gGALN_ref(x));

gGALN(k) = 
{
  + 1/4 - 1/4*I
  - 1/100 * (2 - 6*I)   / (1 + x)                    
  + (3/40 + 3/40*I)     / (1-conj(b)*x)
  - 1/200 * (61 - 23*I) / (1 - b*x)
  + 1/20  * (3 + I)     / (1 - b*x)^2
  + (1/10 - 1/20*I)     / (1 - 2*b*x)
  + 1/2*gAL(b*x) - 1/8 - 1/8*1/(1-2*(b*x))
;
}
check_exact(gGALN(x), gGALN_ref(x));

gGALN(k) = 
{
  + 1/8 - 1/4*I
  - 1/100 * (2 - 6*I)   / (1 + x)                    
  + (3/40 + 3/40*I)     / (1-conj(b)*x)
  - 1/200 * (61 - 23*I) / (1 - b*x)
  + 1/20  * (3 + I)     / (1 - b*x)^2
  + (-1/40 - 1/20*I)    / (1 - 2*b*x)
  + 1/2*gAL(b*x)
;
}
check_exact(gGALN(x), gGALN_ref(x));




gAL_cubic(x) =  - 2 * (x^2 + 1)/(1-x-2*x^3);
gAL_cubic_k_plus_1(x) =  (gAL_cubic(x) + 2)/ x;
gAL_cubic_k_plus_2(x) =  (gAL_cubic(x) + 2)/ x;



quit
gGARN(k) = 1/4 * (gGS(2*x) - gR(k)*gGR(k));
gGAN(k) = 1/4 * (gGS(2*x) - gB(k)*gGB(k));




g(x) = {
         1/2*I
         + (5 + 9*I)/(4*x - 4)
         + (1 - 3*I)/(20*x + 20)
         + (46/41 + 78/41*I)  /(1 - x)
         + (-3/10 - 1/10*I)  /(1 - b*x)

         + ((-2 + 18*I)*x^2 + (-2 + 18*I)*x + (23 + 39*I))
         /(656*x^3 - 164*I*x + (82 + 82*I));
}
check_equal(Vec(g(x)-O(x^#GAL_samples)), \
            vector(#GAL_samples-3,k,my(k=k+2); join_area_centroid(k)))



g(x) = {
         1/2*I
         + (5 + 9*I)/(4*x - 4)
         + (1 - 3*I)/(20*x + 20)
         + (46/41 + 78/41*I)  /(1 - x)
         + (-3/10 - 1/10*I)  /(1 - b*x)

         + (4/41 + 5/41*I)/b^2
         * ( b^2*(2 - 3/2*I)
             + b^2*x
             + b^2*x^2
           )
         /(1 - b*x - 2*b^3*x^3 );
}
check_equal(Vec(g(x)-O(x^#GAL_samples)), \
            vector(#GAL_samples-3,k,my(k=k+2); join_area_centroid(k)))


g(x) = {
         1/2*I
         + (5 + 9*I)/(4*x - 4)
         + (1 - 3*I)/(20*x + 20)
         + (46/41 + 78/41*I)  /(1 - x)
         + (-3/10 - 1/10*I)  /(1 - b*x)

         + (4/41 + 5/41*I)
         * (
            (2 - 3/2*I)
            / (1 - b*x - 2*b^3*x^3 )

            + (1/2 - 1/2*I)
            * b*x
            / (1 - b*x - 2*b^3*x^3 )

            + -1/2*I
            * b^2*x^2
            / (1 - b*x - 2*b^3*x^3 )
            );
}
check_equal(Vec(g(x)-O(x^#GAL_samples)), \
            vector(#GAL_samples-3,k,my(k=k+2); join_area_centroid(k)))


g(x) = {
         (31/82 + 49/82*I)
         + (-21/164 - 57/164*I)  /(1 - x)
         + (1/20 - 3/20*I) /(1 + x)
         + (-3/10 - 1/10*I)  /(1 - b*x)

         + (4/41 + 5/41*I)
         * (
              (5/2 - 2*I)
            * b*x
            / (1 - b*x - 2*b^3*x^3 )

            + -1/2*I
            * b^2*x^2
            / (1 - b*x - 2*b^3*x^3 )

            + (2 - 3/2*I)*2
            * b^3*x^3
            / (1 - b*x - 2*b^3*x^3 )
            );
}
check_equal(Vec(g(x)-O(x^#GAL_samples)), \
            vector(#GAL_samples-3,k,my(k=k+2); join_area_centroid(k)))



g(x) = {
         (31/82 + 49/82*I)
         + (-21/164 - 57/164*I)  /(1 - x)
         + (1/20 - 3/20*I) /(1 + x)
         + (-3/10 - 1/10*I)  /(1 - b*x)

         + (4/41 + 5/41*I)
         * (
              (5/2 - 2*I) * b*x
            + -1/2*I      * b^2*x^2
            + (2 - 3/2*I)*2  * b^3*x^3
            )
            / (1 - b*x - 2*b^3*x^3 )
;
}
check_equal(Vec(g(x)-O(x^#GAL_samples)), \
            vector(#GAL_samples-3,k,my(k=k+2); join_area_centroid(k)))

g(x) = {
         (31/82 + 49/82*I)
         + (-21/164 - 57/164*I)  /(1 - x)
         + (1/20 - 3/20*I) /(1 + x)
         + (-3/10 - 1/10*I)  /(1 - b*x)

         + (4/41 + 5/41*I)
         * (1-b*x)*(
              (5/2 - 2*I) * b*x
            + -1/2*I      * b^2*x^2
            + (2 - 3/2*I)*2  * b^3*x^3
            )
            / ((1-b*x)*(1 - b*x - 2*b^3*x^3 ))
;
}
check_equal(Vec(g(x)-O(x^#GAL_samples)), \
            vector(#GAL_samples-3,k,my(k=k+2); join_area_centroid(k)))

g(x) = {
         (31/82 + 49/82*I)
         + (-21/164 - 57/164*I)  /(1 - x)
         + (1/20 - 3/20*I) /(1 + x)
         + (-3/10 - 1/10*I)  /(1 - b*x)

         + (4/41 + 5/41*I)
         * ((16 - 12*I)*x^4
            + (-3 + 13*I)*x^3
            + (-3 - 5*I)*x^2
            + (9/2 + 1/2*I)*x)
            / ((1-b*x)*(1 - b*x - 2*b^3*x^3 ))
;
}
check_equal(Vec(g(x)-O(x^#GAL_samples)), \
            vector(#GAL_samples-3,k,my(k=k+2); join_area_centroid(k)))

g(x) = {
         (31/82 + 49/82*I)
         + (-21/164 - 57/164*I)  /(1 - x)
         + (1/20 - 3/20*I) /(1 + x)
         + (-3/10 - 1/10*I)  /(1 - b*x)

         + (16 - 12*I)/(2*b^4) * (4/41 + 5/41*I)

         + (4/41 + 5/41*I)
         * (

            - (16 - 12*I)/(2*b^4) * ((1-b*x)*(1 - b*x - 2*b^3*x^3 ))
            + (16 - 12*I)*x^4
            + (-3 + 13*I)*x^3
            + (-3 - 5*I)*x^2
            + (9/2 + 1/2*I)*x)
            / ((1-b*x)*(1 - b*x - 2*b^3*x^3 ))
;
}
check_equal(Vec(g(x)-O(x^#GAL_samples)), \
            vector(#GAL_samples-3,k,my(k=k+2); join_area_centroid(k)))


g(x) = {
         1/2*I
         + (-21/164 - 57/164*I)  /(1 - x)
         + (1/20 - 3/20*I) /(1 + x)
         + (-3/10 - 1/10*I)  /(1 - b*x)

         + (4/41 + 5/41*I)
         * (+ (2 - 3/2*I)
            + (-3/2 + I)  * b*x
            - 1/2         * b^2*x^2
            + 1/2*I       * b^3*x^3
           )
            / ((1-b*x)*(1 - b*x - 2*b^3*x^3 ))
;
}
check_equal(Vec(g(x)-O(x^#GAL_samples)), \
            vector(#GAL_samples-3,k,my(k=k+2); join_area_centroid(k)))


g(x) = {
         1/2*I
         + 1/164*(-21 - 57*I) /(1 - x)
         + 1/20 *(  1 - 3*I ) /(1 + x)
         + 1/10 *( -3 -   I ) /(1 - b*x)

         + 1/82
         * ((31 + 8*I) + (8 + 10*I)*x + (8 + 10*I)*x^2)
         / (1 - b*x - 2*b^3*x^3)
         ;
}
gGALN = g;
check_equal(Vec(g(x)-O(x^#GAL_samples)), \
            vector(#GAL_samples-3,k,my(k=k+2); join_area_centroid(k)))
check_equal(gf_terms(g(x),100), \
            vector(100,k,my(k=k-1); join_area_centroid(k)))

\\ gAL(x) = -(1/2)/(1-x) + (1/2)/(1-x-2*x^3) );
\\ \\ gAL(x) = x^3 / ((1 - x - 2*x^3)*(1-x));
\\ gL(x) = -1 + (2 + 2*x^2) / (1 - x - 2*x^3);
\\ gR(x) = 1 - 2/(1-x) + 2*(1 + x + x^2)/(1-x-2*x^3);

check_exact(gAL(x), x^3 / ((1 - x - 2*x^3)*(1-x)))
check_exact(gAL(x), -(1/2)/(1-x) + (1/2)/(1-x-2*x^3))
gAL_cubic(x)          =  (1/2)/(1-x-2*x^3);
gAL_cubic_k_plus_1(x) =  (gAL_cubic(x) - 1/2)/x;
gAL_cubic_k_plus_2(x) =  (gAL_cubic_k_plus_1(x) - 1/2)/x;

check_exact(gAL_cubic_k_plus_1(x), (1/2     + x^2)/(1-x-2*x^3));
check_exact(gAL_cubic_k_plus_2(x), (1/2 + x + x^2)/(1-x-2*x^3));

{
check_exact(
matsolve(mattranspose([1/2, b*0, b^2*0;
                       1/2, b*0, b^2*1;
                       1/2, b*1, b^2*1]),
                      [(31 + 8*I); (8 + 10*I); (8 + 10*I) ]),
[57 + 20*I; -4 - 5*I; 9 + I]
)
}

{
check_exact(
              (57 + 20*I)*gAL_cubic(b*x)
            + (-4 -  5*I)*gAL_cubic_k_plus_1(b*x)
            + ( 9 +    I)*gAL_cubic_k_plus_2(b*x),

            ((31 + 8*I) + (8 + 10*I)*x + (8 + 10*I)*x^2)
            / (1 - b*x - 2*b^3*x^3)
)
}

gAL_k_plus_1(x) =  gAL(x)/x;
gAL_k_plus_2(x) =  gAL_k_plus_1(x)/x;

check_exact(gAL(x) + (1/2)/(1-x), gAL_cubic(x))
check_exact(gAL(b*x) + (1/2)/(1-b*x), gAL_cubic(b*x))

check_exact(gAL(b*x) + (1/2)/(1-b*x), gAL_cubic(b*x))
check_exact(gAL_k_plus_1(b*x) + (1/2)/(1-b*x), gAL_cubic_k_plus_1(b*x))
check_exact(gAL_k_plus_2(b*x) + (1/2)/(1-b*x), gAL_cubic_k_plus_2(b*x))

g(x) = {
         1/2*I
         + 1/164*(-21 - 57*I) /(1 - x)
         + 1/20 *(  1 - 3*I ) /(1 + x)
         + 1/10 *( -3 -   I ) /(1 - b*x)

         + 1/82
         * (
            (57 + 20*I)*(gAL(b*x)  + (1/2)/(1-b*x))
            + (-4 -  5*I)*(gAL_k_plus_1(b*x)  + (1/2)/(1-b*x))
            + ( 9 +    I)*(gAL_k_plus_2(b*x)   + (1/2)/(1-b*x))
           )
         ;
}
check_exact(g(x), gGALN(x))
check_equal(Vec(g(x)-O(x^#GAL_samples)), \
            vector(#GAL_samples-3,k,my(k=k+2); join_area_centroid(k)))
check_equal(gf_terms(g(x),100), \
            vector(100,k,my(k=k-1); join_area_centroid(k)))


g(x) = {
         1/2*I
         + 1/164*(-21 - 57*I) /(1 - x)
         + 1/20 *(  1 - 3*I ) /(1 + x)
         + 1/410*( 32 - I   ) /(1 - b*x)
         + 1/82
         * (
              (57 + 20*I)*gAL(b*x)         
            + (-4 -  5*I)*gAL_k_plus_1(b*x)
            + ( 9 +    I)*gAL_k_plus_2(b*x)
           )
         ;
}
check_exact(g(x), gGALN(x))

GALN(k) = {
         if(k==0,1/2*I,0)
         + 1/164*(-21 - 57*I)
         + 1/20 *(  1 - 3*I )*(-1)^k
         + 1/410*( 32 - I   )*b^k
         + 1/82 * b^k
         * (
              (57 + 20*I)*AL(k)         
            + (-4 -  5*I)*AL(k+1)
            + ( 9 +    I)*AL(k+2)
           )
         ;
}
check_equal(gf_terms(g(x),100), \
            vector(100,k,my(k=k-1); GALN(k)))


\\-----------------------------------------------------------------------------
\\ by LQ

GLQ_by_GALN(k) = 1/LQ(k) * (GALN(k) + JEQ(k) + b^(k+1)*AL(k+1) - GALN(k+1));
vector(10,k,GLQ_by_GALN(k))

\\ GLQN(k) = GALN(k) + JEQ(k) + b^(k+1)*AL(k+1) - GALN(k+1)
\\ GALN(k+1) = GALN(k) - GLQN(k) + JEQ(k) + b^(k+1)*AL(k+1)
\\ GALN(k) = GALN(k-1)  - GLQN(k-1) + JEQ(k-1) + b^k*AL(k)

GALN_by_LQ(k) = if(k==0,return(GALN(0))); \
                GALN(k-1)  - GLQN(k-1) + JEQ(k-1) + b^k*AL(k);
check_equal(gf_terms(g(x),100), \
            vector(100,k,my(k=k-1); GALN_by_LQ(k)))

GALN_by_LQ(k) = if(k<1,return(GALN(k))); \
                GALN(1) + sum(j=1,k, JEQ(j-1) + b^j*AL(j) - GLQN(j-1) );
check_equal(gf_terms(g(x),100), \
            vector(100,k,my(k=k-1); GALN_by_LQ(k)))

GALN_by_LQ(k) = sum(j=0,k-1, JEQ(j) + b^(j+1)*AL(j+1) - GLQN(j) );
check_equal(gf_terms(g(x),100), \
            vector(100,k,my(k=k-1); GALN_by_LQ(k)))

GALN_by_LQ(k) = {
  sum(j=0,k-1, - LQ(j)*1/2*b^max(1,j))
  + sum(j=0,k-1, (3/5 + 1/5*I)*b^j - (1/10 - 3/10*I)*(-1)^j)
  + sum(j=0,k-1, b^(j+1)*AL(j+1));
}
check_equal(gf_terms(g(x),100), \
            vector(100,k,my(k=k-1); GALN_by_LQ(k)))

GALN_by_LQ(k) = {
  if(k==0,0,-1/2*I)
  + sum(j=0,k-1, - LQ(j)*1/2*b^j) 
  + sum(j=0,k-1, (3/5 + 1/5*I)*b^j - (1/10 - 3/10*I)*(-1)^j)
  + sum(j=0,k-1, b^(j+1)*AL(j+1));
}
check_equal(gf_terms(g(x),100), \
            vector(100,k,my(k=k-1); GALN_by_LQ(k)))

GALN_by_LQ(k) = {
  if(k==0,0,-1/2*I)
  + sum(j=0,k-1, (3/5 + 1/5*I)*b^j )
  - sum(j=0,k-1, (1/10 - 3/10*I)*(-1)^j )
  + sum(j=0,k-1, b^(j+1)*AL(j+1) )
  - 1/2* sum(j=0,k-1, LQ(j)*b^j)
  ;
}
check_equal(gf_terms(g(x),100), \
            vector(100,k,my(k=k-1); GALN_by_LQ(k)))

GALN_by_LQ(k) = {
  if(k==0,0,-1/2*I)
  + (3/5 + 1/5*I) * (b^k - 1)/(b-1)
  - (1/10 - 3/10*I) * ((-1)^k - 1)/(-1 -1)
  + sum(j=0,k-1, b^(j+1)*AL(j+1) )
  - 1/2* sum(j=0,k-1, (AL(j+1)+1+AL(j))*b^j)
  ;
}
check_equal(gf_terms(g(x),100), \
            vector(100,k,my(k=k-1); GALN_by_LQ(k)))

GALN_by_LQ(k) = {
  if(k==0,0,-1/2*I)
  + (-1/4 + 3/4*I)
  + (1/5 - 3/5*I) * b^k
  + (1/20 - 3/20*I) * (-1)^k
  + sum(j=0,k-1, b^(j+1)*AL(j+1) - 1/2* (AL(j+1)+1+AL(j))*b^j)
  ;
}
check_equal(gf_terms(g(x),100), \
            vector(100,k,my(k=k-1); GALN_by_LQ(k)))

GALN_by_LQ(k) = {
  if(k==0,0,-1/2*I)
  + (-1/4 + 3/4*I)
  + (1/5 - 3/5*I) * b^k
  + (1/20 - 3/20*I) * (-1)^k
  + sum(j=0,k-1, b^(j+1)*AL(j+1) - 1/2*AL(j+1)*b^j - 1/2*b^j - 1/2*AL(j)*b^j )
  ;
}
check_equal(gf_terms(g(x),100), \
            vector(100,k,my(k=k-1); GALN_by_LQ(k)))

GALN_by_LQ(k) = {
  if(k==0,0,-1/2*I)
  + (-1/4 + 1/4*I)
  + (1/5 - 1/10*I) * b^k
  + (1/20 - 3/20*I) * (-1)^k
  + sum(j=0,k-1, - 1/2*AL(j)*b^j + (1/2 + I)*AL(j+1)*b^j )
  ;
}
check_equal(gf_terms(g(x),100), \
            vector(100,k,my(k=k-1); GALN_by_LQ(k)))


\\-----------------------------------------------------------------------------
\\ by LQ with generating function

gGLQN(x) = 1/2*I + 1/2*gLQ(b*x);
check_equal(gf_terms(gGLQN(x),100), \
            vector(100,k,my(k=k-1); GLQN(k)))

gJEQ(x) = (3/5 + 1/5*I)/(1-b*x) - (1/10 - 3/10*I)/(1+x);
gf_terms(gJEQ(x),100) == vector(100,k,my(k=k-1); JEQ(k)) || quit;

\\ GLQN(k) = GALN(k) + JEQ(k) + b^(k+1)*AL(k+1) - GALN(k+1)
\\ GALN(k+1) = GALN(k) - GLQN(k) + JEQ(k) + b^(k+1)*AL(k+1)
\\ GALN(k) = GALN(k-1)  - GLQN(k-1) + JEQ(k-1) + b^k*AL(k)
\\ gGALN(x) = x*gGALN(x)  - x*gGLQN(x) + JEQ(k-1) + b^k*AL(k)

gGALN_by_LQ(x) = 1/(1-x) * ( - x*gGLQN(x) + x*gJEQ(x) + gAL(b*x) );
gGALN_by_LQ(x)
gf_terms(g(x),10)
gf_terms(gGALN_by_LQ(x),10)
check_equal(gf_terms(g(x),100), gf_terms(gGALN_by_LQ(x),100))

gGALN_by_LQ(x) = {
    1/2*I
     + (-21/164 - 57/164*I)/(1 - x)
     + (1/20 - 3/20*I)/(1 + x)
     + (-3/10 - 1/10*I)/(1 - (1 + I)*x)
     + ((31/82 + 4/41*I) + (4/41 + 5/41*I)*x + (4/41 + 5/41*I)*x^2)/(1 - (1 + I)*x + (4 - 4*I)*x^3)
}
check_exact(gGALN_by_LQ(x), g(x))

\\-----------------------------------------------------------------------------
\\ cubic r limit

\\ cf GRf = 1/82*((70-15*I) + (3+14*I)*r - (17+11*I)*r^2);

r = 1.695620769559862057416367100;
GALF = 1/82*((57 + 20*I) \
             + (-4 -  5*I)*r \
             + ( 9 +    I)*r^2)

1 - GALF
GALF*2
\\ 0.9279717691368970878548813042 + 0.1755734871507740065718553060*I


\\-----------------------------------------------------------------------------
\\ with GRQN

GRQN(k) = (1 - 1/2*I)  +  (1 + 1/2*I)* sum(i=0,k-1, LQ(i)*b^i);
\\ (GRQN(k) - (1 - 1/2*I)) =   (1 + 1/2*I)* sum(i=0,k-1, LQ(i)*b^i);
\\ sum(i=0,k-1, LQ(i)*b^i) = 1/(1 + 1/2*I) * (GRQN(k) - (1 - 1/2*I))
\\ sum(i=0,k-1, LQ(i)*b^i) = (-3/5 + 4/5*I) + (4/5 - 2/5*I)*GRQN(k)

GALN_by_RQ(k) = {
  if(k==0,0,-1/2*I)
  + sum(j=0,k-1, (3/5 + 1/5*I)*b^j )
  - sum(j=0,k-1, (1/10 - 3/10*I)*(-1)^j )
  + sum(j=0,k-1, b^(j+1)*AL(j+1) )
  - 1/2* ((-3/5 + 4/5*I) + (4/5 - 2/5*I)*GRQN(k))
  ;
}
check_equal(gf_terms(g(x),100), \
            vector(100,k,my(k=k-1); GALN_by_RQ(k)))

\\ gLQ(x) = (1 + x^2) / (1 - x - 2*x^3);
\\ gAL(x) = 1/2*( - 1/(1-x) + 1/(1-x-2*x^3) );
gLQ_k_plus_1(x) = (gLQ(x)-1)/x;
gLQ_k_plus_2(x) = (gLQ_k_plus_1(x)-1)/x;
check_exact(gLQ_k_plus_1(x), (1 +   x + 2*x^2)/(-2*x^3 - x + 1) )
check_exact(gLQ_k_plus_2(x), (2 + 2*x + 2*x^2)/(-2*x^3 - x + 1) )

m = {[1, 0, 1;
      1, 1, 2;
      2, 2, 2]};
check_exact(matsolve(mattranspose(m), [1;0;0]), [1; -1; 1/2])

check_exact(2*(gAL(x) + (1/2)/(1-x)), \
            (gLQ(x) - gLQ_k_plus_1(x) + 1/2*gLQ_k_plus_2(x)))
check_exact(gAL(x), \
            -(1/2)/(1-x) \
            + 1/2*gLQ(x) \
            - 1/2*gLQ_k_plus_1(x) \
            + 1/4*gLQ_k_plus_2(x) )

AL_by_LQ(k) = -1/2 + 1/2*LQ(k) - 1/2*LQ(k+1) + 1/4*LQ(k+2);
check_equal(vector(100,k,my(k=k-1); AL_by_LQ(k)), \
            vector(100,k,my(k=k-1); AL(k)))

gLQ_k_minus_1(x) = x*gLQ(x);
check_exact(gLQ_k_minus_1(x), -1/2 + (1/2 + 1/2*x)/(1 - x -2*x^3) )
m = {[1/2, 1/2, 0;
       1,   0,  1;
       1,   1,  2]};
matdet(m)
check_exact(matsolve(mattranspose(m), [1;0;0]), [1; 1; -1/2])

check_exact(2*(gAL(x) + (1/2)/(1-x)), \
            (gLQ_k_minus_1(x)+1/2 + gLQ(x) - 1/2*gLQ_k_plus_1(x)))
{
check_exact(gAL(x),
            (1/4
             - (1/2)/(1-x)
             + 1/2*gLQ_k_minus_1(x)
             + 1/2*gLQ(x)
             - 1/4*gLQ_k_plus_1(x)))
}

\\ AL by LQ -1,0,1
AL_by_LQ_minus(k) = if(k<0,error("AL_k_by_LQ_minux() is for k>=0")); \
  if(k==0,return(0)); \
  - 1/2 + 1/2*LQ(k-1) + 1/2*LQ(k) - 1/4*LQ(k+1);
\\ vector(10,k,my(k=k-1); AL_by_LQ_minus(k))
\\ vector(10,k,my(k=k-1); AL(k))
check_equal(vector(100,k,my(k=k-1); AL_by_LQ_minus(k)), \
            vector(100,k,my(k=k-1); AL(k)))
