\\ Copyright 2014, 2016, 2017 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file.  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.


read("../tools/memoize.gp");

b = 1+I;
L(k) = if(k<0,error("L k<0"), k<=3,2^k, L(k-1) + 2*L(k-3));
L=memoize(L);
R(k)=1 + sum(j=0,k-1, L(j));
R = memoize(R);
B(k) = R(k+1);
A(k) = 2^(k-1) - B(k)/4;
JA(k) = A(k+1) - 2*A(k);
JEQ(k) = (3/5 + 1/5*I)*b^k - (1/10 - 3/10*I)*(-1)^k;
LQ(k) = if(k<0,error("LQ() is for k>=0")); \
        if(k==0,return(1)); \
        if(k<=3,return(2^(k-1))); \
        LQ(k-1) + 2*LQ(k-3);
LQ = memoize(LQ);
gLQ(x) = (1 + x^2) / (1 - x - 2*x^3);

GLQ(k) = 1/2*b^max(1,k);
GLQN(k) = LQ(k)*1/2*b^max(1,k);

\\-----------------------------------------------------------------------------

nearly_equal(x,y) = abs(x-y) <= max(abs(x),max(abs(y),1))*1e-10;

differ(x,y) =
{
  \\ if(type(x)!=type(y),
  \\    return(Strprintf("different types %s %s",type(x),type(y))));
  if(type(x)=="t_COMPLEX" || type(y)=="t_COMPLEX",
     my(d=differ(real(x),real(y)));
     if(d != 0, return(Strprintf("real %s",d)));
     d=differ(imag(x),imag(y));
     if(d != 0, return(Strprintf("imag %s",d)));
     return(0));

  if(type(x)=="t_VEC" || type(x)=="t_COL",
     if(length(x)!=length(y),
        return(Strprintf("different lengths %s %s",length(x),length(y))));
     for(i=1,#x,
        my(d=differ(x[i],y[i]));
        if(d != 0, return(Strprintf("at index %d %s",i,d))));
     return(0));

  if(! nearly_equal(x,y),
     return(Strprintf("different values\n%s\n%s",x,y)));
  return(0);
}

check_count = 0;
check_equal(got,want) =
{
  check_count++;
  my(d = differ(got,want));
  if(d != 0,
     print("wrong\n",d); quit(1),
     print("ok "check_count));
}

check_exact(got,want) =
{
  check_count++;
  if(got!= want,
     print("wrong\n");
     print("got  "got);
     print("want "want);
     quit(1),
     print("ok "check_count));
}

\\-----------------------------------------------------------------------------

GJA_samples = 1/2*[ 0+0*I, 0+0*I, 0+0*I, -5+3*I, -16+-4*I, -21+-29*I, 16+-92*I, 195+-133*I, 544+100*I, 755+1099*I, -608+3204*I, -6445+4395*I, -18336+-3452*I ];
GJA(k) = GJA_samples[k+1];


\\            9----8    5---4               2
\\            |    |    |   |
\\           10--11,7---6   3---2           1
\\                 |            |
\\      16   13---12        0---1       <- Y=0
\\       |    |
\\      15---14
\\
\\      -4   -3   -2   -1   0   1
JA(3)==1 || quit;
GJA(3)==-5/2 + 3/2*I || quit;

\\            9----8    5---4               2
\\            |    |    |   |
\\           10--11,7---6   3---2           1
\\                 |            |
\\ 17---16   13---12        0---1       <- Y=0
\\  | J  |    |
\\ 18-19,15-14,22-23                       -1
\\       | J  |    |
\\      20--21,25-24                       -2
\\            |
\\           26---27                       -3
\\                 |
\\    --32   29---28                       -4
\\       |    |
\\      31---30                            -5
\\
\\  ^    ^    ^    ^    ^   ^   ^
\\ -5   -4   -3   -2   -1  X=0  1 ...
JA(4)==2 || quit;

\\ math-image --path=DragonCurve --all --output=numbers_dash --size=70x30

\\ k=5
\\        9--8  5--4
\\        |  |  |  |
\\       10-11--6  3--2
\\           |        |
\\ 17-16 13-12     0--1
\\  |  |  |
\\ 18-19-22-23
\\     |  |  |
\\    20-25-24
\\        |
\\       26-27 46-47       94-95
\\           |  |  |        |  |
\\ 33-32 29-44-49-48    92-97-64
\\  |J |  |J |  |        |  |
\\ 34-35-38-43-50-51 54-55-98-99
\\     |J |  |  |  |  |  |  |
\\    36-37 40-41 52-57-56-89-84
\\                    |  |  |  |
\\                   58-57
JA(5)==3 || quit;
JA(6)==6 || quit;


\\     +-+ +-+
\\     | | | |
\\     +-+-+ +-+
\\       |     |
\\ +-+ +-+   0-+
\\ | | |
\\ +-+-+-+
\\   | | |
\\   +-+-+
\\     |
\\     +-+ +-+
\\       | | |
\\ +-+ +-+-+-+
\\ | | | | |
\\ +-+-+-+-+-+ +-+
\\   | | | | | | |J
\\   +-+ +-+ +-+-+
\\             |J
\\             +-+
\\               |J
\\           E +-+
\\          J| |J
\\           +-+
\\            J
\\


\\ at k=8 want -4+4*I = 2*b^3

join_area_centroid =
{
  memoize(k->
          if(k<3, return(GJA(k)));

          join_area_centroid(k-1) + JEQ(k-1)
          + (b^k - b^(k-1))*(JA(k-1)+1)

          + (b^k + 2*b^(k-5)) * 2*JA(k-3));
}
\\ at k=6
my(k=6); my(E=-8*I); \
check_equal(join_area_centroid(k-1) + JEQ(k-1)  + (b^k - b^(k-1))*(JA(k-1)+1), \
            E-1/2-1/2*I  + E+1/2-3/2*I  + E+3/2-1/2*I  + E+5/2+1/2*I)

my(k=6); check_equal(JA(k-3), 1)
my(k=6); my(E=-8*I); \
check_equal(join_area_centroid(k), \
            E-1/2-1/2*I  + E+1/2-3/2*I  + E+3/2-1/2*I  + E+5/2+1/2*I \
            + 2*(E+2+2*I))

vector(7,k,my(k=k-1); join_area_centroid(k))
vector(7,k,my(k=k-1); GJA(k))
check_equal(vector(#GJA_samples,k,my(k=k-1); join_area_centroid(k)), \
            vector(#GJA_samples,k,my(k=k-1); GJA(k)))




join_area_centroid =
{
  memoize(k->
          if(k<3, return(GJA(k)));

          join_area_centroid(k-1)

          + (3/5 + 6/5*I)/b * b^k
          + (1/10 - 3/10*I) * (-1)^k

          +         I* b^(k-1) *JA(k-1)
          + (-4 + 2*I)*b^(k-3) *JA(k-3));
}
check_equal(vector(#GJA_samples,k,my(k=k-1); join_area_centroid(k)), \
            vector(#GJA_samples,k,my(k=k-1); GJA(k)))


join_area_centroid(k) =
{
  my(base=2);
  if(k<base, return(0));

  sum(j=base,k-1,
        (3/5 + 6/5*I) * b^j
        - (1/10 - 3/10*I) * (-1)^j
        
        +         I* b^j     *JA(j)
        + (-4 + 2*I)*b^(j-2) *JA(j-2));
}
check_equal(vector(#GJA_samples,k,my(k=k-1); join_area_centroid(k)), \
            vector(#GJA_samples,k,my(k=k-1); GJA(k)))


join_area_centroid(k) =
{
  my(base=2);
  if(k<base, return(0));

  (-5/4 - 9/4*I)
  + (1/20 - 3/20*I) * (-1)^k
  + (6/5 - 3/5*I)   * b^k
  +
  sum(j=base,k-1,
        +         I* b^j     *JA(j)
        + (-4 + 2*I)*b^(j-2) *JA(j-2));
}
check_equal(vector(#GJA_samples,k,my(k=k-1); join_area_centroid(k)), \
            vector(#GJA_samples,k,my(k=k-1); GJA(k)))


join_area_centroid(k) =
{
  my(base=2);
  if(k<base, return(0));

  (-5/4 - 9/4*I)
  + (1/20 - 3/20*I) * (-1)^k
  + (6/5 - 3/5*I)   * b^k
  +
  sum(j=base,k-1,
        +         I* b^j *JA(j)
        + (1 + 2*I)*b^j  *JA(j-2));
}
check_equal(vector(#GJA_samples,k,my(k=k-1); join_area_centroid(k)), \
            vector(#GJA_samples,k,my(k=k-1); GJA(k)))

join_area_centroid(k) =
{
  my(base=2);
  if(k<base, return(0));

  (-5/4 - 9/4*I)
  + (1/20 - 3/20*I) * (-1)^k
  + (6/5 - 3/5*I)   * b^k
  +
  sum(j=base,k-1,
        + I*b^j * JA(j)
        + I*b^j * 2*JA(j-2)

        + b^j  *JA(j-2));
}
check_equal(vector(#GJA_samples,k,my(k=k-1); join_area_centroid(k)), \
            vector(#GJA_samples,k,my(k=k-1); GJA(k)))

join_area_centroid(k) =
{
  my(base=2);
  if(k<base, return(0));

  (-5/4 - 9/4*I)
  + (1/20 - 3/20*I) * (-1)^k
  + (6/5 - 3/5*I)   * b^k
  +
  sum(j=base,k-1,
        + I*b^j * (JA(j+1) - 1)
        + b^j  *JA(j-2));
}
check_equal(vector(#GJA_samples,k,my(k=k-1); join_area_centroid(k)), \
            vector(#GJA_samples,k,my(k=k-1); GJA(k)))

join_area_centroid(k) =
{
  my(base=2);
  if(k<base, return(0));

  (-5/4 - 9/4*I)
  + (1/20 - 3/20*I) * (-1)^k
  + (6/5 - 3/5*I)   * b^k
  +
  sum(j=base,k-1,
        - I*b^j
        + I*b^j * JA(j+1)
        + b^j  *JA(j-2));
}
check_equal(vector(#GJA_samples,k,my(k=k-1); join_area_centroid(k)), \
            vector(#GJA_samples,k,my(k=k-1); GJA(k)))

join_area_centroid(k) =
{
  my(base=2);
  if(k<base, return(0));

  (-5/4 - 1/4*I)
  + (1/20 - 3/20*I) * (-1)^k
  + (1/5 - 3/5*I)   * b^k
  + sum(j=base,k-1,
          + I*b^j * JA(j+1)
          +   b^j * JA(j-2));
}
check_equal(vector(#GJA_samples,k,my(k=k-1); join_area_centroid(k)), \
            vector(#GJA_samples,k,my(k=k-1); GJA(k)))


vector(14,k,sum(j=0,k,JA(k)))/2


\\-----------------------------------------------------------------------------
\\ Generating Function

gJA(x) = 1/2*( - 1/(1-x) + 1/(1-x-2*x^3) );
\\ gJA(x) = x^3 / ((1 - x - 2*x^3)*(1-x));
gL(x) = -1 + (2 + 2*x^2) / (1 - x - 2*x^3);
gR(x) = 1 - 2/(1-x) + 2*(1 + x + x^2)/(1-x-2*x^3);

gf_terms(g,n) = my(x=variable(g)); \
  concat(vector(min(n,valuation(g,x)), i, 0), \
         Vec(g + O(x^n)));

\\ starting at x^3
g(x) = {
         1/(1-x) *(
                   - x*(GJA(0)
                        + x*GJA(1)*x)
                   
                   + (3/5 + 6/5*I)/b * (1/(1-b*x)  - 1 - b*x - b^2*x^2)
                   + (1/10 - 3/10*I) * (1/(1-(-1)*x)  - 1 + x - x^2 )
                   
                   + I*x*gJA(b*x)
                   + (-4 + 2*I) *x^3 * gJA(b*x)
                  );
}
#GJA_samples
\\ g(x)
Vec(g(x) - O(x^7))
vector(#GJA_samples-3,k,my(k=k+2); join_area_centroid(k))
check_equal(Vec(g(x)-O(x^#GJA_samples)), \
            vector(#GJA_samples-3,k,my(k=k+2); join_area_centroid(k)))

g(x) = {
         3*I
         + (-1/2 + 3/2*I)*x
         + (5 + 9*I)/(4*x - 4)
         + (1 - 3*I)/(20*x + 20)
         + (-3 + 9*I)/(10*x + (-5 + 5*I))
         + 1/(1-x) *(
                   + I          *x   * gJA(b*x)
                   + (-4 + 2*I) *x^3 * gJA(b*x)
                  );
}
check_equal(Vec(g(x)-O(x^#GJA_samples)), \
            vector(#GJA_samples-3,k,my(k=k+2); join_area_centroid(k)))


g(x) = {
         1/2*I
         + (5 + 9*I)/(4*x - 4)
         + (1 - 3*I)/(20*x + 20)
         + (46/41 + 78/41*I)  /(1 - x)
         + (-3/10 - 1/10*I)  /(1 - b*x)

         + ((-2 + 18*I)*x^2 + (-2 + 18*I)*x + (23 + 39*I))
         /(656*x^3 - 164*I*x + (82 + 82*I));
}
check_equal(Vec(g(x)-O(x^#GJA_samples)), \
            vector(#GJA_samples-3,k,my(k=k+2); join_area_centroid(k)))



g(x) = {
         1/2*I
         + (5 + 9*I)/(4*x - 4)
         + (1 - 3*I)/(20*x + 20)
         + (46/41 + 78/41*I)  /(1 - x)
         + (-3/10 - 1/10*I)  /(1 - b*x)

         + (4/41 + 5/41*I)/b^2
         * ( b^2*(2 - 3/2*I)
             + b^2*x
             + b^2*x^2
           )
         /(1 - b*x - 2*b^3*x^3 );
}
check_equal(Vec(g(x)-O(x^#GJA_samples)), \
            vector(#GJA_samples-3,k,my(k=k+2); join_area_centroid(k)))


g(x) = {
         1/2*I
         + (5 + 9*I)/(4*x - 4)
         + (1 - 3*I)/(20*x + 20)
         + (46/41 + 78/41*I)  /(1 - x)
         + (-3/10 - 1/10*I)  /(1 - b*x)

         + (4/41 + 5/41*I)
         * (
            (2 - 3/2*I)
            / (1 - b*x - 2*b^3*x^3 )

            + (1/2 - 1/2*I)
            * b*x
            / (1 - b*x - 2*b^3*x^3 )

            + -1/2*I
            * b^2*x^2
            / (1 - b*x - 2*b^3*x^3 )
            );
}
check_equal(Vec(g(x)-O(x^#GJA_samples)), \
            vector(#GJA_samples-3,k,my(k=k+2); join_area_centroid(k)))


g(x) = {
         (31/82 + 49/82*I)
         + (-21/164 - 57/164*I)  /(1 - x)
         + (1/20 - 3/20*I) /(1 + x)
         + (-3/10 - 1/10*I)  /(1 - b*x)

         + (4/41 + 5/41*I)
         * (
              (5/2 - 2*I)
            * b*x
            / (1 - b*x - 2*b^3*x^3 )

            + -1/2*I
            * b^2*x^2
            / (1 - b*x - 2*b^3*x^3 )

            + (2 - 3/2*I)*2
            * b^3*x^3
            / (1 - b*x - 2*b^3*x^3 )
            );
}
check_equal(Vec(g(x)-O(x^#GJA_samples)), \
            vector(#GJA_samples-3,k,my(k=k+2); join_area_centroid(k)))



g(x) = {
         (31/82 + 49/82*I)
         + (-21/164 - 57/164*I)  /(1 - x)
         + (1/20 - 3/20*I) /(1 + x)
         + (-3/10 - 1/10*I)  /(1 - b*x)

         + (4/41 + 5/41*I)
         * (
              (5/2 - 2*I) * b*x
            + -1/2*I      * b^2*x^2
            + (2 - 3/2*I)*2  * b^3*x^3
            )
            / (1 - b*x - 2*b^3*x^3 )
;
}
check_equal(Vec(g(x)-O(x^#GJA_samples)), \
            vector(#GJA_samples-3,k,my(k=k+2); join_area_centroid(k)))

g(x) = {
         (31/82 + 49/82*I)
         + (-21/164 - 57/164*I)  /(1 - x)
         + (1/20 - 3/20*I) /(1 + x)
         + (-3/10 - 1/10*I)  /(1 - b*x)

         + (4/41 + 5/41*I)
         * (1-b*x)*(
              (5/2 - 2*I) * b*x
            + -1/2*I      * b^2*x^2
            + (2 - 3/2*I)*2  * b^3*x^3
            )
            / ((1-b*x)*(1 - b*x - 2*b^3*x^3 ))
;
}
check_equal(Vec(g(x)-O(x^#GJA_samples)), \
            vector(#GJA_samples-3,k,my(k=k+2); join_area_centroid(k)))

g(x) = {
         (31/82 + 49/82*I)
         + (-21/164 - 57/164*I)  /(1 - x)
         + (1/20 - 3/20*I) /(1 + x)
         + (-3/10 - 1/10*I)  /(1 - b*x)

         + (4/41 + 5/41*I)
         * ((16 - 12*I)*x^4
            + (-3 + 13*I)*x^3
            + (-3 - 5*I)*x^2
            + (9/2 + 1/2*I)*x)
            / ((1-b*x)*(1 - b*x - 2*b^3*x^3 ))
;
}
check_equal(Vec(g(x)-O(x^#GJA_samples)), \
            vector(#GJA_samples-3,k,my(k=k+2); join_area_centroid(k)))

g(x) = {
         (31/82 + 49/82*I)
         + (-21/164 - 57/164*I)  /(1 - x)
         + (1/20 - 3/20*I) /(1 + x)
         + (-3/10 - 1/10*I)  /(1 - b*x)

         + (16 - 12*I)/(2*b^4) * (4/41 + 5/41*I)

         + (4/41 + 5/41*I)
         * (

            - (16 - 12*I)/(2*b^4) * ((1-b*x)*(1 - b*x - 2*b^3*x^3 ))
            + (16 - 12*I)*x^4
            + (-3 + 13*I)*x^3
            + (-3 - 5*I)*x^2
            + (9/2 + 1/2*I)*x)
            / ((1-b*x)*(1 - b*x - 2*b^3*x^3 ))
;
}
check_equal(Vec(g(x)-O(x^#GJA_samples)), \
            vector(#GJA_samples-3,k,my(k=k+2); join_area_centroid(k)))


g(x) = {
         1/2*I
         + (-21/164 - 57/164*I)  /(1 - x)
         + (1/20 - 3/20*I) /(1 + x)
         + (-3/10 - 1/10*I)  /(1 - b*x)

         + (4/41 + 5/41*I)
         * (+ (2 - 3/2*I)
            + (-3/2 + I)  * b*x
            - 1/2         * b^2*x^2
            + 1/2*I       * b^3*x^3
           )
            / ((1-b*x)*(1 - b*x - 2*b^3*x^3 ))
;
}
check_equal(Vec(g(x)-O(x^#GJA_samples)), \
            vector(#GJA_samples-3,k,my(k=k+2); join_area_centroid(k)))


g(x) = {
         1/2*I
         + 1/164*(-21 - 57*I) /(1 - x)
         + 1/20 *(  1 - 3*I ) /(1 + x)
         + 1/10 *( -3 -   I ) /(1 - b*x)

         + 1/82
         * ((31 + 8*I) + (8 + 10*I)*x + (8 + 10*I)*x^2)
         / (1 - b*x - 2*b^3*x^3)
         ;
}
gGJAN = g;
check_equal(Vec(g(x)-O(x^#GJA_samples)), \
            vector(#GJA_samples-3,k,my(k=k+2); join_area_centroid(k)))
check_equal(gf_terms(g(x),100), \
            vector(100,k,my(k=k-1); join_area_centroid(k)))

\\ gJA(x) = -(1/2)/(1-x) + (1/2)/(1-x-2*x^3) );
\\ \\ gJA(x) = x^3 / ((1 - x - 2*x^3)*(1-x));
\\ gL(x) = -1 + (2 + 2*x^2) / (1 - x - 2*x^3);
\\ gR(x) = 1 - 2/(1-x) + 2*(1 + x + x^2)/(1-x-2*x^3);

check_exact(gJA(x), x^3 / ((1 - x - 2*x^3)*(1-x)))
check_exact(gJA(x), -(1/2)/(1-x) + (1/2)/(1-x-2*x^3))
gJA_cubic(x)          =  (1/2)/(1-x-2*x^3);
gJA_cubic_k_plus_1(x) =  (gJA_cubic(x) - 1/2)/x;
gJA_cubic_k_plus_2(x) =  (gJA_cubic_k_plus_1(x) - 1/2)/x;

check_exact(gJA_cubic_k_plus_1(x), (1/2     + x^2)/(1-x-2*x^3));
check_exact(gJA_cubic_k_plus_2(x), (1/2 + x + x^2)/(1-x-2*x^3));

{
check_exact(
matsolve(mattranspose([1/2, b*0, b^2*0;
                       1/2, b*0, b^2*1;
                       1/2, b*1, b^2*1]),
                      [(31 + 8*I); (8 + 10*I); (8 + 10*I) ]),
[57 + 20*I; -4 - 5*I; 9 + I]
)
}

{
check_exact(
              (57 + 20*I)*gJA_cubic(b*x)
            + (-4 -  5*I)*gJA_cubic_k_plus_1(b*x)
            + ( 9 +    I)*gJA_cubic_k_plus_2(b*x),

            ((31 + 8*I) + (8 + 10*I)*x + (8 + 10*I)*x^2)
            / (1 - b*x - 2*b^3*x^3)
)
}

gJA_k_plus_1(x) =  gJA(x)/x;
gJA_k_plus_2(x) =  gJA_k_plus_1(x)/x;

check_exact(gJA(x) + (1/2)/(1-x), gJA_cubic(x))
check_exact(gJA(b*x) + (1/2)/(1-b*x), gJA_cubic(b*x))

check_exact(gJA(b*x) + (1/2)/(1-b*x), gJA_cubic(b*x))
check_exact(gJA_k_plus_1(b*x) + (1/2)/(1-b*x), gJA_cubic_k_plus_1(b*x))
check_exact(gJA_k_plus_2(b*x) + (1/2)/(1-b*x), gJA_cubic_k_plus_2(b*x))

g(x) = {
         1/2*I
         + 1/164*(-21 - 57*I) /(1 - x)
         + 1/20 *(  1 - 3*I ) /(1 + x)
         + 1/10 *( -3 -   I ) /(1 - b*x)

         + 1/82
         * (
            (57 + 20*I)*(gJA(b*x)  + (1/2)/(1-b*x))
            + (-4 -  5*I)*(gJA_k_plus_1(b*x)  + (1/2)/(1-b*x))
            + ( 9 +    I)*(gJA_k_plus_2(b*x)   + (1/2)/(1-b*x))
           )
         ;
}
check_exact(g(x), gGJAN(x))
check_equal(Vec(g(x)-O(x^#GJA_samples)), \
            vector(#GJA_samples-3,k,my(k=k+2); join_area_centroid(k)))
check_equal(gf_terms(g(x),100), \
            vector(100,k,my(k=k-1); join_area_centroid(k)))


g(x) = {
         1/2*I
         + 1/164*(-21 - 57*I) /(1 - x)
         + 1/20 *(  1 - 3*I ) /(1 + x)
         + 1/410*( 32 - I   ) /(1 - b*x)
         + 1/82
         * (
              (57 + 20*I)*gJA(b*x)         
            + (-4 -  5*I)*gJA_k_plus_1(b*x)
            + ( 9 +    I)*gJA_k_plus_2(b*x)
           )
         ;
}
check_exact(g(x), gGJAN(x))

GJAN(k) = {
         if(k==0,1/2*I,0)
         + 1/164*(-21 - 57*I)
         + 1/20 *(  1 - 3*I )*(-1)^k
         + 1/410*( 32 - I   )*b^k
         + 1/82 * b^k
         * (
              (57 + 20*I)*JA(k)         
            + (-4 -  5*I)*JA(k+1)
            + ( 9 +    I)*JA(k+2)
           )
         ;
}
check_equal(gf_terms(g(x),100), \
            vector(100,k,my(k=k-1); GJAN(k)))


\\-----------------------------------------------------------------------------
\\ by LQ

GLQ_by_GJAN(k) = 1/LQ(k) * (GJAN(k) + JEQ(k) + b^(k+1)*JA(k+1) - GJAN(k+1));
vector(10,k,GLQ_by_GJAN(k))

\\ GLQN(k) = GJAN(k) + JEQ(k) + b^(k+1)*JA(k+1) - GJAN(k+1)
\\ GJAN(k+1) = GJAN(k) - GLQN(k) + JEQ(k) + b^(k+1)*JA(k+1)
\\ GJAN(k) = GJAN(k-1)  - GLQN(k-1) + JEQ(k-1) + b^k*JA(k)

GJAN_by_LQ(k) = if(k==0,return(GJAN(0))); \
                GJAN(k-1)  - GLQN(k-1) + JEQ(k-1) + b^k*JA(k);
check_equal(gf_terms(g(x),100), \
            vector(100,k,my(k=k-1); GJAN_by_LQ(k)))

GJAN_by_LQ(k) = if(k<1,return(GJAN(k))); \
                GJAN(1) + sum(j=1,k, JEQ(j-1) + b^j*JA(j) - GLQN(j-1) );
check_equal(gf_terms(g(x),100), \
            vector(100,k,my(k=k-1); GJAN_by_LQ(k)))

GJAN_by_LQ(k) = sum(j=0,k-1, JEQ(j) + b^(j+1)*JA(j+1) - GLQN(j) );
check_equal(gf_terms(g(x),100), \
            vector(100,k,my(k=k-1); GJAN_by_LQ(k)))

GJAN_by_LQ(k) = {
  sum(j=0,k-1, - LQ(j)*1/2*b^max(1,j))
  + sum(j=0,k-1, (3/5 + 1/5*I)*b^j - (1/10 - 3/10*I)*(-1)^j)
  + sum(j=0,k-1, b^(j+1)*JA(j+1));
}
check_equal(gf_terms(g(x),100), \
            vector(100,k,my(k=k-1); GJAN_by_LQ(k)))

GJAN_by_LQ(k) = {
  if(k==0,0,-1/2*I)
  + sum(j=0,k-1, - LQ(j)*1/2*b^j) 
  + sum(j=0,k-1, (3/5 + 1/5*I)*b^j - (1/10 - 3/10*I)*(-1)^j)
  + sum(j=0,k-1, b^(j+1)*JA(j+1));
}
check_equal(gf_terms(g(x),100), \
            vector(100,k,my(k=k-1); GJAN_by_LQ(k)))

GJAN_by_LQ(k) = {
  if(k==0,0,-1/2*I)
  + sum(j=0,k-1, (3/5 + 1/5*I)*b^j )
  - sum(j=0,k-1, (1/10 - 3/10*I)*(-1)^j )
  + sum(j=0,k-1, b^(j+1)*JA(j+1) )
  - 1/2* sum(j=0,k-1, LQ(j)*b^j)
  ;
}
check_equal(gf_terms(g(x),100), \
            vector(100,k,my(k=k-1); GJAN_by_LQ(k)))

GJAN_by_LQ(k) = {
  if(k==0,0,-1/2*I)
  + (3/5 + 1/5*I) * (b^k - 1)/(b-1)
  - (1/10 - 3/10*I) * ((-1)^k - 1)/(-1 -1)
  + sum(j=0,k-1, b^(j+1)*JA(j+1) )
  - 1/2* sum(j=0,k-1, (JA(j+1)+1+JA(j))*b^j)
  ;
}
check_equal(gf_terms(g(x),100), \
            vector(100,k,my(k=k-1); GJAN_by_LQ(k)))

GJAN_by_LQ(k) = {
  if(k==0,0,-1/2*I)
  + (-1/4 + 3/4*I)
  + (1/5 - 3/5*I) * b^k
  + (1/20 - 3/20*I) * (-1)^k
  + sum(j=0,k-1, b^(j+1)*JA(j+1) - 1/2* (JA(j+1)+1+JA(j))*b^j)
  ;
}
check_equal(gf_terms(g(x),100), \
            vector(100,k,my(k=k-1); GJAN_by_LQ(k)))

GJAN_by_LQ(k) = {
  if(k==0,0,-1/2*I)
  + (-1/4 + 3/4*I)
  + (1/5 - 3/5*I) * b^k
  + (1/20 - 3/20*I) * (-1)^k
  + sum(j=0,k-1, b^(j+1)*JA(j+1) - 1/2*JA(j+1)*b^j - 1/2*b^j - 1/2*JA(j)*b^j )
  ;
}
check_equal(gf_terms(g(x),100), \
            vector(100,k,my(k=k-1); GJAN_by_LQ(k)))

GJAN_by_LQ(k) = {
  if(k==0,0,-1/2*I)
  + (-1/4 + 1/4*I)
  + (1/5 - 1/10*I) * b^k
  + (1/20 - 3/20*I) * (-1)^k
  + sum(j=0,k-1, - 1/2*JA(j)*b^j + (1/2 + I)*JA(j+1)*b^j )
  ;
}
check_equal(gf_terms(g(x),100), \
            vector(100,k,my(k=k-1); GJAN_by_LQ(k)))


\\-----------------------------------------------------------------------------
\\ by LQ with generating function

gGLQN(x) = 1/2*I + 1/2*gLQ(b*x);
check_equal(gf_terms(gGLQN(x),100), \
            vector(100,k,my(k=k-1); GLQN(k)))

gJEQ(x) = (3/5 + 1/5*I)/(1-b*x) - (1/10 - 3/10*I)/(1+x);
gf_terms(gJEQ(x),100) == vector(100,k,my(k=k-1); JEQ(k)) || quit;

\\ GLQN(k) = GJAN(k) + JEQ(k) + b^(k+1)*JA(k+1) - GJAN(k+1)
\\ GJAN(k+1) = GJAN(k) - GLQN(k) + JEQ(k) + b^(k+1)*JA(k+1)
\\ GJAN(k) = GJAN(k-1)  - GLQN(k-1) + JEQ(k-1) + b^k*JA(k)
\\ gGJAN(x) = x*gGJAN(x)  - x*gGLQN(x) + JEQ(k-1) + b^k*JA(k)

gGJAN_by_LQ(x) = 1/(1-x) * ( - x*gGLQN(x) + x*gJEQ(x) + gJA(b*x) );
gGJAN_by_LQ(x)
gf_terms(g(x),10)
gf_terms(gGJAN_by_LQ(x),10)
check_equal(gf_terms(g(x),100), gf_terms(gGJAN_by_LQ(x),100))

gGJAN_by_LQ(x) = {
    1/2*I
     + (-21/164 - 57/164*I)/(1 - x)
     + (1/20 - 3/20*I)/(1 + x)
     + (-3/10 - 1/10*I)/(1 - (1 + I)*x)
     + ((31/82 + 4/41*I) + (4/41 + 5/41*I)*x + (4/41 + 5/41*I)*x^2)/(1 - (1 + I)*x + (4 - 4*I)*x^3)
}
check_exact(gGJAN_by_LQ(x), g(x))

\\-----------------------------------------------------------------------------
\\ cubic r limit

\\ cf GRf = 1/82*((70-15*I) + (3+14*I)*r - (17+11*I)*r^2);

r = 1.695620769559862057416367100;
GJAf = 1/82*((57 + 20*I) \
             + (-4 -  5*I)*r \
             + ( 9 +    I)*r^2)

1 - GJAf
GJAf*2
\\ 0.9279717691368970878548813042 + 0.1755734871507740065718553060*I


\\-----------------------------------------------------------------------------
\\ with GRQN

GRQN(k) = (1 - 1/2*I)  +  (1 + 1/2*I)* sum(i=0,k-1, LQ(i)*b^i);
\\ (GRQN(k) - (1 - 1/2*I)) =   (1 + 1/2*I)* sum(i=0,k-1, LQ(i)*b^i);
\\ sum(i=0,k-1, LQ(i)*b^i) = 1/(1 + 1/2*I) * (GRQN(k) - (1 - 1/2*I))
\\ sum(i=0,k-1, LQ(i)*b^i) = (-3/5 + 4/5*I) + (4/5 - 2/5*I)*GRQN(k)

GJAN_by_RQ(k) = {
  if(k==0,0,-1/2*I)
  + sum(j=0,k-1, (3/5 + 1/5*I)*b^j )
  - sum(j=0,k-1, (1/10 - 3/10*I)*(-1)^j )
  + sum(j=0,k-1, b^(j+1)*JA(j+1) )
  - 1/2* ((-3/5 + 4/5*I) + (4/5 - 2/5*I)*GRQN(k))
  ;
}
check_equal(gf_terms(g(x),100), \
            vector(100,k,my(k=k-1); GJAN_by_RQ(k)))

\\ gLQ(x) = (1 + x^2) / (1 - x - 2*x^3);
\\ gJA(x) = 1/2*( - 1/(1-x) + 1/(1-x-2*x^3) );
gLQ_k_plus_1(x) = (gLQ(x)-1)/x;
gLQ_k_plus_2(x) = (gLQ_k_plus_1(x)-1)/x;
check_exact(gLQ_k_plus_1(x), (1 +   x + 2*x^2)/(-2*x^3 - x + 1) )
check_exact(gLQ_k_plus_2(x), (2 + 2*x + 2*x^2)/(-2*x^3 - x + 1) )

m = {[1, 0, 1;
      1, 1, 2;
      2, 2, 2]};
check_exact(matsolve(mattranspose(m), [1;0;0]), [1; -1; 1/2])

check_exact(2*(gJA(x) + (1/2)/(1-x)), \
            (gLQ(x) - gLQ_k_plus_1(x) + 1/2*gLQ_k_plus_2(x)))
check_exact(gJA(x), \
            -(1/2)/(1-x) \
            + 1/2*gLQ(x) \
            - 1/2*gLQ_k_plus_1(x) \
            + 1/4*gLQ_k_plus_2(x) )

JA_by_LQ(k) = -1/2 + 1/2*LQ(k) - 1/2*LQ(k+1) + 1/4*LQ(k+2);
check_equal(vector(100,k,my(k=k-1); JA_by_LQ(k)), \
            vector(100,k,my(k=k-1); JA(k)))

gLQ_k_minus_1(x) = x*gLQ(x);
check_exact(gLQ_k_minus_1(x), -1/2 + (1/2 + 1/2*x)/(1 - x -2*x^3) )
m = {[1/2, 1/2, 0;
       1,   0,  1;
       1,   1,  2]};
matdet(m)
check_exact(matsolve(mattranspose(m), [1;0;0]), [1; 1; -1/2])

check_exact(2*(gJA(x) + (1/2)/(1-x)), \
            (gLQ_k_minus_1(x)+1/2 + gLQ(x) - 1/2*gLQ_k_plus_1(x)))
{
check_exact(gJA(x),
            (1/4
             - (1/2)/(1-x)
             + 1/2*gLQ_k_minus_1(x)
             + 1/2*gLQ(x)
             - 1/4*gLQ_k_plus_1(x)))
}

\\ JA by LQ -1,0,1
JA_by_LQ_minus(k) = if(k<0,error("JA_k_by_LQ_minux() is for k>=0")); \
  if(k==0,return(0)); \
  - 1/2 + 1/2*LQ(k-1) + 1/2*LQ(k) - 1/4*LQ(k+1);
\\ vector(10,k,my(k=k-1); JA_by_LQ_minus(k))
\\ vector(10,k,my(k=k-1); JA(k))
check_equal(vector(100,k,my(k=k-1); JA_by_LQ_minus(k)), \
            vector(100,k,my(k=k-1); JA(k)))
