\\ Copyright 2014, 2017 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file.  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.


read("../tools/memoize.gp");

b = 1+I;

L(k) = if(k<0,error("L() is for k>=0")); \
        if(k<=3,return(2^k)); \
        L(k-1) + 2*L(k-3);
L=memoize(L);
gL(x) = -1 + 2* (1 + x^2) / (1 - x - 2*x^3);

R(k)=1 + sum(j=0,k-1, L(j));
R = memoize(R);
gR(x)= 1 - 2/(1-x) + 2*(1 + x + x^2)/(1-x-2*x^3);

GLN(k) = \
    if(k==0, -1/2 + I, 0)               \
    + 1/25 * (2 - 6*I) * (-1)^k       \
    - 1/10 * (2 + I)     * conj(b)^k  \
    + 1/50 * (31 - 33*I) * b^k        \
    - 1/5  * (3 + I)     * k*b^k      \
    + 1/2  * L(k)        * b^k;

gGLN(x) =                                           \
  -1/2 + I                                          \
  + 1/25 * (2 - 6*I)   / (1 + x)                    \
  - 1/10 * (2 + I)     / (1 - conj(b)*x)            \
  + 1/50 * (61 - 23*I) / (1 - b*x)                  \
  - 1/5  * (3 + I)     / (1 - b*x)^2                \
  + 1/2 * gL(b*x);

gf_terms(g,n) = my(var=variable(g)); \
  concat(vector(min(n,valuation(g,var)), i, 0), \
         Vec(g + O(var^n)));

\\-----------------------------------------------------------------------------

nearly_equal(x,y) = abs(x-y) <= max(abs(x),max(abs(y),1))*1e-10;

differ(x,y) =
{
  \\ if(type(x)!=type(y),
  \\    return(Strprintf("different types %s %s",type(x),type(y))));
  if(type(x)=="t_COMPLEX" || type(y)=="t_COMPLEX",
     my(d=differ(real(x),real(y)));
     if(d != 0, return(Strprintf("real %s",d)));
     d=differ(imag(x),imag(y));
     if(d != 0, return(Strprintf("imag %s",d)));
     return(0));

  if(type(x)=="t_VEC" || type(x)=="t_COL",
     if(length(x)!=length(y),
        return(Strprintf("different lengths %s %s",length(x),length(y))));
     for(i=1,#x,
        my(d=differ(x[i],y[i]));
        if(d != 0, return(Strprintf("at index %d %s",i,d))));
     return(0));

  if(! nearly_equal(x,y),
     return(Strprintf("different values\n%s\n%s",x,y)));
  return(0);
}
check_count = 0;
check_equal(got,want) =
{
  check_count++;
  my(d = differ(got,want));
  if(d != 0,
     print("wrong\n",d); quit(1),
     print("ok"));
}

check_exact(got,want) =
{
  check_count++;
  if(got!= want,
     print("wrong\n");
     print("got  "got);
     print("want "want);
     quit(1),
     print("ok "check_count));
}

\\-----------------------------------------------------------------------------

GRtotal_samples = {1/2*[
  1 + (0)*I,
  3 + (1)*I,
  4 + (6)*I,
  -6 + (18)*I,
  -52 + (24)*I,
  -136 + (-40)*I,
  -184 + (-288)*I,
  148 + (-836)*I,
  1608 + (-1224)*I,
  4732 + (516)*I,
  7256 + (8856)*I
]};

GRtotal(k) = 1/2 + sum(j=0,k-1, L(j)*b^(j+1) - I*GLN(j));
check_exact(R(0), 1)
check_exact(GRtotal(0), 1/2)
check_exact(R(1), 2)
check_exact(GRtotal(1), 1/2 + 1+1/2*I)
check_exact(R(2), 4)
check_exact(GRtotal(2), 1/2 + 1+1/2*I + 1/2+I + 3/2*I)
check_exact(vector(length(GRtotal_samples),k,my(k=k-1); GRtotal(k)), GRtotal_samples)


GRtotal(k) = 1/2 + sum(j=0,k-1, b*L(j)*b^j - I*GLN(j));
check_exact(vector(length(GRtotal_samples),k,my(k=k-1); GRtotal(k)), GRtotal_samples)

gGRtotal(x) = (1/2)/(1-x) + x/(1-x) * (b*gL(b*x) - I*gGLN(x));
check_exact(gf_terms(gGRtotal(x), length(GRtotal_samples)), GRtotal_samples)
check_exact(gf_terms(gGRtotal(x), 100), vector(100,k,my(k=k-1); GRtotal(k)))

gGRtotal(x) =                                                                     \
  (57/41 - 21/41*I)/(1 - x)                                                   \
  + (3/25 + 1/25*I)/(1 + x)                                                   \
  + (-1/5 - 1/10*I)/(1 - (1 - I)*x)                                           \
  + (-101/50 + 43/50*I)/(1 - (1 + I)*x)                                       \
  + (3/5 + 1/5*I)/(1 - (2 + 2*I)*x + 2*I*x^2)                                 \
  + ((25/41 - 20/41*I) + (62/41 + 16/41*I)*x + (62/41 + 16/41*I)*x^2)        \
    / (1 - b*x - 2*b^3*x^3);
check_exact(gf_terms(gGRtotal(x), 100), vector(100,k,my(k=k-1); GRtotal(k)))

gGRtotal(x) =                                               \
  (57/41 - 21/41*I)/(1 - x)                             \
  + (3/25 + 1/25*I)/(1 + x)                             \
  + (-1/5 - 1/10*I)/(1 - conj(b)*x)                     \
  + (-101/50 + 43/50*I)/(1 - b*x)                       \
  + (3/5 + 1/5*I)/(1 - b*x)^2                           \
  + 1/41                                                \
    * ((25 - 20*I) + (62 + 16*I)*x + (62 + 16*I)*x^2) \
    / (1 - b*x - 2*b^3*x^3);
check_exact(gf_terms(gGRtotal(x), 100), vector(100,k,my(k=k-1); GRtotal(k)))

gR_cubic(x)          =  (2 + 2*x + 2*x^2)/(1-x-2*x^3);
gR_cubic_k_plus_1(x) =  (gR_cubic(x) - 2)/x;
check_exact(gR_cubic_k_plus_1(x), (4 + 2*x + 4*x^2)/(1-x-2*x^3));
gR_cubic_k_plus_2(x) =  (gR_cubic_k_plus_1(x) - 4)/x;
check_exact(gR_cubic_k_plus_2(x), (6 + 4*x + 8*x^2)/(1-x-2*x^3));

{
check_exact(                                                
matsolve(mattranspose([2, b*2, b^2*2;
                       4, b*2, b^2*4;
                       6, b*4, b^2*8]),  
                      [(25 - 20*I); (62 + 16*I); (62 + 16*I) ]),       
   1/2* [ 70 - 15*I;   3 + 14*I;     -17 - 11*I ]                      
)
}

{
check_exact(
    1/2* (   (70 - 15*I)*gR_cubic(b*x)
           + ( 3 + 14*I)*gR_cubic_k_plus_1(b*x)
           - (17 + 11*I)*gR_cubic_k_plus_2(b*x)
         )
    ,
    ((25 - 20*I) + (62 + 16*I)*x + (62 + 16*I)*x^2) \
    / (1 - b*x - 2*b^3*x^3)
)}

gGRtotal(x) =                                       \
  (57/41 - 21/41*I)/(1 - x)                     \
  + (3/25 + 1/25*I)/(1 + x)                     \
  + (-1/5 - 1/10*I)/(1 - conj(b)*x)             \
  + (-101/50 + 43/50*I)/(1 - b*x)               \
  + (3/5 + 1/5*I)/(1 - b*x)^2                   \
  + 1/82 *                                      \
    (   (70 - 15*I)*gR_cubic(b*x)               \
      + ( 3 + 14*I)*gR_cubic_k_plus_1(b*x)      \
      - (17 + 11*I)*gR_cubic_k_plus_2(b*x)      \
    );
check_exact(gf_terms(gGRtotal(x), 100), vector(100,k,my(k=k-1); GRtotal(k)))

gR_k_plus_1(x) =  (gR(x) - 1)/x;
check_exact(gR_k_plus_1(x), -2/(1 - x)  +  (4 + 2*x + 4*x^2)/(1 - x - 2*x^3))
gR_k_plus_2(x) =  (gR_k_plus_1(x) - 2)/x;
check_exact(gR_k_plus_2(x), -2/(1 - x) + (6 + 4*x + 8*x^2)/(1 - x - 2*x^3))

check_exact(gR(b*x) - gR_cubic(b*x), - (1 + b*x)/(1 - b*x))
check_exact(gR(b*x) - gR_cubic(b*x), 1 - 2/(1 - b*x))
check_exact(gR_k_plus_1(b*x) - gR_cubic_k_plus_1(b*x), -2/(1-b*x))
check_exact(gR_k_plus_2(b*x) - gR_cubic_k_plus_2(b*x), -2/(1-b*x))


{
gGRtotal(x) =
  (57/41 - 21/41*I)/(1 - x)
  + (3/25 + 1/25*I)/(1 + x)
  + (-1/5 - 1/10*I)/(1 - conj(b)*x)
  + (-101/50 + 43/50*I)/(1 - b*x)
  + (3/5 + 1/5*I)/(1 - b*x)^2
  + 1/82 *
    (   (70 - 15*I)*gR(b*x)
      + ( 3 + 14*I)*gR_k_plus_1(b*x)
      - (17 + 11*I)*gR_k_plus_2(b*x)
    )
  - 1/82 *
    (   (70 - 15*I)*(1 - 2/(1 - b*x))
      + ( 3 + 14*I)*(-2/(1-b*x))
      - (17 + 11*I)*(-2/(1-b*x))
    );
}
check_exact(gf_terms(gGRtotal(x), 100), vector(100,k,my(k=k-1); GRtotal(k)))

{
gGRtotal(x) =
  - 1/82 *(70 - 15*I)
  + 1/41   * (   57 - 21*I  )/(1 - x)
  + 1/25   * (    3 + I     )/(1 + x)
  + 1/10   * (   -2 - I     )/(1 - conj(b)*x)
  + 1/2050 * (-1341 + 1163*I)/(1 - b*x)
  + 1/5    * (    3 + I     )/(1 - b*x)^2
  + 1/82   *
    (   (70 - 15*I)*gR(b*x)
      + ( 3 + 14*I)*gR_k_plus_1(b*x)
      - (17 + 11*I)*gR_k_plus_2(b*x)
    )
  ;
}
check_exact(gf_terms(gGRtotal(x), 100), vector(100,k,my(k=k-1); GRtotal(k)))

check_exact(gf_terms(1/41   * (   57 - 21*I  )/(1 - x),   10), \
            vector(10,k,my(k=k-1);  1/41   * (   57 - 21*I  )))

{
GRtotal(k) =
  if(k==0, - 1/82 *(70 - 15*I), 0)
  + 1/41   * (   57 - 21*I  )
  + 1/25   * (    3 + I     )*(-1)^k
  + 1/10   * (   -2 - I     )*conj(b)^k
  + 1/2050 * (-1341 + 1163*I)*b^k
  + 1/5    * (    3 + I     )*(k+1)*b^k
  + 1/82 * b^k *
    (   (70 - 15*I)*R(k)
      + ( 3 + 14*I)*R(k+1)
      - (17 + 11*I)*R(k+2)
    )
  ;
}
check_exact(vector(length(GRtotal_samples),k,my(k=k-1); GRtotal(k)), GRtotal_samples)
check_exact(gf_terms(gGRtotal(x), 100), vector(100,k,my(k=k-1); GRtotal(k)))

{
GRtotal(k) =
  if(k==0, - 1/82 *(70 - 15*I), 0)
  + 1/41   * (   57 - 21*I  )
  + 1/25   * (    3 + I     )*(-1)^k
  + 1/10   * (   -2 - I     )*conj(b)^k
  + 1/2050 * (-111 + 1573*I)*b^k
  + 1/5    * (    3 + I     )*k*b^k
  + 1/82 * b^k *
    (   (70 - 15*I)*R(k)
      + ( 3 + 14*I)*R(k+1)
      - (17 + 11*I)*R(k+2)
    )
  ;
}
check_exact(vector(length(GRtotal_samples),k,my(k=k-1); GRtotal(k)), GRtotal_samples)
check_exact(gf_terms(gGRtotal(x), 100), vector(100,k,my(k=k-1); GRtotal(k)))
