\\ Copyright 2016, 2017 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file.  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.

default(recover,0);
read("dragon-defines.gp");

check_count = 0;
check_exact(got,want,name="") =
{
  check_count++;
  if(got!= want,
     print("wrong "name);
     print("got  "got);
     print("want "want);
     quit(1),
     print("ok "check_count" "name));
}

\\-----------------------------------------------------------------------------
\\ crossing lines around hull of middle segment

{
  segs_enclose = [
                   \\ anti-clockwise from -1-I
                   [ -1-I, I ],
                   [ -1-I, 1 ],
                   [ -2*I, I ],
                   [ -2*I, 1 ],
                   [ 1-I, -I ],
                   [ 1-I, 1 ],
                   [ 2, -I ],
                   [ 2, I ],
                   [ 1+I, 1 ],
                   [ 1+I, -1 ],
                   [ -1+I,  1 ],
                   [ -1+I, -I ]
                  ];
}

{
  segs_internal = [
                   [ 0, 1 ],
                   
                   \\ internals
                   [ 0, -1 ],
                   [ 0, I ],
                   [ 0, -I ],
                   [ 1+I, -I ],
                   [ 2, -1 ],
                   [ 1-I, I ],
                   [ 1-I, -1 ]
                  ];
}
segs = Set(concat([segs_enclose,segs_internal]));


\\-----------------------------------------------------------------------------
for(i=1,#segs,for(j=i+1,#segs,if(segs[i]==segs[j],print("duplicate ",segs[j]))));
check_exact(vector_unique(segs), segs, "uniqueness");

segs_print(segs) =
{
  my(m=concat([[0,1],
               apply(e->e[1],segs),      \\ segment starts
               apply(vecsum,segs) ]));   \\ segment ends
  my(y_max=vecmax(imag(m)));
  forstep(y=y_max,vecmin(imag(m)),-1,
     if(y<y_max,
       \\ above verticals
       print1("   ");
       for(x=vecmin(real(m)), vecmax(real(m)),
          if(vector_contains(segs,[x+I*y,I]),    print1("^     "),
             vector_contains(segs,[x+I*y+I,-I]), print1("|     "),
                                                 print1("      "));
          );
       print();

       print1("   ");
       for(x=vecmin(real(m)), vecmax(real(m)),
          if(vector_contains(segs,[x+I*y,I]),    print1("|     "),
             vector_contains(segs,[x+I*y+I,-I]), print1("|     "),
                                                 print1("      "));
          );
       print();

       print1("   ");
       for(x=vecmin(real(m)), vecmax(real(m)),
          if(vector_contains(segs,[x+I*y,I]),    print1("|     "),
             vector_contains(segs,[x+I*y+I,-I]), print1("v     "),
                                                 print1("      "));
          );
       print());

     \\ row
     print1("   ");
     for(x=vecmin(real(m)), vecmax(real(m)),
        print1(if(x==0&&y==0,"O","*"));
        if(vector_contains(segs,[x+I*y,1]),    print1(if(x==0&&y==0,
                                                         "====>",
                                                         "---->")),
           vector_contains(segs,[x+1+I*y,-1]), print1("<----"),
                                               print1("     "));
     );
     print();
  );
  print();
}

print("enclose");
segs_print(segs_enclose);
print("internal");
segs_print(segs_internal);
print("segs");
segs_print(segs);

expand_segments(v) = \
  concat(apply(e->my(z=e[1],d=e[2]); [ [b*z,d], [b*(z+d),d*-I] ], v));

check_exact(expand_segments([[0,1]]), [ [0,1], [1+I,-I] ]);
check_exact(expand_segments([[0,I]]), [ [0,I], [-1+I,1] ]);
check_exact(expand_segments([[0,-1]]), [ [0,-1], [-1-I,I] ]);
check_exact(expand_segments([[-1-I,1]]), [ [-2*I,1], [1-I,-I] ]);

check_exact(expand_segments([[-1+I,-I]]), [ [-2,-I], [-1-I,-1] ]);
check_exact(expand_segments([[-1+I,1]]), [ [-2,1], [-1+I,-I] ]);

print("number of segs ",#segs);

rotate1(v) = apply(e->my(z=e[1],d=e[2]); z-=1+I; [z*I,d*I], v);
expand1(v) = rotate1(expand_segments(v));
check_exact(expand1([[0,1]]), [ [1-I,I], [0,1] ]);
check_exact(expand1([[-1-I,I]]), [ [3-I,-1], [2-2*I,I] ]);

check_exact(setminus(Set(segs),Set(expand_segments(segs))), [], "expand0");
check_exact(setminus(Set(segs),Set(expand1(segs))),         [], "expand1");

\\ state 0 right side enc, state 1 left side enc
\\  0 ------> 1         1 ------> 0
\\   0\     /1           1\     /0
\\ 0a  \   /  1b       1a  \   /  0b
\\     1v v0               0v v1
segs_set = Set(segs);
transition(state,bit) =
{
  if(!(bit==0||bit==1),error("bit ",bit));
  if(state[1]==bit,
     \\ a  0,0 or 1,1
     concat([bit],
            setintersect(Set(expand_segments(state[^1])), segs_set)),

     \\ b  0,1 or 1,0
     concat([bit],
            setintersect(Set(expand1(state[^1])), segs_set)));
}
{
  states=List([
               Set([0, [0,1]]),   \\ side 0 right
               Set([1, [0,1]])    \\ side 1 left, with bits flipped
              ]);
  my(i=1);
  while(i<=#states,
        for(bit=0,1,
           my(new_state=transition(states[i],bit));
           if(!vector_contains(states,new_state),
              listput(states,new_state);
             \\ print("add ",#states);
             ));
       i++);
}
print("states ",#states);

check_exact(transition(states[1],0), [0, [0,1], [1+I,-I]], "transition 0");
check_exact(transition(states[1],1), [1, [0,1], [1-I,I]], "transition 1");

\\ check_exact(transition([0, [-1+I,-I]],0), [0, [-2,-I], [-1-I,-1]], "transition 0");
\\ check_exact(vector_contains(segs,[-2,-I]),1);
\\ check_exact(vector_contains(segs,[-1-I,-1]),1);

segs_enclose_set = Set(segs_enclose);
state_is_enclosed(state) = setminus(segs_enclose_set, state[^1]) == [];

{
  R_squares = [ [
                 [ 0, 1 ],       \\ below m
                 [ 0, -I ],
                 [ 1-I, -1 ],
                 [ 1-I, I ]
                ],
               [
                 [ -1+I, 1 ],       \\ top left
                 [ -1+I, -I ],
                 [ 0, -1 ],
                 [ 0, I ]
                ],
               [
                 [ 1+I, 1 ],       \\ top right
                 [ 1+I, -I ],
                 [ 2, -1 ],
                 [ 2, I ]
                ]
              ];
}
{
  L_squares = [ [
                 [ 0, 1 ],       \\ above m
                 [ 0, I ],
                 [ 1+I, -1 ],
                 [ 1+I, -I ]
                ],
               [
                 [ -1-I, 1 ],       \\ left of m
                 [ -1-I, I ],
                 [ 0, -1 ],
                 [ 0, -I ]
                ],
               [
                 [ 1-I, 1 ],       \\ right of m
                 [ 1-I, I ],
                 [ 2, -1 ],
                 [ 2, -I ]
                ],
               [
                 [ -2*I, 1 ],       \\ below m
                 [ -2*I, I ],
                 [ 1-I, -1 ],
                 [ 1-I, -I ]
                ]
              ];
}
R_squares=apply(s->Set(s),R_squares);  \\ segments sort
L_squares=apply(s->Set(s),L_squares);

R_squares
apply(segs_print,R_squares);

L_squares
apply(segs_print,L_squares);
               
state_is_RLpred(state,side) =
{
  my(squares=if(side==state[1],R_squares,L_squares));
  for(i=1,#squares,
     my(u=#setminus(squares[i],state[^1]));
     if(u==1||u==2||u==3,return(1)));
  0;
}
state_is_Lpred(state) =
{
  for(i=1,#L_squares,
     my(u=#setminus(L_squares[i],state[^1]));
     if(u==1||u==2||u==3,return(1)));
}

print("Rpred ",select(i->state_is_RLpred(states[i],0), vector(#states,i,i)));
print("Lpred ",select(i->state_is_RLpred(states[i],1), vector(#states,i,i)));

print("non-Rpred ",select(i->!state_is_RLpred(states[i],0), vector(#states,i,i)));
print("non-Lpred ",select(i->!state_is_RLpred(states[i],1), vector(#states,i,i)));

state_to_index(state) =
{
  for(i=1,#states,if(state==states[i],return(i)));
  error("oops, state not found: ",state);
}
state_print(state) =
{
  print("state=",state);
  print("side ",state[1], "  index=",state_to_index(state),
        " R ",state_is_Rpred(state),
        " L ",state_is_Lpred(state));
  segs_print(state[^1]);
  print("----------------");
}

print("----------------------------------------------------------------------");
print("non-RLpred should remain non-RLpred");
{
  my(bad=0);
  for(i=1,#states,
     my(state=states[i]);
     for(side=0,1,
        if(!state_is_RLpred(state,side),
           for(bit=0,1,
              my(new_state=transition(state,bit));
              if(state_is_RLpred(new_state,side),
                 print("from");
                 state_print(state);
                 print("to bit="bit);
                 state_print(new_state);
                 print("non-"side" became side "side" again\n\n\n\n");
                 bad++)))));
  if(bad,print(bad," bad");
     quit(1);
    );
}
print("ok");

print("----------------------------------------------------------------------");
print("eventually non-RLpred");

{
  eventually_non_RLpred_states
  = [Set(select(state->!state_is_RLpred(state,0),states)),
     Set(select(state->!state_is_RLpred(state,1),states))];
}
{
  for(side=0,1,
     my(more=1);
     while(more,
           more=0;
           for(i=1,#states,
              my(state=states[i]);
              if(setsearch(eventually_non_RLpred_states[side+1],state),
                 next()); \\ already
              if(setsearch(eventually_non_RLpred_states[side+1],
                           transition(state,0))
                 && setsearch(eventually_non_RLpred_states[side+1],
                              transition(state,1)),
                 eventually_non_RLpred_states[side+1]
                 = setunion(eventually_non_RLpred_states[side+1],
                            [state]); \\ new
             more=1))));
}
{
  for(side=0,1,
     print("eventually non-"side);
     \\print(eventually_states);
     print(apply(state_to_index,eventually_non_RLpred_states[side+1]));
     print("count ",#eventually_non_RLpred_states[side+1]);
     print("leaving ",#states-#eventually_non_RLpred_states[side+1]);
  );
}

\\ print("EVENTUALLY BEGIN");
\\ apply(state_print,eventually_states);
\\ print("EVENTUALLY END");

print("---------------------------------------------------------------------");
print("table");
{
  for(i=1,#states,
     print("  "i"=>{0=>" state_to_index(transition(states[i],0)),
           ",1=>", state_to_index(transition(states[i],1)), "},"));
}
print("Rpred accepting");
for(i=1,#states,if(!state_is_RLpred(states[i],0),print(i"=>'Rnon',")));
\\for(i=1,#states,if(state_is_RLpred(states[i],0),print(i"=>'Rpred',")));
for(i=1,#states,if(!state_is_RLpred(states[i],1),print(i"=>'Lnon',")));


\\ print("runs");
\\ {
\\   my(state=states[1]);
\\   for(i=1,10,
\\      state_print(state);
\\      state=transition(state,0);
\\ 
\\      state_print(state);
\\      state=transition(state,0);
\\ 
\\      state_print(state);
\\      print("now 1");
\\      state=transition(state,1);
\\ 
\\      );
\\ }

transitions(state,v) = for(i=1,#v,state=transition(state,v[i])); state;

vector_bit_flip(v) = apply(n->1-n,v);
fracper_bit_flip(v) = \
  concat(vector_bit_flip(v[^-1]), [vector_bit_flip(v[#v])]);
check_exact(fracper_bit_flip([1,1,0, [0,1,1,0]]), [0,0,1, [1,0,0,1]]);

transitions_of_fracper(state,fv,side) =
{
  for(i=1,#fv-1, \\ fixed part
     my(bit=fv[i]);
     state=transition(state,bit);
     if(state_is_RLpred(state,side),return(state)));
  fv=fv[#fv];  \\ periodic part
  my(seen=[]);
  while(1,
        for(i=1,#fv, \\ fixed part
           my(bit=fv[i]);
           state=transition(state,bit);
           if(state_is_RLpred(state,side),return(state)));
        seen=setunion(seen,[state]);
        state=transitions(state,fv);
        if(setsearch(seen,state),return(state)));
}

fRpred_by_states(f) =
{
  my(fv=frac_to_fracper(f,2));
  my(state=transitions_of_fracper(states[1],fv,0));
  state_is_RLpred(state,0);
}
fLpred_by_states(f) =
{
  my(fv=frac_to_fracper(f,2));
  fv=fracper_bit_flip(fv);
  my(state=transitions_of_fracper(states[2],fv,1));
  state_is_RLpred(state,1);
}

fInPred_by_states(f) = !fRpred_by_states(f) && !fLpred_by_states(f);

show_transitions_of_fracper(state,fv) =
{
  print("initial, fixed part ",fv[^1]);
  state_print(state);
  for(i=1,#fv-1, \\ fixed part
     my(bit=fv[i]);
     state=transition(state,bit);
     print("bit "bit" to");
     state_print(state));

  seen=[];
  fv=fv[#fv];  \\ periodic part
  print(" periodic part ",fv);
  while(1,
        seen=setunion(seen,[state]);
        for(i=1,#fv,
           my(bit=fv[i]);
           state=transition(state,bit);
           print("bit "bit" to");
           state_print(state));
        if(setsearch(seen,state),
           print("previously seen");
           break());
        print(" periodic again");
       );
  print("final R: ",state_is_Rpred(state));
}

\\-----------------------------------------------------------------------------

Rpred_transitions_of_fracper(state,fv) =
{
  my(state=Rpred_transitions(state,fv[^-1]));  \\ fixed part
  my(seen=[]);
  fv=fv[#fv];  \\ periodic part
  while(1,
        seen=setunion(seen,[state]);
        state=Rpred_transitions(state,fv);
        if(setsearch(seen,state),break()));
  state;
}
fRpred_by_RL_of_fracper(fv) = Rpred_transitions_of_fracper(state,fv)!=Rpred_non;
fRpred_by_RL(f) =
{
  my(state=Rpred_transitions_of_fracper(Rpred_1e,frac_to_fracper(f,2)));
  state!=Rpred_non;
}
fLpred_by_RL(f) =
{
  my(state=Rpred_transitions_of_fracper(Rpred_1o,fracper_bit_flip(frac_to_fracper(f,2))));
  state!=Rpred_non;
}
fInPred_by_RL(f) = !fRpred_by_RL(f) && !fLpred_by_RL(f);

Rpred_state_name=["1e", "1o", "2eA", "3oA", "3oB", "2eB_3oC", "3eA", "3eB", "3eC", "non"];
show_Rpred_transitions_of_fracper(state,fv) =
{
  print("at "state" "Rpred_state_name[state]);
  for(i=1,#fv-1,
     my(bit=fv[i]);
     state=Rpred_transition(state,bit);
     print("bit "bit" to "state" "Rpred_state_name[state]));
  my(seen=[]);
  fv=fv[#fv];  \\ periodic part
  print("periodic");
  while(1,
        seen=setunion(seen,[state]);
        for(i=1,#fv,
           my(bit=fv[i]);
           state=Rpred_transition(state,bit);
           print("bit "bit" to "state" "Rpred_state_name[state]));
        if(setsearch(seen,state),break()));
  state;
}


\\-----------------------------------------------------------------------------
\\ RL by cases

\\ n = num mod 30
\\ n+1 after
\\ n-11 before is n+19 pred at 2*n+30
\\
\\ 15*(2*n+1)-7 == 30*n+8
\\ 15*(2*n+1)+7 == 30*n+22

fRpred_by_cases(f) =
{
  my(den=denominator(f));
  if(den%15==0,
     my(pow=den/15);
     if(is_power_of_2(pow),
        my(k=logint(pow,2));
        if(den!=15*2^k,error());
  
        my(num=numerator(f));
        my(n=num\30,r=num%30);
        my(fback=(2*n + r/15)/2^k);
        if(f!=fback,error("oops"));

        \\ if(r==13,  return(Spred(k,2*n) && (n==0 || turn(2*n)==1)));

        \\ no 11, but 3 present to make right side
        if(r==1,  return(Spred(k,2*n) && (n==0 || turn(2*n)==1)));
        
        \\ no 1, and no 3 to make left side
        if(r==11, return(Spred(k,2*n) && (n==0 || turn(2*n)==1)));
        
        if(r==29,  return(Spred(k,2*n+2) && turn(2*n+2)==1));
        if(r==19,  return(Spred(k,2*n+2) && turn(2*n+2)==1));
  

        \\ if(r==19, return(Spred(k,2*n+2)));  \\ 2n-11
  
        if(r==7, return(Rpred(2*n)));
        if(r==23, return(Lpred(k,2*n-1))); 
        \\ if(r==7, return(Spred(k,2*n+1) && turn(2*n+1)==1));  \\ no 13
       ));

  my(fv=frac_to_fracper(f,2));
  if(fv[#fv]==[],fv[#fv]=[0]);
  my(state=Rpred_transitions_of_fracper(Rpred_1e,fv));
  state!=Rpred_non;
}
fLpred_by_cases(f) =
{
  \\ my(den=denominator(f));
  \\ if(den%15==0,
  \\    my(pow=den/15);
  \\    if(is_power_of_2(pow),
  \\       my(k=logint(pow,2));
  \\       if(den!=15*2^k,error());
  \\ 
  \\       my(num=numerator(f));
  \\       my(n=num\30,r=num%30);
  \\       my(fback=(2*n + r/15)/2^k);
  \\       if(f!=fback,error("oops"));
  \\ 
  \\       \\ print("fLpred at f="f" 2*n="2*n" r="r);
  \\ 
  \\       if(r==29, return(Spred(k,2*n+2)));
  \\       if(r==11, return(Spred(k,2*n)));
  \\ 
  \\      \\ if(r==7, return(Spred(k,2*n+1) && turn(2*n+1)==-1));
  \\      ));

  my(fv=frac_to_fracper(f,2));
  if(fv[#fv]==[],fv[#fv]=[0]);
  fv=fracper_bit_flip(fv);
  my(state=Rpred_transitions_of_fracper(Rpred_1o,fv));
  state!=Rpred_non;
}
fInPred_by_cases(f) = !fRpred_by_cases(f) && !fLpred_by_cases(f);

\\ check_exact(fRpred_by_cases(7/15),1, "Rpred_by_cases 7/15");
\\ check_exact(fLpred_by_cases(7/15),0, "Lpred_by_cases 7/15");
\\ 
\\ check_exact(fRpred_by_cases(13/15),0, "Rpred_by_cases 13/15");
\\ check_exact(fLpred_by_cases(13/15),1, "Lpred_by_cases 13/15");

\\-----------------------------------------------------------------------------
print("fifteenths");
{
  for(i=1,14,print(i, " Rpred ",fRpred_by_states(i/15)," t=",fRpred_by_cases(i/15),
                  "  Lpred ",fLpred_by_states(i/15)," t=",fLpred_by_cases(i/15)));
}
print("30ths");
{
  for(i=1,29,print(i, " Rpred ",fRpred_by_states(i/30)," t=",fRpred_by_cases(i/30),
                  "  Lpred ",fLpred_by_states(i/30)," t=",fLpred_by_cases(i/30)));
}
\\ 1/2 = 15/30 wrong Lpred

\\ f=1/2
\\ fv=frac_to_fracper(f,2)
\\ if(fv[#fv]==[],fv[#fv]=[0]); fv
\\ fv=fracper_bit_flip(fv)
\\ show_Rpred_transitions_of_fracper(Rpred_1o,v)
\\ quit

print("KM ",fInPred_by_states(KM_sum(8)));
frac_to_fracper(KM_sum(6),2)

print("sevenths");
frac_to_fracper(8/15,2)
{
  for(i=1,6,print(i, " Rpred ",fRpred_by_states(i/7)," t=",fRpred_by_cases(i/7),
                  "  Lpred ",fLpred_by_states(i/7)," t=",fLpred_by_cases(i/7)));
}

\\ show_transitions_of_fracper(states[1],frac_to_fracper(6/7,2));



\\-----------------------------------------------------------------------------
print("15ths");

{
  my(pred_by_states=fRpred_by_states);
  my(pred_by_cases=fRpred_by_cases);
  my(m=15);
  my(bad=0);
  for(k=0,10,
     my(den=m*2^k);
     for(num=1,den-1,
        if(gcd(num,den)!=1,next());
        my(f=num/den);
        my(o=num % (2*m));
        if(o>m,o-=2*m);
        my(s=pred_by_cases(f));
        my(t=pred_by_states(f));
        if(s!=t,
           my(z=dragon_fpoint(f));
           print("m="m" "num"/"den" o="o
                 " cases="s" states="t"  at "z);
           bad++;
           if(bad>12,quit))));
}
quit

f=147/224; print("f="f);
frac_to_fracper(f,2)
show_Rpred_transitions_of_fracper(Rpred_1e,frac_to_fracper(f,2))
show_Rpred_transitions_of_fracper(Rpred_1e,fracper_bit_flip(frac_to_fracper(f,2)))
{
  for(k=0,10,
     my(n=floor(f*2^k));
     print("Lpred "n" of "2^k" = "Lpred(k,n));
  );
}

state_print(fInPred_final_state(f))

\\ k=10;
\\ floor(3/7*2^k)
\\ ceil(3/7*2^k)
\\ dragon_point(floor(3/7*2^k))
\\ dragon_point(ceil(3/7*2^k))

\\ state_is_enclosed_eventually(state)


\\-----------------------------------------------------------------------------
\\ within k bits

\\ {
\\   for(k=1,3,
\\      for(n=0,2^k-1,
\\         my(v=binary_padded(n,k));
\\         my(s=fInPred_v_by_cases(v));
\\         my(t=fInPred_v_by_states(v));
\\         if(s!=t,
\\            print(v,"  ",s," ",t))));
\\ }
\\ fracper_to_frac([1,[1,0]],2)

