#!/usr/bin/perl -w

# Copyright 2016, 2017, 2018, 2019 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use Math::PlanePath::ComplexMinus 117; # v.117 for level_to_n_range()

use FindBin;
use lib $FindBin::Bin;
use lib "$FindBin::Bin/../tools";
use MyLineSegments;
use MyPrintwrap ':all';

# uncomment this to run the ### lines
# use Smart::Comments;


my $level = 8;
print "  % generated by tools/$FindBin::Script\n";
print "  \\newcommand{\\MyLevel}{$level}\n";

my $path = Math::PlanePath::ComplexMinus->new;
my ($n_lo, $n_hi) = $path->level_to_n_range($level);


my $n_segments = MyLineSegments->new (name => 'n');
my $conj_segments = MyLineSegments->new (name => 'rot');
my $both_segments = MyLineSegments->new (name => 'both');

sub conj {
  my ($x,$y) = @_;
  return ($x, -$y);
}
sub unconj {
  my ($x,$y) = @_;
  return ($x, -$y);
}

sub xy_to_type {
  my ($x,$y) = @_;
  my $n   = $path->xy_to_n($x,$y);
  my ($rx,$ry) = unconj($x,$y);
  my $rot = $path->xy_to_n($rx,$ry);
  ### xy_to_type(): "$x,$y $n   $rx,$y $rot"
  if ($n <= $n_hi) {
    return ($rot <= $n_hi ? 'both' : 'n');
  } else {
    return ($rot <= $n_hi ? 'rot' : '');
  }
}

sub boundary_to_segs {
  my ($a,$b) = @_;
  if ($a eq 'both') {
    if ($b eq 'both') { return; }                  # hollow
    else              { return $both_segments; }   # boundary
  }
  if ($a eq 'n') {
    if ($b eq 'both') { return; }
    return $n_segments;
  }
  if ($a eq 'rot') {
    if ($b eq 'n')    { return; }                  # n--rot as "n"
    if ($b eq 'both') { return; }
    return $conj_segments;
  }
  die "boundary_to_segs() a=$a b=$b";
}

my %both;

foreach my $n ($n_lo .. $n_hi) {
  ### $n
  my ($x,$y) = $path->n_to_xy($n);
  foreach my $rot (0,1) {
    my $a = xy_to_type($x,$y);
    if ($a eq 'both') {
      $both{"$x,$y"} = 1;
    }
    ### at: "$x,$y  a=$a (rot=$rot)"
    foreach my $segments (boundary_to_segs($a, xy_to_type($x,$y-1))) {
      $segments->add(2*$x-1,2*$y-1, 2*$x+1,2*$y-1);   # S
    }
    foreach my $segments (boundary_to_segs($a, xy_to_type($x,$y+1))) {
      $segments->add(2*$x-1,2*$y+1, 2*$x+1,2*$y+1);   # N
    }
    foreach my $segments (boundary_to_segs($a, xy_to_type($x-1,$y))) {
      $segments->add(2*$x-1,2*$y-1, 2*$x-1,2*$y+1);   # W
    }
    foreach my $segments (boundary_to_segs($a, xy_to_type($x+1,$y))) {
      $segments->add(2*$x+1,2*$y-1, 2*$x+1,2*$y+1);   # E
    }
    ($x,$y) = conj($x,$y);
  }
}

$Printwrap = 2;
printwrap("\\fill[my intersect] ");
$Printwrap += 2;
$both_segments->printwrap;
printwrap(";\n");

$Printwrap = 2;
printwrap("\\draw[my squares line] ");
$Printwrap += 2;
$n_segments->printwrap;
printwrap(";\n");

$Printwrap = 2;
printwrap("\\draw[my conj line] ");
$Printwrap += 2;
$conj_segments->printwrap;
printwrap(";\n");

my $MinusConjA = scalar keys %both;
print "% GP-Test  MinusConjA($level) == $MinusConjA\n";
print "  \\newcommand{\\MyMinusConjA}{$MinusConjA}\n";
print "  % end generated\n";
exit 0;
