#!/usr/bin/perl -w

# Copyright 2021 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  See the file COPYING.  If not, see
# <http://www.gnu.org/licenses/>.

use 5.010;
use strict;
use warnings;
use FindBin;
use File::Slurp;
use List::Util 'max','min','sum';
use IPC::Run;
use Math::OEIS;
use Math::OEIS::Names;
$|=1;

# uncomment this to run the ### lines
# use Smart::Comments;

# https://oeis.org/wiki/CiteR

my $source_dir = File::Spec->catdir ($FindBin::Bin, File::Spec->updir);
my $dragon_tex_filename = File::Spec->catfile($source_dir,'dragon.tex');
my $dragon_tex_str = File::Slurp::read_file($dragon_tex_filename);
print "$dragon_tex_filename ",count_lines($dragon_tex_str)," lines\n";

my $dragon_tex_str_sans_comments = $dragon_tex_str;
$dragon_tex_str_sans_comments =~ s/((^|[^\\])(\\\\)*)%[^\n]*/$1/mg;

my %anums;
while ($dragon_tex_str_sans_comments
       =~ /My[a-zA-Z]*Anumber.*?(?<anum>A\d{6,})/sg) {
  $anums{$+{'anum'}}++;
}
my $total = sum(0, values %anums);
my $distinct = keys %anums;
print "A-number tags $total, distinct $distinct\n";

my $index_filename = Math::OEIS->local_filename('Index/CiteR.html');
my $index_str = File::Slurp::read_file($index_filename);
print "$index_filename ",count_lines($index_str)," lines\n";

$index_str =~ /<li> Kevin Ryde, Iterations of the Dragon Curve.*/ or die;
my $index_line = $&;
my %index_anums;
while ($index_line =~ /A\d{6,7}/g) {
  $index_anums{$&} = 1;
}

foreach my $anum (sort keys %anums) {
  if (! exists $index_anums{$anum}) {
    my $name = Math::OEIS::Names->anum_to_name($anum) // '[unknown]';
    print "$anum new  $name\n";
  }
}
foreach my $anum (sort keys %index_anums) {
  if (! exists $anums{$anum}) {
    my $name = Math::OEIS::Names->anum_to_name($anum) // '[unknown]';
    print "$anum delete  $name\n";
  }
}

print "\n";
print join(', ', sort keys %anums),"\n";
exit 0;

#------------------------------------------------------------------------------

# Return the line number of character position $pos in $str.
# If $pos is undef or not supplied, use pos($str).
sub pos_linenum {
  my ($str, $pos) = @_;
  $pos //= pos($_[0]);
  $str = substr($str, 0, $pos);
  return 1 + scalar($str =~ tr/\n//);
}

sub count_lines {
  my ($str) = @_;
  my $count = ($str =~ tr/\n/\n/);
  if ($str =~ /[^\n]$/) { $count++; }
  return $count;
}

#------------------------------------------------------------------------------
