#!/usr/bin/perl -w

# Copyright 2020 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.

use 5.010;
use strict;
use warnings;
use Math::BaseCnv 'cnv';

use FindBin;
use lib::abs "$FindBin::Bin/../tools";
use MyFLAT;
$|=1;

# uncomment this to run the ### lines
# use Smart::Comments;


# Franz-Josef Brandenburg, "Uniformly Growing k-th Power-Free Homomorphisms",
# Theoretical Computer Science, volume 23, 1983
#
# power 3/2 is reps length >= 3/2*original
# strict power 3/2 is reps length > 3/2*original
# ab power 3/2 = aba
# ab strict power 3/2 = abab
#
# w is k'th power-free if w does not have the k'th power as a substring
# w is weakly k'th power-free if w does not have the k'th power as a strict substring
#
# weakly k=2 overlap-free, no x w x w x for block w and single symbol x

{
  # compact
  my @v = ('');
  for my $len (0 .. 1000) {
    print $len," ",scalar(@v),"\n";
    @v = grep {!/^(.)(.*?)\1\2\1/} map {("0$_","1$_")} @v; }
  exit 0;
}

{
  # binary weakly squarefrees
  my @strs = ('');
  my @got;
  foreach my $len (1 .. 100) {
    @strs = map { my $old = $_;
                  map { my $new = $_.$old;
                        str_has_initial_overlap($new) ? () : ($new) } '0','1'
                      } @strs;
    # if ($len <= 5) {
    #   print "count ",scalar(@strs)," strings ",join(', ',sort @strs),"\n";
    # }
    print scalar(@strs),",";
    push @got, scalar(@strs);
  }
  # print join(',',@got),"\n";
  print "\n";
  exit 0;
}

sub str_has_initial_overlap {
  my ($str) = @_;
  return ($str =~ /^((.)(.*?))\1\2/);
}

# OEIS_samples("A007777")
# 1, 2, 4, 6, 10, 14, 20, 24, 30, 36, 44, 48, 60, 60, 62, 72, 82, 88, 96
# cf
#    2  4  6  10  14  20  24  30  36  44  48  60  60  72  82  88  96
# later
# 228,248,272,284,296,300
# x w x w x

# GP-DEFINE  Brandenburg = {[
# GP-DEFINE    2,4,6,10,14,20,24,30,36,44, 48,60,60,72,82,88,96,112,120,
# GP-DEFINE    120,136,148,164,152,154,148,162,176,190, 196,210,216,224,
# GP-DEFINE    228,248,272,284,296,300, 296,320,332,356,356,376,400,416,
# GP-DEFINE    380,382, 376,382,356,374,392,410,432,458,464,486, 476,498,
# GP-DEFINE    500,522,528,540,548,568,560,592, 592,620,660,688,688,722,
# GP-DEFINE    724,740,724,724, 716,748,788,824,816,856,868,880,868,912,
# GP-DEFINE    908,960,976,1008,1000
# GP-DEFINE  ]};
# GP-Test  #Brandenburg == 94
# GP-DEFINE  Brandenburg_inserted = {
# GP-DEFINE    concat([Brandenburg[1..13],
# GP-DEFINE           62,
# GP-DEFINE           Brandenburg[14..#Brandenburg]]);
# GP-DEFINE  }
# GP-Test  #Brandenburg_inserted == 95
# GP-Test  OEIS_samples("A007777")[10..20] == \
# GP-Test    [36, 44, 48, 60, 60, 62, 72, 82, 88, 96, 112]
# GP-DEFINE  my(v=OEIS_samples("A007777")); \
# GP-DEFINE  A007777(n) = v[n+1];
# GP-Test  my(v=OEIS_samples("A007777")); \
# GP-Test    vector(#v,n,n--; A007777(n)) == v  \\ OFFSET=0
# GP-Test  A007777(0) == 1
# GP-Test  A007777(1) == 2
# GP-Test  A007777(14) == 62


# v=OEIS_samples("A007777")[^1]
# Brandenburg[1..#v]-v
# Brandenburg_inserted[1..#v] == v

# Brandenburg_inserted == [2,4,6,10,14,20,24,30,36,44,48,60,60,62,72,82,88,96,112,120,120,136,148,164,152,154,148,162,176,190,196,210,216,224,228,248,272,284,296,300,296,320,332,356,356,376,400,416,380,382,376,382,356,374,392,410,432,458,464,486,476,498,500,522,528,540,548,568,560,592,592,620,660,688,688,722,724,740,724,724,716,748,788,824,816,856,868,880,868,912,908,960,976,1008,1000]
