// Copyright 2015 Kevin Ryde
//
// This file is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 3, or (at your option) any later
// version.
//
// This file is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
// or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
// for more details.
//
// You should have received a copy of the GNU General Public License along
// with this file.  See the file COPYING.  If not, see
// <http://www.gnu.org/licenses/>.


// This code draws by recursing to draw a dragon as two level-1
// dragons directed inwards.  The middle is half length along and half
// length perpendicular to the right.
//
//   x1,y1              x2,y2
//      --_          _--
//          -      -
//            mx,my
//
// Each line segment is a little rectangular polygon which is then
// extruded in the Z axis to 3D.
//
// Explicit coordinates are used for "from" and "to" locations.  An
// alternative would be multmatrix() to make shrunken sub-curves but
// that seems a lot slower in a full "--render".  3D transformed cubes
// or explicit polyhedrons seems slower than extruding too.
//

level = 8;
linewidth = .1;
$fn = 20;

module dragon(level, x1,y1, x2,y2) {
    if (level == 0) {
        dx = (x2-x1) * linewidth/2;
        dy = (y2-y1) * linewidth/2;
        polygon([ [x1-dx-dy, y1-dy+dx], [x1-dx+dy, y1-dy-dx],
                  [x2+dx+dy, y2+dy-dx], [x2+dx-dy, y2+dy+dx] ]);
    } else {
        mx = (x1 + x2 - y1 + y2)/2;
        my = (x1 - x2 + y1 + y2)/2;
        dragon(level-1, x1,y1, mx,my);
        dragon(level-1, x2,y2, mx,my);
   }
}


sphere(2*linewidth);  // highlight the start of the curve

linear_extrude(center=true,height=linewidth)
dragon(level, 0,0, pow(2,level/2),0);


// Local variables:
// compile-command: "openscad --imgsize=1000,800 -o /tmp/dragon-extruded.png dragon-extruded.scad && xzgv /tmp/dragon-extruded.png"
// End: