\\ Copyright 2014, 2015, 2017 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file.  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.


check_equal(x,y) =
{
  if(x != y, \
     print("bad"); \
     print("x= ",x,"\ny= ",y); \
     quit(1));
}

b = 1+I;
b_real_sign_vector = [1,1,0,-1, -1,-1,0,1];
b_real_sign(k) = b_real_sign_vector[(k%8)+1];
b_real(k) = b_real_sign(k) * 2^floor(k/2);
b_imag_sign_vector = [0,1,1,1, 0,-1,-1,-1];
b_imag_sign(k) = b_imag_sign_vector[(k%8)+1];
b_imag(k) = b_imag_sign(k) * 2^floor(k/2);
check_equal(vector(64,k,my(k=k-1); real(b^k)), \
            vector(64,k,my(k=k-1);  b_real(k)));
check_equal(vector(64,k,my(k=k-1); imag(b^k)), \
            vector(64,k,my(k=k-1);  b_imag(k)));
b_conj_sign_vector = b_real_sign_vector - I*b_imag_sign_vector
\\ [1, 0, -1, -2, -1, 0, 1, 2]



p_vector = [ -1, -b, -b^2, -b^3-3, 1, b, b^2, b^3+3 ];
p(m) = m%=8; p_vector[m+1];

P1(k)  = - (b^(k+3) + p(k+3))/3;
P2(k)  = - (b^(k+2) + p(k+2))/3;
P3(k)  = - (b^(k+1) + p(k+1))/3;
P4(k)  = - (b^(k+0) + p(k+0))/3;

rectangle_area_at_angle(v,z) = {
  v=v/z;
  abs(z^2)
  * (vecmin(apply(real,v)) - vecmax(apply(real,v)))
  * (vecmin(apply(imag,v)) - vecmax(apply(imag,v)));
}

T_P1(k) = P1(k+1);
T_P2(k) = P2(k+1);
T_P3(k) = P3(k+1);
T_P4(k) = P4(k+1);
T_P1o(k) = b^(k+1) - P1(k+1);
T_P2o(k) = b^(k+1) - P2(k+1);
T_P3o(k) = b^(k+1) - P3(k+1);
T_P4o(k) = b^(k+1) - P4(k+1);

twindragon_hull_vertices(k) = {
  if(k==0, return([ T_P1(k), 
                    T_P3(k),
                    T_P1o(k),
                    T_P3o(k) ]));
  if(k==1, return([ T_P1(k), 
                    T_P2(k),
                    T_P4(k),
                    T_P1o(k),
                    T_P2o(k),
                    T_P4o(k) ]));
  if(k==2, return([ T_P1(k), 
                    T_P2(k),
                    T_P3(k),
                    T_P1o(k),
                    T_P2o(k),
                    T_P3o(k) ]));
  [ T_P1(k),
    T_P2(k),
    T_P3(k),
    T_P4(k),
    T_P1o(k),
    T_P2o(k),
    T_P3o(k),
    T_P4o(k) ];
}
twindragon_hull_vertices(0) == [1, 0, I, 1+I] || error("oops")

TMRvert_ref(k) = rectangle_area_at_angle(twindragon_hull_vertices(k), b^(k+1));
TMRdiag_ref(k) = rectangle_area_at_angle(twindragon_hull_vertices(k), b^(k+2));
TMRvert_ref(0) == 2 || error("oops")
TMRdiag_ref(0) == 1 || error("oops")

\\-----------------------------------------------------------------------------

check() = my(bad=0); \
          for(k=0,100, \
              my(want = TMRvert_ref(k)); \
              my(got  = TMRvert(k)); \
              if(got != want, \
                 bad+=1;print("k=",k," want ",want," got ",got); \
                 if(bad>10,break))); \
          if(bad!=0,error("bad")); \
          print("ok");

TMRvert(k) = 2^(k+1) \
    * real( (b^(k+1)-P3(k+1) - P3(k+1)) / b^(k+1) ) \
    * imag( (b^(k+1)-P1(k+1) - P1(k+1)) / b^(k+1) );
check()

TMRvert(k) = 2^(k+1) \
    * real( (b^(k+1) + 2*(b^(k+2) + p(k+2)) /3) / b^(k+1) ) \
    * imag( (b^(k+1) + 2*(b^(k+4) + p(k+4)) /3) / b^(k+1) );
check()

TMRvert(k) = 2^(k+1) \
    * real( 5/3 + 2/3*I   + 2/3*p(k+2) / b^(k+1) ) \
    * imag( 1 + 4/3*I     + 2/3*p(k+4) / b^(k+1) );
check()

TMRvert(k) = 2^(k+1) \
    * ( 5/3 + real( 2/3*p(k+2) * conj(b)^(k+1) / 2^(k+1) )) \
    * ( 4/3 + imag( 2/3*p(k+4) * conj(b)^(k+1) / 2^(k+1) ));
check()

TMRvert(k) = 2^(k+1) \
    * ( 5/3 + real( 2/3*p(k+2) * 2^floor((k+1)/2) * (b_real_sign(k+1) - I*b_imag_sign(k+1)) / 2^(k+1) )) \
    * ( 4/3 + imag( 2/3*p(k+4) * 2^floor((k+1)/2) * (b_real_sign(k+1) - I*b_imag_sign(k+1)) / 2^(k+1) ));
check()

TMRvert(k) = 2^(k+1) \
    * ( 5/3 + real( 2/3*p(k+2) * 1/2^ceil((k+1)/2) * (b_real_sign(k+1) - I*b_imag_sign(k+1)) )) \
    * ( 4/3 + imag( 2/3*p(k+4) * 1/2^ceil((k+1)/2) * (b_real_sign(k+1) - I*b_imag_sign(k+1)) ));
check()

TMRvert(k) = 2^(k+1) \
    * ( 5/3 + 2/3*1/2^ceil((k+1)/2) * real( p(k+2) * ([1, 1 - I, -I, -1 - I, -1, -1 + I, I, 1 + I][(k+1)%8+1]) )) \
    * ( 4/3 + 2/3*1/2^ceil((k+1)/2) * imag( p(k+4) * ([1, 1 - I, -I, -1 - I, -1, -1 + I, I, 1 + I][(k+1)%8+1]) ));
check()

TMRvert(k) = 2^(k+1) \
    * ( 5/3 + 2/3*1/2^ceil((k+1)/2) * real( [ -1, -b, -b^2, -b^3-3, 1, b, b^2, b^3+3 ][(k+2)%8+1] * ([1, 1 - I, -I, -1 - I, -1, -1 + I, I, 1 + I][(k+1)%8+1]) )) \
    * ( 4/3 + 2/3*1/2^ceil((k+1)/2) * imag( [ -1, -b, -b^2, -b^3-3, 1, b, b^2, b^3+3 ][(k+4)%8+1] * ([1, 1 - I, -I, -1 - I, -1, -1 + I, I, 1 + I][(k+1)%8+1]) ));
check()

TMRvert(k) = 2^(k+1) \
    * ( 5/3 + 2/3*1/2^ceil((k+1)/2) * real( [-b^2, -b^3-3, 1, b, b^2, b^3+3, -1, -b  ][(k)%8+1] * ([1 - I, -I, -1 - I, -1, -1 + I, I, 1 + I, 1][(k)%8+1]) )) \
    * ( 4/3 + 2/3*1/2^ceil((k+1)/2) * imag( [ 1, b, b^2, b^3+3, -1, -b, -b^2, -b^3-3 ][(k)%8+1] * ([1 - I, -I, -1 - I, -1, -1 + I, I, 1 + I, 1][(k)%8+1]) ));
check()

vector(8,k,my(k=k-1); real( [-b^2, -b^3-3, 1, b, b^2, b^3+3, -1, -b  ][(k)%8+1] * ([1 - I, -I, -1 - I, -1, -1 + I, I, 1 + I, 1][(k)%8+1]) ))
vector(8,k,my(k=k-1); imag( [ 1, b, b^2, b^3+3, -1, -b, -b^2, -b^3-3 ][(k)%8+1] * ([1 - I, -I, -1 - I, -1, -1 + I, I, 1 + I, 1][(k)%8+1]) ))
TMRvert(k) = 2^(k+1) \
    * ( 5/3 + 2/3*1/2^ceil((k+1)/2) * [-2, -2, -1, -1, -2, -2, -1, -1][(k)%8+1] ) \
    * ( 4/3 + 2/3*1/2^ceil((k+1)/2) * [-1, -1, -2, -2, -1, -1, -2, -2][(k)%8+1] );
check()

TMRvert(k) = 2^(k+1) \
    * (   4/3*5/3  \
       + 4/3*2/3*1/2^ceil((k+1)/2) * [-2, -2, -1, -1, -2, -2, -1, -1][(k)%8+1] \
       + 5/3*2/3*1/2^ceil((k+1)/2) * [-1, -1, -2, -2, -1, -1, -2, -2][(k)%8+1] \
       + 2/3*1/2^ceil((k+1)/2) * [-1, -1, -2, -2, -1, -1, -2, -2][(k)%8+1]*2/3*1/2^ceil((k+1)/2) * [-2, -2, -1, -1, -2, -2, -1, -1][(k)%8+1] );
check()

vector(8,k,my(k=k-1); 2/3*[-1, -1, -2, -2, -1, -1, -2, -2][(k)%8+1]*2/3*[-2, -2, -1, -1, -2, -2, -1, -1][(k)%8+1])
TMRvert(k) = 2^(k+1) \
    * (   20/9  \
       + 1/2^ceil((k+1)/2) * [-26/9, -26/9, -28/9, -28/9, -26/9, -26/9, -28/9, -28/9][(k)%8+1] \
       + 1/2^(k+1) * [8/9*1/2, 8/9, 8/9*1/2, 8/9, 8/9*1/2, 8/9, 8/9*1/2, 8/9][(k)%8+1] );
check()

TMRvert(k) = 40/9*2^k  \
            + 2^ceil(k/2) * [-26/9, -26/9, -28/9, -28/9][(k)%4+1] \
            + [4/9, 8/9][(k)%2+1];
check()

\\-----------------------------------------------------------------------------

check() = my(bad=0); \
          for(k=0,100, \
              my(want = TMRdiag_ref(k)); \
              my(got  = TMRdiag(k)); \
              if(got != want, \
                 bad+=1;print("k=",k," want ",want," got ",got); \
                 if(bad>10,break))); \
          if(bad!=0,error("bad")); \
          print("ok");

TMRdiag(k) = 2^(k+2) \
    * real( (b^(k+1)-P3(k+1) - P3(k+1)) / b^(k+2) ) \
    * imag( (b^(k+1)-P1(k+1) - P1(k+1)) / b^(k+2) );
check()

TMRdiag(k) = 2^(k+2) \
    * real( (b^(k+1) + 2*(b^(k+2) + p(k+2)) /3) / b^(k+2) ) \
    * imag( (b^(k+1) + 2*(b^(k+4) + p(k+4)) /3) / b^(k+2) );
check()

TMRdiag(k) = 2^(k+2) \
    * real( (b^(k+1) + 2/3*b^(k+2) + 2/3*p(k+2)) / b^(k+2) ) \
    * imag( (b^(k+1) + 2/3*b^(k+4) + 2/3*p(k+4)) / b^(k+2) );
check()

TMRdiag(k) = 2^(k+2) \
    * real( 1/b + 2/3     + 2/3*p(k+2) / b^(k+2) ) \
    * imag( 1/b + 2/3*b^2 + 2/3*p(k+4) / b^(k+2) );
check()

TMRdiag(k) = 2^(k+2) \
    * real( 7/6 - 1/2*I + 2/3*p(k+2) / b^(k+2) ) \
    * imag( 1/2 + 5/6*I + 2/3*p(k+4) / b^(k+2) );
check()

TMRdiag(k) = 2^(k+2) \
    * (7/6 + real( 2/3*p(k+2) / b^(k+2) )) \
    * (5/6 + imag( 2/3*p(k+4) / b^(k+2) ));
check()

TMRdiag(k) = 2^(k+2) \
    * (7/6 + real( 2/3*p(k+2) / 2^floor((k+2)/2) / (b_real_sign(k+2) + I*b_imag_sign(k+2)) )) \
    * (5/6 + imag( 2/3*p(k+4) / 2^floor((k+2)/2) / (b_real_sign(k+2) + I*b_imag_sign(k+2)) ));
check()


TMRdiag(k) = 2^(k+2) \
    * (7/6 +  1/2^floor((k+2)/2) * real( 2/3*p(k+2) / (b_real_sign(k+2) + I*b_imag_sign(k+2)) )) \
    * (5/6 + 1/2^floor((k+2)/2) * imag( 2/3*p(k+4) / (b_real_sign(k+2) + I*b_imag_sign(k+2)) ));
check()

vector(8,k,my(k=k-1); real( 2/3*p(k+2) / (b_real_sign(k+2) + I*b_imag_sign(k+2)) ))
vector(8,k,my(k=k-1); imag( 2/3*p(k+4) / (b_real_sign(k+2) + I*b_imag_sign(k+2)) ))
TMRdiag(k) = 2^(k+2) \
    * (7/6 +  1/2^floor((k+2)/2) * [-4/3, -1/3, -2/3, -2/3, -4/3, -1/3, -2/3, -2/3][(k%8)+1] ) \
    * (5/6 +  1/2^floor((k+2)/2) * [-2/3, -2/3, -4/3, -1/3, -2/3, -2/3, -4/3, -1/3][(k%8)+1]);
check()

TMRdiag(k) = 2^(k+2) * ( \
      7/6 *5/6 \
    + 5/6 *1/2^floor((k+2)/2) * [-4/3, -1/3, -2/3, -2/3, -4/3, -1/3, -2/3, -2/3][(k%8)+1] \
    + 7/6 *1/2^floor((k+2)/2) * [-2/3, -2/3, -4/3, -1/3, -2/3, -2/3, -4/3, -1/3][(k%8)+1] \
    + 1/2^floor((k+2)/2) * [-2/3, -2/3, -4/3, -1/3, -2/3, -2/3, -4/3, -1/3][(k%8)+1]*1/2^floor((k+2)/2) * [-4/3, -1/3, -2/3, -2/3, -4/3, -1/3, -2/3, -2/3][(k%8)+1]  \
    );
check()

vector(8,k,my(k=k-1);  [-2/3, -2/3, -4/3, -1/3, -2/3, -2/3, -4/3, -1/3][(k%8)+1] * [-4/3, -1/3, -2/3, -2/3, -4/3, -1/3, -2/3, -2/3][(k%8)+1] )
TMRdiag(k) = 2^(k+2) * ( \
      7/6 *5/6 \
    + 1/2^floor((k+2)/2) * [-17/9, -19/18, -19/9, -17/18, -17/9, -19/18, -19/9, -17/18][(k%8)+1] \
    + 1/2^floor((k+2)/2)*1/2^floor((k+2)/2) * [8/9, 2/9, 8/9, 2/9, 8/9, 2/9, 8/9, 2/9][(k%8)+1]  \
    );
check()

TMRdiag(k) = 35/36*2^(k+2) \
    + 2^ceil((k+2)/2) * [-17/9, -19/18, -19/9, -17/18, -17/9, -19/18, -19/9, -17/18][(k%8)+1] \
    + 2^(k+2) *1/2^floor((k+2)/2)*1/2^floor((k+2)/2) * [8/9, 2/9, 8/9, 2/9, 8/9, 2/9, 8/9, 2/9][(k%8)+1];
check()

TMRdiag(k) = 35/9*2^k \
    - 2^ceil((k)/2) * [34/9, 19/9, 38/9, 17/9, 34/9, 19/9, 38/9, 17/9][(k%8)+1] \
    + [8/9, 4/9][(k%2)+1];
check()

TMRdiag(k) = 1/9 * (35*2^k \
    - 2^ceil((k)/2) * [34, 19, 38, 17, 34, 19, 38, 17][(k%8)+1] \
    + [8, 4][(k%2)+1]);
check()

TMR_samples = [ 1, 4, 8, 24, 48, 108, 216, 468, 936, 1924, 3848, 7844 ]
