; Copyright 2019 Kevin Ryde
;
; This file is free software; you can redistribute it and/or modify it
; under the terms of the GNU General Public License as published by the Free
; Software Foundation; either version 3, or (at your option) any later
; version.
;
; This file is distributed in the hope that it will be useful, but
; WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
; or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
; for more details.
;
; You should have received a copy of the GNU General Public License along
; with this file.  See the file COPYING.  If not, see
; <http://www.gnu.org/licenses/>.


; Usage: xfractint type=lsystem lfile=dragon.l lname=DragonR params=8
;
; Or interactively, the usual key "t", choose type lsystem, "F6"
; files, "F6" again the current directory, choose dragon.l, etc.
;
; "lname" is one of the rule specifications below.  Interactively, "t"
; and choose type lsystem (again) goes to the available L-systems in
; the current file.
;
; "params=8" is the expansion level.  Interactively, key "z" changes
; that alone.


;-----------------------------------------------------------------------------

; These R,L,T,U,V per "Lindenmayer System" in my write-up.

DragonR {      ; dragon curve right boundary
  Angle 4
  Axiom RF
  R = RF+L
  L = TF-
  T = RF+UF-
  U = VF-F-F+
  V = TF-F+V
}

DragonB {      ; dragon curve whole boundary
  Angle 4
  Axiom RF++LF
  R = RF+L
  L = TF-
  T = RF+UF-
  U = VF-F-F+
  V = TF-F+V
}

TwindragonB {        ; twindragon curve whole boundary
  Angle 4            ; "order" is twindragon k=0 unit square, etc
  Axiom RF+LF+RF+LF
  R = RF+L
  L = TF-
  T = RF+UF-
  U = VF-F-F+
  V = TF-F+V
}

TwindragonBback {      ; twindragon curve whole boundary, by back-to-back
  Angle 4              ; "order" is each of the two back-to-backs
  Axiom RF+RF          ; should have order>=1
  R = RF+L
  L = TF-
  T = RF+UF-
  U = VF-F-F+
  V = TF-F+V
}


;-----------------------------------------------------------------------------
; Right boundary by segment expansions of Rpred and Rsides.

; q = 1e
; r = 1o
; s = 2eA
; t = 2eB

; u = 3oA
; v = 3oB
; w = 3oC      ; same as t

; x = 3eA
; y = 3eB
; z = 3eC

; L and R boundaries
; Axiom q++r
;
; Twindragon
; Axiom q+q

DragonRbySegments {      ; dragon curve right boundary, by segments
  Angle 4
  Axiom q
  F =
  q = Fq+Fr    ; 1e
  r = Fs-Ft    ; 1o
  s = Fq+Fu    ; 2eA
  t = Fv-Ft    ; 2eB
  u = Fx-Fy    ; 3oA
  v = Fz+Fu    ; 3oB
  x = Fs       ; 3eA
  y = +        ; 3eB
  z = Ft       ; 3eC
}

;;
;; R3 {
;;   Angle 4
;; ;  Axiom q
;;
;; ; L and R boundaries
;; ;  Axiom q++r
;;
;; ; Twindragon
;;   Axiom q+q
;;
;;
;;   F=
;;   q=Fq+Fr   ; 1e
;;   r=Fs-Ft
;;   s=Fq+Fu   ; 2eA
;;   t=Fv-Fw
;;   u=Fx-Fy   ; 3oA
;;   v=Fz+Fu
;;   w=Fv-Fw
;;   x=Fs      ; 3eA
;;   y=+
;;   z=Ft
;;   }

; Local variables:
; compile-command: "xfractint type=lsystem lfile=dragon.l lname=DragonR params=8"
; End:
