#!/usr/bin/perl -w

# Copyright 2016, 2017, 2019 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.

use 5.004;
use strict;
use Math::BigRat;

use FindBin;
use lib $FindBin::Bin;
use MyDragon 'div';
use MyPrintwrap ':all';

# uncomment this to run the ### lines
# use Smart::Comments;


my $level = 10;

print "  % generated by tools/$FindBin::Script\n";

my @points;
{
  my $x = Math::BigRat->new('1/3');
  my $y = Math::BigRat->new('-1/3');
  my $dx = Math::BigRat->new('1/6');
  my $dy = Math::BigRat->new(0);
  foreach (1 .. $level+1) {
    push @points, "$x,$y";
    $x += $dx;
    $y += $dy;
    ($dx,$dy) = div($dx,$dy); $dx/=2; $dy/=2;
  }
  push @points, '1/2,-1/2';
}
{
  my @end_points = ('2/3,0');
  my $x = Math::BigRat->new('2/3');
  my $y = Math::BigRat->new('-1/2');
  my $dx = Math::BigRat->new('-1/12');
  my $dy = Math::BigRat->new('-1/12');
  foreach (1 .. $level-1) {
    push @end_points, "$x,$y";
    $x += $dx;
    $y += $dy;
    ($dx,$dy) = div($dx,$dy); $dx/=2; $dy/=2;
  }
  push @points, reverse @end_points;
}

$Printwrap = 2;
printwrap("\\draw ");
$Printwrap += 2;
my $join = '';
foreach my $p (@points) {
  printwrap("$join($p)");
  $join = '--';
}
printwrap(";\n");

{
  $Printwrap = 2;
  printwrap("\\draw[my dotted] ($points[0])--(2/3,-1/3)");
  $Printwrap += 2;
  my $length = 1/6;  # first is 1/6 horizontal
  foreach my $i (2 .. $level) {
    my $from = $points[$i];
    my $to = $points[-$i];
    if ($length < .005) {
      ### dotted stop at: "i=$i length=$length"
      last;
    }
    printwrap(" ($from)--($to)");
    $join = '--';
    $length /= 2;
  }
  printwrap(";\n");
}

print "  % end generated\n";
exit 0;
