#!/usr/bin/perl -w

# Copyright 2015, 2016, 2017, 2018, 2019 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.005;
use strict;
use Math::PlanePath::DragonCurve;

use FindBin;
use lib $FindBin::Bin;
use MyDragon 'xy_rotate_plus90','xy_rotate_minus90','rect_corners_sort',
  'BlobN','BridgeRother','Lpred_k','Rpred','JN';
use MyLineSegments;
use MyPrintwrap ':all';

# uncomment this to run the ### lines
# use Smart::Comments;


my $level = 8;

print "  % generated by tools/$FindBin::Script\n";
my $path = Math::PlanePath::DragonCurve->new;
my (undef,$n_hi) = $path->level_to_n_range($level);
my $n_lo = BlobN($level) - 3;
### $n_lo
### $n_hi

print "  \\newcommand{\\MyLevel}{$level}\n";
print "  \\newcommand{\\MyLevelSubTwo}{",$level-2,"}\n";
print "  \\newcommand{\\MyLevelSubThree}{",$level-3,"}\n";

my $JN = JN($level);
my $JotherBlob_n = BridgeRother($level+1) - 2*$n_hi;
### $JotherBlob_n

# unless (Lpred_k($level,$BlobN) && Rpred($level,$BlobN)) {
#   die "oops, not a bridge";
# }
# if (Lpred_k($level,$BlobN+1) && Rpred($level,$BlobN+1)) {
#   die "oops, next is a bridge too";
# }

my ($x_hi,$y_hi) = $path->n_to_xy(2*$n_hi);
sub transform_other {
  my ($x,$y) = @_;
  ($x,$y) = xy_rotate_minus90($x,$y);
  return ($x_hi+$x, $y_hi+$y);
}

{
  my ($x,$y) = transform_other($path->n_to_xy($n_lo));
  print "  \\coordinate (start) at ($x,$y);\n";
}
{
  my ($x,$y) = transform_other($path->n_to_xy($n_hi));
  print "  \\node (end) at ($x,$y) [my dot];\n";
}

my %seen;  # boundary squares
my %squares = ( LQ => [], JA => []);

my $segments = MyLineSegments->new;
foreach my $n ($n_lo .. $n_hi-1) {
  ### $n
  my ($x,$y) = $path->n_to_xy($n);
  my ($dx,$dy) = $path->n_to_dxdy($n);
  my ($x2,$y2) = ($x+$dx, $y+$dy);
  $segments->add(transform_other($x,$y), transform_other($x2,$y2));

  if (Lpred_k($level,$n) && $n >= $JN) {
    my ($lx,$ly) = xy_rotate_plus90($dx,$dy);
    my ($xc,$yc) = rect_corners_sort ($x,$y, $x2+$lx,$y2+$ly);
    if (! $seen{"$xc,$yc"}++) {
      my $type = ($n < $JotherBlob_n ? 'LQ' : 'JA');
      push @{$squares{$type}}, "$xc,$yc";
    }
  }
}

{
  $Printwrap = 2;
  printwrap("\\draw ");
  $Printwrap += 2;
  $segments->printwrap;
  printwrap(";\n");
}

foreach my $side ('LQ','JA') {
  my $aref = $squares{$side};
  $Printwrap = 2;
  printwrap("\\fill[my ${side} colour] foreach \\p in {");
  $Printwrap += 2;
  foreach my $xy (@$aref) {
    printwrap("($xy),");
  }
  printwrap("}\n");
  printwrap("{ \\p +(\\MyMargin,\\MyMargin) rectangle +(1-\\MyMargin,1-\\MyMargin) };\n");
}

my $OtherJAblobRQ = scalar(@{$squares{'LQ'}});
my $PreBlob  = scalar(@{$squares{'JA'}});
print "  \\newcommand{\\MyPreBlobLQ}{$PreBlob}\n";
print "  \\newcommand{\\OtherJAblobRQ}{$OtherJAblobRQ}\n";
print "% GP-Test  JA($level) == $OtherJAblobRQ + $PreBlob\n";
print "% GP-Test  BlobRQ($level) - JAblobRQ($level+2) - 1 == $OtherJAblobRQ\n";
print "  % end generated\n";
exit 0;
