#!/usr/bin/perl -w

# Copyright 2016, 2017, 2019 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use List::Util 'min';
use Math::PlanePath::DragonCurve 117; # v.117 for level_to_n_range()

use FindBin;
use lib $FindBin::Bin;
use MyDragon 'xy_rotate_plus90','xy_rotate_minus90','BlobStart_xy','BlobEnd_xy',
  'transform_to_horizontal';
use MyLineSegments;
use MyPrintwrap ':all';


my $level = 12;

print "  % generated by tools/$FindBin::Script\n";
my $path = Math::PlanePath::DragonCurve->new;

my ($n_lo, $n_hi) = $path->level_to_n_range($level);
my ($x_hi,$y_hi) = $path->n_to_xy($n_hi);
my ($x_join,$y_join) = $path->n_to_xy($n_hi/2);

sub transform {
  my ($x,$y) = @_;
  return transform_to_horizontal($x,$y, $level);
}

my $x_middle = $x_hi/2;
my $y_middle = $y_hi/2;

sub blob_crossing_line {
  my ($sx,$sy, $tx,$ty) = @_;
  foreach my $rotate (0, 1) {
    if ($rotate) {
      $sx = $x_hi - $sx;
      $sy = $y_hi - $sy;
      $tx = $x_hi - $tx;
      $ty = $y_hi - $ty;
    }
    my ($sx,$sy) = transform($sx,$sy);
    my ($tx,$ty) = transform($tx,$ty);
    my $len = ($sx-$tx)**2 + ($sy-$ty)**2;
    my $style = ($len <= 2 ? "my zero line" : "my line");
    if ($rotate) { $style .= " thin"; }
    printwrap(" \\draw[$style] ($sx,$sy)--($tx,$ty);");
  }
}

foreach my $rotate (0, 1) {
  $Printwrap = 2;

  # start blobs
  foreach my $k (4 .. $level-1) {
    my ($sx,$sy) = BlobStart_xy($k);
    my ($tx,$ty) = BlobEnd_xy($k);
    blob_crossing_line ($sx,$sy, $tx,$ty);

    # same for second sub-curve
    ($sx,$sy) = xy_rotate_minus90($sx,$sy);
    ($tx,$ty) = xy_rotate_minus90($tx,$ty);
    $sx += $x_hi; $sy += $y_hi;
    $tx += $x_hi; $ty += $y_hi;
    blob_crossing_line ($sx,$sy, $tx,$ty);
  }

  # end blobs, back from join
  foreach my $k (4 .. $level-3) {
    my ($sx,$sy) = BlobStart_xy($k);
    my ($tx,$ty) = BlobEnd_xy($k);
    ($sx,$sy) = xy_rotate_minus90($sx,$sy);
    ($tx,$ty) = xy_rotate_minus90($tx,$ty);
    $sx += $x_join;
    $sy += $y_join;
    $tx += $x_join;
    $ty += $y_join;
    blob_crossing_line ($sx,$sy, $tx,$ty);

    # same for second sub-curve
    ($sx,$sy) = xy_rotate_minus90($sx,$sy);
    ($tx,$ty) = xy_rotate_minus90($tx,$ty);
    $sx += $x_hi; $sy += $y_hi;
    $tx += $x_hi; $ty += $y_hi;
    blob_crossing_line ($sx,$sy, $tx,$ty);
  }
  printwrap_newline();
}

{
  my ($x,$y) = transform(0,0);
  print "  \\node (start) at ($x,$y) [my dot];\n";
}
{
  my ($x,$y) = transform($x_hi/2,$y_hi/2);
  print "  \\node (middle) at ($x,$y) [my dot];\n";
}
{
  my ($x,$y) = transform($x_hi,$y_hi);
  print "  \\node (end) at ($x,$y) [my dot];\n";
}

print "  % end generated\n";
exit 0;
