#!/usr/bin/perl -w

# Copyright 2014, 2015, 2016, 2017, 2018 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use Math::PlanePath::ComplexPlus 117; # v.117 for level_to_n_range()

use FindBin;
use lib $FindBin::Bin;
use MyDragon 'TAVertexToUpper','TAVertexToLower',
  'TAVertexToLeft','TAVertexToRight';
use MyPrintwrap ':all';

# uncomment this to run the ### lines
# use Smart::Comments;


my $level = 5;

print "  % generated by tools/$FindBin::Script\n";
print "  \\newcommand{\\MyLevel}{$level}\n";

my $path = Math::PlanePath::ComplexPlus->new;
my ($n_lo, $n_hi) = $path->level_to_n_range($level);

my @solid = (1,2);
push @solid, map {$_+4} @solid;
push @solid, map {$_+8} @solid;
push @solid, map {$_+16} @solid;
### @solid

my %solid;
@solid{@solid} = ();  # hash slice
### %solid

$Printwrap = 2;
foreach my $n ($n_lo .. $n_hi) {
  my ($x,$y) = $path->n_to_xy($n);
  my $n2 = sprintf '%0*b', $level, $n;
  my $solid = exists $solid{$n} ? ' solid' : '';
  printwrap(" \\node (p$n) at ($x,$y) [my$solid dot];");
}
printwrap_newline();

my $num_edges = 0;
$Printwrap = 2;
printwrap("\\draw");
$Printwrap += 2;
foreach my $from ($n_lo .. $n_hi) {
  foreach my $to (TAVertexToLeft($from),
                  TAVertexToRight($from),
                  TAVertexToLower($from),
                  TAVertexToUpper($from)) {
    if (defined $to
        && $to > $from
        && $to <= $n_hi) {
      printwrap(" (p$from)--(p$to)");
      $num_edges++;
    }
  }
}
printwrap(";\n");
$num_edges == 2**$level - 1
  or die "oops, num_edges=$num_edges";

my $varname = $FindBin::Script;
$varname =~ s/\.pl//;
$varname =~ tr/-/_/;
print "% GP-DEFINE  \\\\ with +1 for 1-based vpar\n";
print "% GP-DEFINE  ${varname}_set1 = ",
      "Set([", join(',',map{$_+1}@solid), "]);\n";

print "  % end generated\n";
exit 0;
