#!/usr/bin/perl -w

# Copyright 2015, 2016, 2017, 2018, 2019 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.

use 5.004;
use strict;
use Math::PlanePath::DragonCurve 117; # v.117 for level_to_n_range()

use FindBin;
use lib $FindBin::Bin;
use MyDragon 'hull_expand','xy_rotate_minus90','transform_to_horizontal';
use MyLineSegments;
use MyPrintwrap ':all';

use Math::PlanePath::GcdRationals;
BEGIN { *gcd = \&Math::PlanePath::GcdRationals::_gcd; }


my $level = 8;

print "  % generated by tools/$FindBin::Script\n";
print "  \\newcommand{\\MyLevel}{$level}\n";

sub transform {
  my ($x,$y) = @_;
  return transform_to_horizontal($x,$y, $level);
}

my $ex = 1;
my $ey = 0;
my $points = [ [0,0],
               [$ex,$ey],
             ];
foreach (1 .. $level) {
  ($points,$ex,$ey) = hull_expand($points,$ex,$ey);
}
{
  my ($x,$y) = transform(0,0);
  print "  \\coordinate (start) at ($x,$y);\n";
}
{
  my ($x,$y) = transform($ex,$ey);
  print "  \\coordinate (end) at ($x,$y);\n";
}

my $path = Math::PlanePath::DragonCurve->new;
my ($n_lo, $n_hi) = $path->level_to_n_range($level);

sub is_visited {
  my ($x,$y) = @_;
  my $n = $path->xy_to_n($x,$y);
  return (defined $n && $n <= $n_hi);
}

{
  my $segments = MyLineSegments->new;
  foreach my $n ($n_lo .. $n_hi-1) {
    $segments->add($path->n_to_xy($n),
                   $path->n_to_xy($n + 1));
  }
  $Printwrap = 2;
  printwrap("\\draw[my dragon line] ");
  $Printwrap += 2;
  $segments->printwrap (transform => \&transform);
  printwrap(";\n");
}

{
  $Printwrap = 2;
  printwrap("\\draw[my hull line]\n");
  $Printwrap = 4;
  my $join = '';
  foreach my $p (@$points) {
    my ($x,$y) = @$p;
    ($x,$y) = transform($x,$y);
    printwrap("$join($x,$y)");
    $join = '--';
  }
  printwrap("${join}cycle;\n");
}

{
  my @boundary_points;

  my $HBP = 0;
  my $HBV = 0;
  my $HBVgaps = 0;
  my $prev_visited = 1;
  $Printwrap = 2;
  foreach my $i (0 .. $#$points) {
    my $p1 = $points->[$i];
    my $p2 = $points->[($i+1) % scalar(@$points)];
    my ($x,$y) = @$p1;
    my ($x2,$y2) = @$p2;

    my $P = $i+1;
    my ($tx,$ty) = transform($x,$y);
    printwrap(" \\node (P$P) at ($tx,$ty) [my dot];");

    my $dx = $x2 - $x;
    my $dy = $y2 - $y;
    my $g = gcd($dx,$dy);
    $dx /= $g;
    $dy /= $g;

    is_visited($x,$y) || die;
    if (abs($dx) > 1 || abs($dy) > 1) {
      # gap when multi-point step
      $HBVgaps++;
    }
    $prev_visited = 1;

    # hull vertex P
    $HBP++;
    $HBV++;
    $x += $dx;
    $y += $dy;

    while ($x != $x2 || $y != $y2) {
      my $visited = is_visited($x,$y);
      if ($prev_visited && ! $visited) {
        $HBVgaps++;
      }
      if (is_visited($x,$y)) {
        my ($tx,$ty) = transform($x,$y);
        push @boundary_points, "$tx,$ty";
        $HBV++;
      }
      $HBP++;
      $x += $dx;
      $y += $dy;
      $prev_visited = $visited;
    }
  }
  printwrap_newline();

  $Printwrap = 2;
  printwrap("\\path foreach \\p in {");
  $Printwrap = 4;
  foreach my $p (@boundary_points) {
    printwrap("($p),");
  }
  printwrap("}");
  printwrap(" { \\p node[my dot] };\n");
  print "  \\newcommand{\\MyHBV}{$HBV}\n";
  print "  \\newcommand{\\MyHBVgaps}{$HBVgaps}\n";
  print "% GP-Test  HBP($level)==$HBP\n";
  print "% GP-Test-Last  HBV($level)==$HBV\n";
  print "% GP-Test-Last  HBVgaps($level)==$HBVgaps\n";
}

print "  % end generated\n";
exit 0;
