#!/usr/bin/perl -w

# Copyright 2014, 2015, 2017, 2019 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use Math::PlanePath::DragonCurve 117; # v.117 for level_to_n_range()

use FindBin;
use lib $FindBin::Bin;
use MyDragon ':all';
use MyPrintwrap ':all';

my $type = '4-2';
if (@ARGV) { $type = shift @ARGV; }

print "  % generated by tools/$FindBin::Script $type\n";
my $path = Math::PlanePath::DragonCurve->new;

my %type_to_elem = ('4-2' => [4,2, 16,32],
                    '5-1' => [5,1, 7,21],
                    '5-2' => [5,2, 32,64],
                    '6-1' => [6,1, 14,39],
                   );
my $elem = $type_to_elem{$type}
  || die "unrecognised type $type";

my ($level, $multiply, $len_1,$len_2) = @$elem;

$Printwrap = 2;
printwrap("\\begin{scope}[shift=(pos-$level-$multiply)]\n");
$Printwrap += 2;
my ($n_lo, $n_hi) = $path->level_to_n_range($level);

my $n_length = $n_hi;
$n_lo = ($multiply-1)*$n_length;
my $n_join = ($multiply)*$n_length;
$n_hi = ($multiply+1)*$n_length;
my ($x_join,$y_join) = $path->n_to_xy($n_join);

my $is_touch = sub {
  my ($x,$y) = @_;
  my @n_list = $path->xy_to_n_list($x,$y);
  @n_list = grep {$_ >= $n_lo && $_ < $n_hi} @n_list;
  return @n_list == 2 && $n_list[0] < $n_join && $n_list[1] >= $n_join;
};

my $rotate = sub {
  my ($x,$y) = @_;
  foreach (1 .. 2) {
    ($x,$y) = xy_rotate_minus90($x,$y);
  }
  return ($x,$y);
};

my $transform = sub {
  my ($x,$y) = @_;
  $x -= $x_join;
  $y -= $y_join;
  return $rotate->($x,$y);
};


my $dot_name = "join-$level-$multiply";
my $arrow_1 = ($multiply & 1 ? '->' : '<-');
my $arrow_2 = ($multiply & 1 ? '<-' : '->');
printwrap("\\node ($dot_name) at (0,0) [my dot];\n");
printwrap("\\draw[$arrow_1] ");
$Printwrap += 2;
my $join = '';

my $n_start = (defined $len_1 ? $n_join-$len_1 : $n_lo);
my $n_end   = (defined $len_2 ? $n_join+$len_2 : $n_hi);
foreach my $n ($n_start .. $n_end) {
  my ($x,$y) = $path->n_to_xy($n);
  if ($n == $n_join) {
    printwrap("--($dot_name);\n");
    $Printwrap -= 2;
    printwrap("\\draw[$arrow_2] ($dot_name)");
    $Printwrap += 2;
    next;
  }
  if ($n == $n_join + $n_length) {
    ($x,$y) = $transform->($x,$y);
    printwrap("--($x,$y);\n");
    $Printwrap -= 2;
    printwrap("\\draw[$arrow_1] ($x,$y)");
    $Printwrap += 2;
    next;
  }
  my $t = $is_touch->($x,$y);
  ($x,$y) = $transform->($x,$y);
  if ($t) {
    my ($sdx,$sdy) = $path->n_to_dxdy($n-1);
    my ($tdx,$tdy) = $path->n_to_dxdy($n);
    ($sdx,$sdy) = $rotate->($sdx,$sdy);
    ($tdx,$tdy) = $rotate->($tdx,$tdy);
    my $sx = $x . chamfer(-$sdx);
    my $sy = $y . chamfer(-$sdy);
    my $tx = $x . chamfer($tdx);
    my $ty = $y . chamfer($tdy);
    printwrap("$join($sx,$sy)");
    printwrap("--($tx,$ty)");
  } else {
    printwrap("$join($x,$y)");
  }
  $join = '--';
}
printwrap(";\n");
$Printwrap = 2;
printwrap("\\end{scope}\n");

print "  % end generated\n";
exit 0;
