#!/usr/bin/perl -w

# Copyright 2014, 2015, 2016, 2017, 2019 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.

use 5.004;
use strict;
use Lingua::EN::Numbers;
use Math::Geometry::Planar;
use Math::PlanePath::DragonCurve 117; # v.117 for level_to_n_range()

use FindBin;
use lib $FindBin::Bin;
use MyLineSegments;
use MyPrintwrap ':all';


print "  % generated by tools/$FindBin::Script\n";
my $tests = '';
my $commands = '';

my $pos = 1;
foreach my $level (7,8) {
  $Printwrap = 2;
  printwrap("\\begin{scope}[shift=(pos-$pos)]\n");

  my $path = Math::PlanePath::DragonCurve->new;
  my ($n_lo, $n_hi) = $path->level_to_n_range($level);
  my $segments = MyLineSegments->new;
  my $points = [ [0,0] ];
  foreach my $n ($n_lo .. $n_hi-1) {
    my ($x1,$y1) = $path->n_to_xy($n);
    my ($x2,$y2) = $path->n_to_xy($n+1);
    $segments->add($x1,$y1, $x2,$y2);
    push @$points, [$x2,$y2];
  }

  my $planar = Math::Geometry::Planar->new;
  $planar->points($points);
  if (@$points >= 5) {
    $planar = $planar->convexhull2;
  }
  $planar = $planar->minrectangle;
  my $minpoints = $planar->points;

  foreach my $i (0 .. $#$minpoints) {
    my $p1 = $minpoints->[$i-1];
    my $p2 = $minpoints->[$i];
    if ($p1->[0] == $p2->[0] || $p1->[1] == $p2->[1]) {
      # horizontal or vertical, delete dragon segments overlapped
      $segments->remove(@$p1, @$p2);
    }
  }

  {
    $Printwrap = 4;
    printwrap("\\draw[my dragon] ");
    $Printwrap += 2;
    $segments->printwrap;
    printwrap(";");
    printwrap(" % joined line points " . $segments->lines_joined_count_points
              . "\n");
  }

  {
    $Printwrap = 4;
    printwrap("\\draw ");
    $Printwrap = 6;
    foreach my $p (@$minpoints) {
      printwrap("($p->[0],$p->[1])--");
    }
    printwrap("cycle;\n");
  }

  {
    $Printwrap = 4;
    my ($x,$y) = $path->n_to_xy($n_hi);
    printwrap("\\coordinate (end-$pos) at ($x,$y);\n");
  }

  $Printwrap = 2;
  printwrap("\\end{scope}\n");

  {
    my $area = $planar->area;
    if ($area != int($area)) {
      $area = (2*$area).'/2';
    }
    $tests .= "% GP-Test-Last  MR($level)==$area\n";
  }

  my $name = Lingua::EN::Numbers::num2en($pos);
  $commands .= "  \\newcommand{\\MyLevel\u$name}{$level}\n";
  {
    my $area = $planar->area;
    if ($area != int($area)) {
      $area = "\\dfrac{".(2*$area)."}{2}";
    }
    $commands .= "  \\newcommand{\\MyMR$name}{$area}\n";
  }

  $pos++;
}

print $commands;
print $tests;
print "  % end generated\n";
exit 0;
