#!/usr/bin/perl -w

# Copyright 2016, 2017, 2018, 2019 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use Math::PlanePath::DragonCurve 117; # v.117 for level_to_n_range()

use FindBin;
use lib $FindBin::Bin;
use MyDragon ':all';
use MyLineSegments;
use MyPrintwrap ':all';

# uncomment this to run the ### lines
# use Smart::Comments;


my $level = 3;
print "  % generated by tools/$FindBin::Script\n";
print "  \\newcommand{\\MyLevel}{$level}\n";

my $path = Math::PlanePath::DragonCurve->new;
my ($n_lo, $n_hi) = $path->level_to_n_range($level+1);
my ($x_hi,$y_hi) = $path->n_to_xy($n_hi);
print "  \\coordinate (end) at ($x_hi,$y_hi);\n";

#    I----------------> b
#    |                  |
#    |                  |
#    |                  |
#    |                 --- 4
#    |                  |
#    |           3,1    |
#    v                  v
#    0 -----------|---> 1
#                 6
#

my @fracs = ([-1, -2],  # 1
             [1, -3],   # 2
             [2, -1],   # 3
             [4, -2],   # 4
             [2, -1],   # 5
             [3, 1]);   # 6
foreach my $i (0 .. $#fracs) {
  my $num = $i+1;
  my ($fs,$fr) = @{$fracs[$i]};
  print "% GP-Test  dragon_fpoint($num/7) == $fs/5 + I*$fr/5\n";
}

my $segments = MyLineSegments->new;
my %points;
foreach my $from_end (0, 1) {   # the two component k+1 dragons
  foreach my $n ($n_lo .. $n_hi-1) {
    my ($x,$y) = $path->n_to_xy($n);
    my ($dx,$dy) = $path->n_to_dxdy($n);
    if ($from_end) {
      ($x,$y) = ($x_hi-$x, $y_hi-$y);
      $dx=-$dx; $dy=-$dy;
    }
    my $x2 = $x + $dx;
    my $y2 = $y + $dy;

    ### segment: "$x,$y to $x2,$y2"
    $segments->add($x,$y, $x2,$y2);

    if ($n % 2) {
      ($x,$y) = ($x2,$y2);
      $dx=-$dx; $dy=-$dy;
      ### odd segment reverse: "$x,$y dxdy $dx,$dy"
    }

    my ($lx,$ly) = xy_rotate_plus90($dx,$dy);
    foreach my $elem (@fracs) {
      my ($fs,$fr) = @$elem;
      my $fx = 5*$x + $fs*$dx + $fr*$lx;
      my $fy = 5*$y + $fs*$dy + $fr*$ly;
      $points{"$fx,$fy"}++;
      ### fxy: "n=$n  $fx,$fy"
    }
  }
}

$Printwrap = 2;
printwrap("\\draw[my grey] ");
$Printwrap += 2;
$segments->printwrap;
printwrap(";\n");

### %points

$Printwrap = 2;
printwrap("\\begin{scope}[scale=1/5]\n");
$Printwrap += 2;
printwrap("\\path foreach \\p in {");
$Printwrap += 2;
my $count = 0;
foreach my $xy (sort keys %points) {
  next unless $points{$xy}==3;
  $count++;
  printwrap("($xy),");
}
$Printwrap -= 2;
printwrap("}");
printwrap(" { \\p node[my dot] };\n");
$Printwrap -= 2;
printwrap("\\end{scope}\n");

print "  \\newcommand{\\MyCount}{$count}\n";
print "% GP-Test  TSeventhTriples($level) == $count\n";
print "  % end generated\n";
exit 0;
