#!/usr/bin/perl -w

# Copyright 2014, 2015, 2016, 2017, 2019 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use Math::PlanePath::DragonCurve 117; # v.117 for level_to_n_range()

use FindBin;
use lib $FindBin::Bin;
use MyLineSegments;
use MyDragon 'transform_to_horizontal';
use MyPrintwrap ':all';


my $level = 7;     # twindragon k level number
my $horiz = 0;     # whether to transform to end horizontal
foreach my $arg (@ARGV) {
  if ($arg eq 'horiz') {
    $horiz = 1;
  } else {
    $level = $ARGV[0];
  }
}

print "  % generated by tools/$FindBin::Script $level",
  ($horiz ? ' horiz' : ''), "\n";

my $path = Math::PlanePath::DragonCurve->new;

sub transform {
  my ($x,$y) = @_;
  if ($horiz) {
    return transform_to_horizontal($x,$y, $level+1);
  } else {
    return ($x,$y);
  }
}

my ($n_lo, $n_hi) = $path->level_to_n_range($level+1);
my ($x_hi,$y_hi) = $path->n_to_xy($n_hi);

$Printwrap = 2;
printwrap("\\node (start) at (0,0) [my dot];\n");
{
  my ($x,$y) = transform($x_hi,$y_hi);
  printwrap("\\node (end) at ($x,$y) [my dot];\n");
}

my $segments = MyLineSegments->new;
foreach my $n ($n_lo .. $n_hi-1) {
  my ($x1,$y1) = $path->n_to_xy($n);
  my ($x2,$y2) = $path->n_to_xy($n+1);
  $segments->add($x1,$y1, $x2,$y2);
  $segments->add($x_hi - $x1, $y_hi - $y1,
                 $x_hi - $x2, $y_hi - $y2);
}

$Printwrap = 2;
printwrap("\\draw ");
$Printwrap += 2;
$segments->printwrap (transform => \&transform);
printwrap(";\n");

print "  % end generated\n";
exit 0;
