#!/usr/bin/perl -w

# Copyright 2014, 2015, 2016, 2017 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use Math::PlanePath::DragonCurve 117; # v.117 for level_to_n_range()

use FindBin;
use lib $FindBin::Bin;
use MyDragon ':all';

# uncomment this to run the ### lines
# use Smart::Comments;


my $max_level = 8;
if (@ARGV) { $max_level = shift @ARGV; }

print "% generated by tools/$FindBin::Script\n";

my ($n_lo, $n_hi);
my @count;
foreach my $arms (2 .. 4) {
  my $path = Math::PlanePath::DragonCurve->new (arms => $arms);

  foreach my $k (0 .. $max_level) {
    my ($n_lo, $n_hi) = $path->level_to_n_range($k);
    ### $arms
    ### $k
    ### $n_lo
    ### $n_hi

    my $path_xyxy_is_traversed_either = sub {
      my ($x1,$y1, $x2,$y2) = @_;
      my $n = $path->xyxy_to_n_either($x1,$y1, $x2,$y2);
      my $ret = (defined $n && $n >= $n_lo && $n <= $n_hi-$arms);
      ### either: "$x1,$y1 to $x2,$y2 is ".($n//'undef')."  ret=".($ret||0)
      return $ret;
    };
    my $path_n_is_right_boundary = sub {
      my ($n) = @_;
      my ($x1,$y1) = $path->n_to_xy($n);
      my ($x2,$y2) = $path->n_to_xy($n+$arms);
      my ($dx,$dy) = $path->n_to_dxdy($n);
      ($dx,$dy) = xy_rotate_minus90($dx,$dy);  # to right side
      my $x1s = $x1 + $dx;
      my $y1s = $y1 + $dy;
      my $x2s = $x2 + $dx;
      my $y2s = $y2 + $dy;
      return (! $path_xyxy_is_traversed_either->($x1,$y1, $x1s,$y1s)
              || ! $path_xyxy_is_traversed_either->($x2,$y2, $x2s,$y2s)
              || ! $path_xyxy_is_traversed_either->($x1s,$y1s, $x2s,$y2s));
    };
    my $path_n_is_left_boundary = sub {
      my ($n) = @_;
      my ($x1,$y1) = $path->n_to_xy($n);
      my ($x2,$y2) = $path->n_to_xy($n+$arms);
      my ($dx,$dy) = $path->n_to_dxdy($n);
      ($dx,$dy) = xy_rotate_plus90($dx,$dy);  # to side
      my $x1s = $x1 + $dx;
      my $y1s = $y1 + $dy;
      my $x2s = $x2 + $dx;
      my $y2s = $y2 + $dy;
      ### left consider: "n=$n  $x1,$y1 to $x2,$y2   and $x1s,$y1s to $x2s,$y2s  within $n_lo to n_hi=$n_hi"
      my $left = (! $path_xyxy_is_traversed_either->($x1,$y1, $x1s,$y1s)
                  || ! $path_xyxy_is_traversed_either->($x2,$y2, $x2s,$y2s)
                  || ! $path_xyxy_is_traversed_either->($x1s,$y1s, $x2s,$y2s));
      ### $left
      return $left;
    };

    foreach my $part ('whole',
                      # 'blob'
                     ) {
      my ($n_start,$n_end);
      if ($part eq 'blob') {
        $n_start = BlobN($k);
        $n_end   = BlobN($k+1) - 3;
      } else {
        $n_start = $n_lo;
        $n_end   = $n_hi;
      }
      ### $n_start
      ### $n_end

      my $L = 0;
      my $R = 0;
      my $B = 0;
      foreach my $n ($n_start .. $n_end-$arms) {
        if ($path_n_is_left_boundary->($n)) {
          $L++;
          $B++;
        }
        if ($path_n_is_right_boundary->($n)) {
          $R++;
          $B++;
        }
      }
      $count[$k]->{$part} = $B;
    }
  }

  foreach my $part ('whole',
                    # 'blob',
                   ) {
    print('% GP-DEFINE  ',
          ($part eq 'whole' ? '' : 'Blob'),
          "Ba$arms",
          '_generated_samples = [');
    my $join = '';
    foreach my $k (0 .. $#count) {
      my $value;
      $value = $count[$k]->{$part} || 0;

      print "$join$value";
      $join = ',';
    }
    print "];\n";
  }
}

print "% end generated\n";
exit 0;
