#!/usr/bin/perl -w

# Copyright 2014, 2015, 2016 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.

use 5.004;
use strict;
use FindBin;
use Math::PlanePath::DragonCurve 117; # v.117 for level_to_n_range()

use FindBin;
BEGIN { unshift @INC, $FindBin::Bin }
use MyDragon 'BlobN','BlobSegments',
  'rect_corners_sort','transform_to_horizontal','chamfer';
use MyLineSegments;
use MyPrintwrap ':all';

# uncomment this to run the ### lines
# use Smart::Comments;


my $level = 9;

print "  % generated by tools/$FindBin::Script\n";
print "  \\newcommand{\\MyLevel}{$level}\n";

my $path = Math::PlanePath::DragonCurve->new;

my $n_lo = BlobN($level);
my $n_hi = BlobN($level+1) - 3;

sub transform {
  my ($x,$y) = @_;
  return transform_to_horizontal($x,$y, $level+1); # for blob horizontal
}

{
  my ($x,$y) = $path->n_to_xy($n_lo);
  ($x,$y) = transform($x,$y);
  print "  \\coordinate (start) at ($x,$y);\n";
}
{
  my ($x,$y) = $path->n_to_xy($n_hi);
  ($x,$y) = transform($x,$y);
  print "  \\coordinate (end) at ($x,$y);\n";
}

my $b4 = BlobSegments($level-4);
my $b2 = BlobSegments($level-2);

my @type;

my %type_to_fill = ('end-blob'    => undef, # 'my end blob',
                    'middle-blob' => 'my middle blob',
                    'twindragon'  => 'my twin',
                   );

foreach my $n ($n_lo .. $n_hi) {    # blob k-2 segments
  $type[$n] = 'middle-blob';
}

my $b4_first_lo = $n_lo;
my $b4_first_hi = $b4_first_lo + $b4;
foreach my $n ($b4_first_lo .. $b4_first_hi-1) {   # blob k-4 first segments
  $type[$n] = 'end-blob';
}

my $b2_lo = $b4_first_hi + 3;
foreach my $n ($b4_first_hi .. $b2_lo-1) {   # bridge segments
  $type[$n] = 'bridge';
}

my $b4_last_hi = $n_hi;
my $b4_last_lo = $b4_last_hi - $b4;
foreach my $n ($b4_last_lo .. $b4_last_hi-1) {
  $type[$n] = 'end-blob';
}
foreach my $n ($b4_last_lo-3 .. $b4_last_lo-1) {   # bridge segments
  $type[$n] = 'bridge';
}

{
  my ($x,$y) = $path->n_to_xy($b4_first_hi);
  ($x,$y) = transform($x,$y);
  print "  \\coordinate (blob-4a-end) at ($x,$y);\n";
}
{
  my ($x,$y) = $path->n_to_xy($b2_lo);
  ($x,$y) = transform($x,$y);
  print "  \\coordinate (blob-2-end) at ($x,$y);\n";
}
{
  my $b2_hi = $b4_last_lo - 3;
  my ($x,$y) = $path->n_to_xy($b2_hi);
  ($x,$y) = transform($x,$y);
  print "  \\coordinate (blob-2-start) at ($x,$y);\n";
}
{
  my ($x,$y) = $path->n_to_xy($b4_last_lo);
  ($x,$y) = transform($x,$y);
  print "  \\coordinate (blob-4b-start) at ($x,$y);\n";
}

my %twin_lo_offset = (4 => 0,
                      5 => 0,
                      6 => 0,
                      7 => 0,
                      8 => 2,
                      9 => 4,
                      10 => 11,
                      11 => 25,
                     );
my $twin_lo = $b2_lo + ($twin_lo_offset{$level} || 0);
my $twin_hi = $twin_lo + 2**($level-2);
foreach my $n ($twin_lo .. $twin_hi-1) {    # twindragon
  $type[$n] = 'twindragon';
}
foreach my $i (0 .. 3) {
  my ($x,$y) = $path->n_to_xy($twin_lo + $i*2**($level-4));
  ($x,$y) = transform($x,$y);
  print "  \\coordinate (twin-corner-$i) at ($x,$y);\n";
}
{
  my ($x1,$y1) = $path->n_to_xy($twin_lo);
  my ($x2,$y2) = $path->n_to_xy($twin_lo + 2**($level-3));
  my $x = ($x1+$x2)/2;
  my $y = ($y1+$y2)/2;
  print "% GP-Test-Last  BlobPartsTM($level) == $x + I*$y  /* untransformed */\n";
  ($x,$y) = transform($x,$y);
  print "  \\coordinate (twin-middle) at ($x,$y);\n";
}

my @want_chamfer_list = ($b4_first_hi - 1,
                         $b4_first_hi + 2,

                         $b4_last_lo - 21,
                         $b4_last_lo - 22,
                         $b4_last_lo - 23,

                         $b4_last_lo - 2,
                         $b4_last_lo + 1,
                         $b4_last_lo + 2,
                        );
# return enclosure type of square at X,Y to X+1,Y+1
sub square_enclose_type {
  my ($x,$y) = @_;
  ### square_enclose_type(): "$x, $y"
  my $type;
  foreach my $n ($path->xyxy_to_n_either($x,$y, $x+1,$y),
                 $path->xyxy_to_n_either($x,$y, $x,$y+1),
                 $path->xyxy_to_n_either($x+1,$y, $x+1,$y+1),
                 $path->xyxy_to_n_either($x,$y+1, $x+1,$y+1)) {
    my $this_type = $type[$n||0] || 0;
    ### $this_type
    $type //=  $this_type;
    unless ($this_type eq $type) {
      return 0;
    }
  }
  return $type;
}

my @corner_dx = (0, -1, 0, -1);
my @corner_dy = (0, 0,  -1,-1);

$Printwrap = 2;
printwrap("\\draw[my line]");
$Printwrap = 4;

my $n_draw_lo = $n_lo;
my $n_draw_hi = $n_hi;
my %squares;
my $join = '';
my $prev_is_bridge = 0;

my @is_bridge = map {($_||'') eq 'bridge'} @type;
sub change_bridge_draw {
  my ($is_bridge) = @_;
  $is_bridge ||= 0;
  return if $is_bridge == $prev_is_bridge;
  $prev_is_bridge = $is_bridge;
  my $style = ($is_bridge ? '[my bridge]' : '[my line]');
  printwrap " coordinate[name=P];\n";
  $Printwrap = 2;
  printwrap("\\draw$style (P)");
  $Printwrap = 4;
  $join = '--';
}

foreach my $n ($n_draw_lo .. $n_draw_hi) {
  ### $n
  my ($x,$y) = $path->n_to_xy($n);
  my @n_list = $path->xy_to_n_list($x,$y);
  @n_list = grep {$_ != $n} @n_list;
  my $n_other = $n_list[0] || 0;
  my $is_touch = ($n_other >= $n_draw_lo && $n_other <= $n_draw_hi);

  foreach my $i (0 .. $#corner_dx) {
    my $mx = $x + $corner_dx[$i];
    my $my = $y + $corner_dy[$i];
    if (my $type = square_enclose_type($mx,$my)) {
      if (my $fill = $type_to_fill{$type}) {
        my $ox = $mx+1; # opposite corner
        my $oy = $my+1;
        ($mx,$my) = transform($mx,$my);
        ($ox,$oy) = transform($ox,$oy);
        ($mx,$my, $ox,$oy) = rect_corners_sort($mx,$my, $ox,$oy);
        $squares{"$mx,$my"} = $fill; # lower left after transform
      }
    }
  }

  # grep {$_==$n || $_==$n_other} @want_chamfer_list

  my $is_bridge = $is_bridge[$n];

  ($x,$y) = transform($x,$y);
  if ($n != $n_draw_lo && $n != $n_draw_hi && $is_touch) {
    my ($sdx,$sdy) = $path->n_to_dxdy($n-1);
    my ($tdx,$tdy) = $path->n_to_dxdy($n);
    ($sdx,$sdy) = transform($sdx,$sdy);
    ($tdx,$tdy) = transform($tdx,$tdy);
    my $sx = $x . chamfer(-$sdx);
    my $sy = $y . chamfer(-$sdy);
    my $tx = $x . chamfer($tdx);
    my $ty = $y . chamfer($tdy);
    printwrap("$join($sx,$sy)");
    if ($prev_is_bridge && ! $is_bridge) { change_bridge_draw($is_bridge); }
    printwrap("$join($tx,$ty)");
  } else {
    printwrap("$join($x,$y)");
  }
  $join = '--';
  change_bridge_draw($is_bridge);
}
printwrap(";\n");

{
  my %fill_types = reverse %squares;
  foreach my $fill (sort keys %fill_types) {

    $Printwrap = 2;
    printwrap("\\fill[$fill] foreach \\p in {");
    $Printwrap = 4;
    foreach my $xy (sort keys %squares) {
      next unless $squares{$xy} eq $fill;
      printwrap("($xy),");
    }
    printwrap("}\n");
    printwrap("{ \\p +(\\MyMargin,\\MyMargin) rectangle +(1-\\MyMargin,1-\\MyMargin) };\n");
  }
}

print "  % end generated\n";
exit 0;
