#!/usr/bin/perl -w

# Copyright 2014, 2015, 2016, 2017 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.

use 5.004;
use strict;
use FindBin;
use Math::PlanePath::DragonCurve 117; # v.117 for level_to_n_range()

use FindBin;
BEGIN { unshift @INC, $FindBin::Bin }
use MyDragon 'hull_expand';
use MyLineSegments;
use MyPrintwrap ':all';


$Printwrap = 2;
printwrap("% generated by tools/$FindBin::Script\n");
my $path = Math::PlanePath::DragonCurve->new;

foreach my $level (0 .. 5) {
  $Printwrap = 2;
  printwrap("\\begin{scope}[shift=(pos-$level)]\n");

  my ($n_lo, $n_hi) = $path->level_to_n_range($level);
  my $segments = MyLineSegments->new;
  foreach my $n ($n_lo .. $n_hi-1) {
    $segments->add($path->n_to_xy($n),
                   $path->n_to_xy($n + 1));
  }

  my $ex = 1;
  my $ey = 0;
  my $points = [ [0,0],
                 [$ex,$ey],
               ];
  foreach (1 .. $level) {
    ($points,$ex,$ey) = hull_expand($points,$ex,$ey);
  }

  foreach my $i (0 .. $#$points) {
    my $p1 = $points->[$i-1];
    my $p2 = $points->[$i];
    if ($p1->[0] == $p2->[0] || $p1->[1] == $p2->[1]) {
      # horizontal or vertical, delete dragon segments overlapped
      $segments->remove(@$p1, @$p2);
    }
  }

  # dragon curve lines
  if ($segments->lines_joined) {
    $Printwrap = 4;
    printwrap("\\draw[my dragon] ");
    $Printwrap += 2;
    $segments->printwrap;
    printwrap(";\n");
  }

  # hull polygon
  {
    $Printwrap = 4;
    printwrap("\\draw[my hull] ");
    $Printwrap = 6;
    my $join = '';
    foreach my $p (@$points) {
      my ($x,$y) = @$p;
      printwrap("$join($x,$y)node[my dot]");
      $join = '--';
    }
    if (@$points > 2) {
      printwrap("${join}cycle");
    }
    printwrap(";\n");
  }

  my $area = MyPlanar::points_to_area($points);
  print "% GP-Test   HA($level) == ",2*$area,"/2\n";

  $Printwrap = 2;
  printwrap("\\end{scope}\n");
}

print "  % end generated\n";
exit 0;
