#!/usr/bin/perl -w

# Copyright 2014, 2015, 2016 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use FindBin;
use Math::PlanePath::DragonCurve 117; # v.117 for level_to_n_range()

use FindBin;
BEGIN { unshift @INC, $FindBin::Bin }
use MyDragon 'xy_to_dir4','@dir4_to_dx','@dir4_to_dy';
use MyLineSegments;
use MyPrintwrap ':all';

# uncomment this to run the ### lines
# use Smart::Comments;


my $level = 3;
my $n_target = 7;

print "  % generated by tools/$FindBin::Script\n";
my $path = Math::PlanePath::DragonCurve->new;
my ($n_lo, $n_hi) = $path->level_to_n_range($level);

my @nodes;
my %node;
$Printwrap = 2;
foreach my $n ($n_lo .. $n_hi) {
  my ($x,$y) = $path->n_to_xy($n);
  my $xy = "$x,$y";
  next if $node{$xy};
  my $name = ($n == $n_target ? "target" : "p$n");
  my $gray = ($n == $n_target ? "" : ",my gray");
  $node{$xy} = $name;
  push @nodes, " \\node ($name) at ($x,$y) [my dot$gray];";
}
printwrap_newline();

my $segments = MyLineSegments->new;
foreach my $n ($n_lo .. $n_hi-1) {
  $segments->add($path->n_to_xy($n),
                 $path->n_to_xy($n+1));
}

my %seen;
my @lines;
foreach my $sign (1, -1) {
  my $n = $n_target;
  my $s = $sign;
  my $pow = 1;
  while ($n) {
    ### at: "$n pow=$pow"
    if ($n & $pow) {
      my ($x2,$y2) = $path->n_to_xy($n);
      $n -= $s*$pow;
      $s *= -1;
      my ($x1,$y1) = $path->n_to_xy($n);

      my $name1 = $node{"$x1,$y1"};
      my $name2 = $node{"$x2,$y2"};

      my $str = "($name1)--($name2)";
      unless ($n == 0 || $seen{$str}++) {
        push @lines, " \\draw[my line] $str;";
      }
      if ($x1 == $x2 || $y1 == $y2) {
        $segments->remove($x1,$y1, $x2,$y2);
      }
    }
    $pow *= 2;
  }
}
printwrap_newline();

$Printwrap = 2;
printwrap("\\draw[my gray] ");
$Printwrap += 2;
$segments->printwrap;
printwrap(";\n");

$Printwrap = 2;
foreach my $node (@nodes) { printwrap($node); }
printwrap_newline();

$Printwrap = 2;
foreach my $line (@lines) { printwrap($line); }
printwrap_newline();

print "  % end generated\n";
exit 0;
