#!/usr/bin/perl -w

# Copyright 2014, 2015, 2016, 2019 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use FindBin;
use Math::PlanePath::DragonCurve 117; # v.117 for level_to_n_range()

use FindBin;
BEGIN { unshift @INC, $FindBin::Bin }
use MyLineSegments;
use MyDragon 'xy_rotate_minus90','transform_to_horizontal';
use MyPrintwrap ':all';

# uncomment this to run the ### lines
# use Smart::Comments;


my $level = 9;   # of twindragon
if (@ARGV) { $level = shift @ARGV; }

print "  % generated by tools/$FindBin::Script\n";

my $path = Math::PlanePath::DragonCurve->new;
my ($n_lo, $n_hi) = $path->level_to_n_range($level+1);
my ($x_hi,$y_hi) = $path->n_to_xy($n_hi);
my $x_mid = $x_hi / 2;
my $y_mid = $y_hi / 2;
my $scale = 2**(($level+1)/2);

sub transform {
  my ($x,$y) = @_;
  return transform_to_horizontal($x,$y, $level+1);
}

sub path_n_is_right_boundary {
  my ($path, $n, $n_limit) = @_;
  ### path_n_is_right_boundary(): $n
  my ($x1,$y1) = $path->n_to_xy($n);
  my ($x2,$y2) = $path->n_to_xy($n+1);
  my ($dx,$dy) = $path->n_to_dxdy($n);
  ($dx,$dy) = xy_rotate_minus90($dx,$dy);  # to right side
  my $x1s = $x1 + $dx;
  my $y1s = $y1 + $dy;
  my $x2s = $x2 + $dx;
  my $y2s = $y2 + $dy;
  return (! path_xyxy_is_traversed_either($path, $x1,$y1, $x1s,$y1s, $n_limit)
          || ! path_xyxy_is_traversed_either($path, $x2,$y2, $x2s,$y2s, $n_limit)
          || ! path_xyxy_is_traversed_either($path, $x1s,$y1s, $x2s,$y2s, $n_limit));
}
sub path_xyxy_is_traversed_either {
  my ($path, $x1,$y1, $x2,$y2, $n_limit) = @_;
  my $n = $path->xyxy_to_n_either($x1,$y1, $x2,$y2);
  return defined $n && $n < $n_limit;
}


my $x_near = 0;
my $y_near = 0;
my $Rnear = $x_mid**2 + $y_mid**2;

my $count_segments = 0;
my $segments = MyLineSegments->new;
foreach my $n ($n_lo .. $n_hi-1) {
  next unless path_n_is_right_boundary($path,$n, $n_hi);
  my ($x1,$y1) = $path->n_to_xy($n);
  my ($x2,$y2) = $path->n_to_xy($n+1);
  $segments->add($x1,$y1, $x2,$y2);
  my $norm = ($x1-$x_mid)**2 + ($y1-$y_mid)**2;
  if ($norm < $Rnear) {
    $Rnear = $norm;
    $x_near = $x1;
    $y_near = $y1;
  }

  $x1 = $x_hi - $x1;
  $y1 = $y_hi - $y1;
  $x2 = $x_hi - $x2;
  $y2 = $y_hi - $y2;
  $segments->add($x1,$y1, $x2,$y2);
  $count_segments += 2;
}

{
  $Printwrap = 2;
  printwrap("\\fill[my gray] ");
  $Printwrap += 2;
  $segments->printwrap (transform => \&transform);
  printwrap(";\n");
}
{
  my ($x,$y) = transform($x_mid,$y_mid);
  print "  \\coordinate (middle) at ($x,$y);\n";
}
{
  my ($x,$y) = transform($x_hi,$y_hi);
  print "  \\coordinate (end) at ($x,$y);\n";
}
{
  my ($x,$y) = transform($x_near,$y_near);
  print "  \\coordinate (Rnear) at ($x,$y);\n";
}
my $x_near_opposite = $x_hi - $x_near;
my $y_near_opposite = $y_hi - $y_near;
{
  my ($x,$y) = transform($x_near_opposite, $y_near_opposite);
  print "  \\coordinate (Rnear-opposite) at ($x,$y);\n";
}

my $draw_count = $segments->lines_joined_count_points;
print "  % total boundary segments $count_segments drawn with $draw_count points\n";
print "% GP-Test  TB($level) == $count_segments\n";

my $norm = ($x_near - $x_near_opposite)**2 + ($y_near - $y_near_opposite)**2;
print "% GP-Test-Last  TRnear($level)^2 == $norm\n";

print "  % end generated\n";
exit 0;
